@@ -1,12 +1,12 @@
 /* (from) $XConsortium: SmeBSB.c,v 1.16 91/03/15 15:59:41 gildea Exp $ */
 
 /*
- * Modifications Copyright 1995 by Paul Mattes.
- *   Permission to use, copy, modify, and distribute this software and its
- *   documentation for any purpose and without fee is hereby granted,
- *   provided that the above copyright notice appear in all copies and that
- *   both that copyright notice and this permission notice appear in
- *   supporting documentation.
+ * Modifications Copyright 1995, 1999, 2000 by Paul Mattes.
+ *  Permission to use, copy, modify, and distribute this software and its
+ *  documentation for any purpose and without fee is hereby granted,
+ *  provided that the above copyright notice appear in all copies and that
+ *  both that copyright notice and this permission notice appear in
+ *  supporting documentation.
  *
  * Copyright 1989 Massachusetts Institute of Technology
  *
@@ -39,6 +39,9 @@
  *          kit@expo.lcs.mit.edu
  */
 
+#include "globals.h"
+#if defined(X3270_MENUS) /*[*/
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xos.h>
@@ -85,19 +88,27 @@ static XtResource resources[] = {
  * Semi Public function definitions. 
  */
 
-static void Redisplay(), Destroy(), Initialize(), FlipColors();
-static void FlipOn(), FlipOff();
-static void PopupMenu();
-static void ClassInitialize();
-static Boolean SetValues();
-static XtGeometryResult QueryGeometry();
+static void FlipColors(Widget);
+static void Initialize(Widget, Widget);
+static void Destroy(Widget);
+static void Redisplay(Widget, XEvent *, Region);
+static void FlipOn(Widget);
+static void FlipOff(Widget);
+static void PopupMenu(Widget);
+static void ClassInitialize(void);
+static Boolean SetValues(Widget, Widget, Widget);
+static XtGeometryResult QueryGeometry(Widget, XtWidgetGeometry *,
+	XtWidgetGeometry *);
 
 /* 
  * Private Function Definitions.
  */
 
-static void GetDefaultSize(), DrawBitmaps(), GetBitmapInfo();
-static void CreateGCs(), DestroyGCs();
+static void GetDefaultSize(Widget, Dimension *, Dimension *);
+static void DrawBitmaps(Widget, GC);
+static void GetBitmapInfo(Widget, Boolean);
+static void CreateGCs(Widget);
+static void DestroyGCs(Widget);
     
 #define superclass (&cmeClassRec)
 CmeBSBClassRec cmeBSBClassRec = {
@@ -108,7 +119,7 @@ CmeBSBClassRec cmeBSBClassRec = {
     /* class_initializer  */	ClassInitialize,
     /* class_part_initialize*/	NULL,
     /* Class init'ed      */	FALSE,
-    /* initialize         */    Initialize,
+    /* initialize         */    (XtInitProc)Initialize,
     /* initialize_hook    */	NULL,
     /* realize            */    NULL,
     /* actions            */    NULL,
@@ -123,7 +134,7 @@ CmeBSBClassRec cmeBSBClassRec = {
     /* destroy            */    Destroy,
     /* resize             */    NULL,
     /* expose             */    Redisplay,
-    /* set_values         */    SetValues,
+    /* set_values         */    (XtSetValuesFunc)SetValues,
     /* set_values_hook    */	NULL,
     /* set_values_almost  */	XtInheritSetValuesAlmost,  
     /* get_values_hook    */	NULL,			
@@ -163,7 +174,7 @@ WidgetClass cmeBSBObjectClass = (WidgetClass) &cmeBSBClassRec;
  */
 
 static void 
-ClassInitialize()
+ClassInitialize(void)
 {
     XawInitializeWidgetSet();
     XtAddConverter( XtRString, XtRJustify, XmuCvtStringToJustify, NULL, 0 );
@@ -177,10 +188,8 @@ ClassInitialize()
  *      Returns: none.
  */
 
-/* ARGSUSED */
 static void
-Initialize(request, new)
-Widget request, new;
+Initialize(Widget request unused, Widget new)
 {
     CmeBSBObject entry = (CmeBSBObject) new;
 
@@ -208,8 +217,7 @@ Widget request, new;
  */
 
 static void
-Destroy(w)
-Widget w;
+Destroy(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
 
@@ -228,12 +236,8 @@ Widget w;
  *      Returns: none.
  */
 
-/* ARGSUSED */
 static void
-Redisplay(w, event, region)
-Widget w;
-XEvent * event;
-Region region;
+Redisplay(Widget w, XEvent *event unused, Region region unused)
 {
     GC gc;
     CmeBSBObject entry = (CmeBSBObject) w;
@@ -302,10 +306,8 @@ Region region;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static Boolean
-SetValues(current, request, new)
-Widget current, request, new;
+SetValues(Widget current, Widget request unused, Widget new)
 {
     CmeBSBObject entry = (CmeBSBObject) new;
     CmeBSBObject old_entry = (CmeBSBObject) current;
@@ -361,9 +363,8 @@ Widget current, request, new;
  */
 
 static XtGeometryResult
-QueryGeometry(w, intended, return_val) 
-Widget w;
-XtWidgetGeometry *intended, *return_val;
+QueryGeometry(Widget w, XtWidgetGeometry *intended,
+	XtWidgetGeometry *return_val)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
     Dimension width, height;
@@ -403,11 +404,8 @@ XtWidgetGeometry *intended, *return_val;
  *      Returns: none.
  */
 
-/*ARGSUSED*/
 static void
-OnCallback(closure, id)
-XtPointer closure;
-XtIntervalId *id;
+OnCallback(XtPointer closure, XtIntervalId *id unused)
 {
     Widget w = (Widget) closure;
     CmeBSBObject entry = (CmeBSBObject) w;
@@ -425,8 +423,7 @@ XtIntervalId *id;
  */
 
 static void 
-FlipOn(w)
-Widget w;
+FlipOn(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
 
@@ -447,11 +444,10 @@ Widget w;
  */
 
 static void 
-FlipOff(w)
-Widget w;
+FlipOff(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
-    Widget menu, temp;
+    Widget menu = NULL, temp;
 
     FlipColors(w);
     if (entry->cme_bsb.menu_name == NULL)
@@ -488,8 +484,7 @@ Widget w;
  */
 
 static void 
-FlipColors(w)
-Widget w;
+FlipColors(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
 
@@ -516,9 +511,7 @@ Widget w;
  */
 
 static void
-GetDefaultSize(w, width, height) 
-Widget w;
-Dimension * width, * height;
+GetDefaultSize(Widget w, Dimension *width, Dimension *height)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
 
@@ -545,9 +538,7 @@ Dimension * width, * height;
  */
 
 static void
-DrawBitmaps(w, gc)
-Widget w;
-GC gc;
+DrawBitmaps(Widget w, GC gc)
 {
     int x_loc, y_loc;
     CmeBSBObject entry = (CmeBSBObject) w;
@@ -601,9 +592,7 @@ GC gc;
  */
 
 static void
-GetBitmapInfo(w, is_left)
-Widget w;
-Boolean is_left;
+GetBitmapInfo(Widget w, Boolean is_left)
 {
     CmeBSBObject entry = (CmeBSBObject) w;    
     unsigned int depth, bw;
@@ -659,8 +648,7 @@ Boolean is_left;
  */
 
 static void
-CreateGCs(w)
-Widget w;
+CreateGCs(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;    
     XGCValues values;
@@ -700,8 +688,7 @@ Widget w;
  */
 
 static void
-DestroyGCs(w)
-Widget w;
+DestroyGCs(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;    
 
@@ -718,11 +705,10 @@ Widget w;
  */
 
 static void
-PopupMenu(w)
-Widget w;
+PopupMenu(Widget w)
 {
     CmeBSBObject entry = (CmeBSBObject) w;
-    Widget menu, temp;
+    Widget menu = NULL, temp;
     Arg arglist[3];
     Cardinal num_args;
     int menu_x, menu_y, menu_width, menu_height, button_width;
@@ -790,8 +776,11 @@ Widget w;
  * highlight and unhighlight.
  */
 
-void _XawCmeBSBApolloHack ()
+void
+_XawCmeBSBApolloHack(void)
 {
     FlipColors();
 }
 #endif /* apollo */
+
+#endif /*]*/
