@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000, 2001 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -13,6 +13,9 @@
  */
 
 #include "globals.h"
+
+#if defined(X3270_MENUS) /*[*/
+
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Command.h>
@@ -20,9 +23,13 @@
 #include <X11/Xaw/Label.h>
 #include "appres.h"
 #include "objects.h"
+#include "resources.h"
 
 #include "aboutc.h"
+#include "charsetc.h"
+#include "keymapc.h"
 #include "popupsc.h"
+#include "screenc.h"
 #include "telnetc.h"
 #include "utilc.h"
 
@@ -34,28 +41,20 @@ extern int ns_rrcvd;
 extern int ns_bsent;
 extern int ns_rsent;
 extern int linemode;
-extern char *build;
 extern Pixmap icon;
-extern struct trans_list *temp_keymaps;
 
 /* Called when OK is pressed on the about popup */
-/*ARGSUSED*/
 static void
-saw_about(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+saw_about(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	XtPopdown(about_shell);
 }
 
 /* Called when the about popup is popped down */
-/*ARGSUSED*/
 static void
-destroy_about(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+destroy_about(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	XtDestroyWidget(about_shell);
 	about_shell = NULL;
@@ -63,8 +62,7 @@ XtPointer call_data;
 
 /* Return a time difference in English */
 static char *
-hms(ts)
-time_t ts;
+hms(time_t ts)
 {
 	time_t t, td;
 	long hr, mn, sc;
@@ -154,14 +152,18 @@ time_t ts;
 
 /* Called when the "About x3270..." button is pressed */
 void
-popup_about()
+popup_about(void)
 {
 	Widget w = NULL, w_prev = NULL;
 	Widget v = NULL;
 	Widget left_anchor = NULL;
 	int vd = 4;
 	char fbuf[1024];
-	char *ftype;
+	const char *ftype;
+	const char *emode;
+#if defined(X3270_TN3270E) /*[*/
+	const char *eopts;
+#endif /*]*/
 	char *xbuf;
 
 	/* Create the popup */
@@ -177,7 +179,7 @@ popup_about()
 	/* Create a form in the popup */
 
 	about_form = XtVaCreateManagedWidget(
-	    "dialog", formWidgetClass, about_shell,
+	    ObjDialog, formWidgetClass, about_shell,
 	    NULL);
 
 	/* Pretty picture */
@@ -204,9 +206,10 @@ popup_about()
 	w = left_anchor;
 	left_anchor = NULL;
 
-	MAKE_SMALL("Modifications Copyright \251 1993, 1994, 1995, 1996 by Paul Mattes.\n\
+	MAKE_SMALL("Modifications Copyright \251 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2001 by Paul Mattes.\n\
 Original X11 Port Copyright \251 1990 by Jeff Sparkes.\n\
 File transfer code Copyright \251 1995 by Dick Altenbern.\n\
+Includes IAC IP patch by Carey Evans, 1998.\n\
  Permission to use, copy, modify, and distribute this software and its documentation\n\
  for any purpose and without fee is hereby granted, provided that the above copyright\n\
  notice appear in all copies and that both that copyright notice and this permission\n\
@@ -229,25 +232,39 @@ Copyright \251 1989 by Georgia Tech Research Corporation, Atlanta, GA 30332.\n\
 
 	MAKE_LABEL(get_message("terminalName"), 4);
 	MAKE_VALUE(termtype);
+	if (connected_lu != CN && connected_lu[0]) {
+		MAKE_LABEL2(get_message("luName"));
+		MAKE_VALUE(connected_lu);
+	}
 
 	MAKE_LABEL(get_message("emulatorFont"), 4);
-	MAKE_VALUE(efontname);
+	MAKE_VALUE(full_efontname);
 	if (*standard_font) {
-		if (*latin1_font)
-			ftype = get_message("fullFont");
-		else
-			ftype = get_message("asciiFont");
-	} else
+		ftype = get_message("xFont");
+	} else {
 		ftype = get_message("cgFont");
+	}
 	xbuf = xs_buffer("  %s", ftype);
 	MAKE_LABEL(xbuf, 0);
 	XtFree(xbuf);
 
-	if (appres.charset) {
-		MAKE_LABEL(get_message("characterSet"), 4);
-		MAKE_VALUE(appres.charset);
-	} else
-		MAKE_LABEL(get_message("defaultCharacterSet"), 4);
+	MAKE_LABEL(get_message("displayCharacterSet"), 4);
+	if (!efont_matches) {
+		xbuf = xs_buffer("ascii-7 (%s %s, %s %s)",
+		    get_message("require"), display_charset(),
+		    get_message("have"), efont_charset);
+		MAKE_VALUE(xbuf);
+		XtFree(xbuf);
+	} else {
+		MAKE_VALUE(efont_charset);
+	}
+
+	MAKE_LABEL(get_message("characterSet"), 4);
+	xbuf = xs_buffer("%s (base %u, code page %u)",
+	    get_charset_name(), (cgcsgid >> 16) & 0xffff,
+	    cgcsgid & 0xffff);
+	MAKE_VALUE(xbuf);
+	XtFree(xbuf);
 
 	if (trans_list != (struct trans_list *)NULL ||
 	    temp_keymaps != (struct trans_list *)NULL) {
@@ -293,23 +310,60 @@ Copyright \251 1989 by Georgia Tech Research Corporation, Atlanta, GA 30332.\n\
 
 	if (CONNECTED) {
 		MAKE_LABEL(get_message("connectedTo"), 4);
-		MAKE_VALUE(current_host);
-		(void) sprintf(fbuf, "  %s", get_message("port"));
-		MAKE_LABEL2(fbuf);
-		(void) sprintf(fbuf, "%d", current_port);
-		MAKE_VALUE(fbuf);
+#if defined(LOCAL_PROCESS) /*[*/
+		if (local_process && !strlen(current_host)) {
+			MAKE_VALUE("(shell)");
+		} else
+#endif /*]*/
+		{
+			MAKE_VALUE(current_host);
+		}
+#if defined(LOCAL_PROCESS) /*[*/
+		if (!local_process) {
+#endif /*]*/
+			(void) sprintf(fbuf, "  %s", get_message("port"));
+			MAKE_LABEL2(fbuf);
+			(void) sprintf(fbuf, "%d", current_port);
+			MAKE_VALUE(fbuf);
+#if defined(LOCAL_PROCESS) /*[*/
+		}
+#endif /*]*/
+		if (IN_E)
+			emode = "TN3270E ";
+		else
+			emode = "";
 		if (IN_ANSI) {
 			if (linemode)
 				ftype = get_message("lineMode");
 			else
 				ftype = get_message("charMode");
+			(void) sprintf(fbuf, "  %s%s, ", emode, ftype);
+		} else if (IN_SSCP) {
+			(void) sprintf(fbuf, "  %s%s, ", emode,
+			    get_message("sscpMode"));
+		} else if (IN_3270) {
+			(void) sprintf(fbuf, "  %s%s, ", emode,
+			    get_message("dsMode"));
 		} else
-			ftype = get_message("dsMode");
-		(void) sprintf(fbuf, "  %s, ", ftype);
+			(void) strcpy(fbuf, "  ");
 		(void) strcat(fbuf, hms(ns_time));
 
 		MAKE_LABEL(fbuf, 0);
 
+#if defined(X3270_TN3270E) /*[*/
+		eopts = tn3270e_current_opts();
+		if (eopts != CN) {
+			(void) sprintf(fbuf, "  %s",
+	 			get_message("tn3270eOpts"));
+			MAKE_LABEL(fbuf, 0);
+			MAKE_VALUE(eopts);
+		} else if (IN_E) {
+			(void) sprintf(fbuf, "  %s",
+				get_message("tn3270eNoOpts"));
+			MAKE_LABEL(fbuf, 0);
+		}
+#endif /*]*/
+
 		if (IN_3270)
 			(void) sprintf(fbuf, "%s %d %s, %d %s\n%s %d %s, %d %s",
 			    get_message("sent"),
@@ -332,6 +386,7 @@ Copyright \251 1989 by Georgia Tech Research Corporation, Atlanta, GA 30332.\n\
 				get_message("byte") : get_message("bytes"));
 		MAKE_LABEL(fbuf, 4);
 
+#if defined(X3270_ANSI) /*[*/
 		if (IN_ANSI) {
 			struct ctl_char *c = net_linemode_chars();
 			int i;
@@ -347,10 +402,10 @@ Copyright \251 1989 by Georgia Tech Research Corporation, Atlanta, GA 30332.\n\
 				MAKE_VALUE(c[i].value);
 			}
 		}
+#endif /*]*/
 	} else if (HALF_CONNECTED) {
 		MAKE_LABEL(get_message("connectionPending"), 4);
 		MAKE_VALUE(current_host);
-		MAKE_LABEL(hms(ns_time), 0);
 	} else
 		MAKE_LABEL(get_message("notConnected"), 4);
 
@@ -371,3 +426,5 @@ Copyright \251 1989 by Georgia Tech Research Corporation, Atlanta, GA 30332.\n\
 
 #undef MAKE_LABEL
 #undef MAKE_VALUE
+
+#endif /*]*/
