@@ -1,5 +1,5 @@
 /*
- * Copyright 1995 by Paul Mattes.
+ * Copyright 1995, 1999 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -12,4 +12,4 @@
  *		Global declarations for apl.c.
  */
 
-extern KeySym APLStringToKeysym();
+extern KeySym APLStringToKeysym(char *s, int *is_gep);
