@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -31,13 +31,15 @@
 #include "resources.h"
 
 #include "ctlrc.h"
-#include "ft_cutc.h"
 #include "ftc.h"
+#include "ft_cutc.h"
+#include "hostc.h"
 #include "kybdc.h"
 #include "macrosc.h"
 #include "popupsc.h"
 #include "screenc.h"
 #include "scrollc.h"
+#include "seec.h"
 #include "selectc.h"
 #include "sfc.h"
 #include "statusc.h"
@@ -52,6 +54,8 @@ extern unsigned char aid;
 /* Globals */
 int             ROWS, COLS;
 int             maxROWS, maxCOLS;
+int		ov_rows, ov_cols;
+int             model_num;
 int             cursor_addr, buffer_addr;
 Boolean         screen_alt = True;	/* alternate screen? */
 Boolean         is_altbuffer = False;
@@ -69,14 +73,18 @@ unsigned char   crm_attr[16];
 static unsigned char *ascreen_buf;	/* alternate 3270 display buffer */
 static struct ea *aea_buf;	/* alternate 3270 extended attribute buffer */
 static unsigned char *zero_buf;	/* empty buffer, for area clears */
-static void     set_formatted();
-static void     ctlr_blanks();
+static void set_formatted(void);
+static void ctlr_blanks(void);
 static unsigned char fake_fa;
 static struct ea fake_ea;
 static Boolean  trace_primed = False;
 static unsigned char default_fg;
 static unsigned char default_gr;
 static unsigned char default_cs;
+static void	ctlr_half_connect(Boolean ignored);
+static void	ctlr_connect(Boolean ignored);
+static int	sscp_start;
+static void ticking_stop(void);
 
 /*
  * code_table is used to translate buffer addresses and attributes to the 3270
@@ -124,36 +132,33 @@ static unsigned char	code_table[64] = {
  * Initialize the emulated 3270 hardware.
  */
 void
-ctlr_init(keep_contents, model_changed)
-Boolean keep_contents;
-Boolean model_changed;
+ctlr_init(unsigned cmask unused)
 {
-	/* Allocate buffers */
-
-	if (model_changed) {
-		if (screen_buf)
-			XtFree((char *)screen_buf);
-		screen_buf = (unsigned char *)XtCalloc(sizeof(unsigned char),
-		    maxROWS * maxCOLS);
-		if (ea_buf)
-			XtFree((char *)ea_buf);
-		ea_buf = (struct ea *)XtCalloc(sizeof(struct ea),
-		    maxROWS * maxCOLS);
-		if (ascreen_buf)
-			XtFree((char *)ascreen_buf);
-		ascreen_buf = (unsigned char *)XtCalloc(sizeof(unsigned char),
-		    maxROWS * maxCOLS);
-		if (aea_buf)
-			XtFree((char *)aea_buf);
-		aea_buf = (struct ea *)XtCalloc(sizeof(struct ea),
-		    maxROWS * maxCOLS);
-		if (zero_buf)
-			XtFree((char *)zero_buf);
-		zero_buf = (unsigned char *)XtCalloc(sizeof(unsigned char),
-		    maxROWS * maxCOLS);
-	}
-
-	if (!keep_contents) {
+	/* Register callback routines. */
+	register_schange(ST_HALF_CONNECT, ctlr_half_connect);
+	register_schange(ST_CONNECT, ctlr_connect);
+	register_schange(ST_3270_MODE, ctlr_connect);
+}
+/*
+ * Reinitialize the emulated 3270 hardware.
+ */
+void
+ctlr_reinit(unsigned cmask)
+{
+	if (cmask & MODEL_CHANGE) {
+		/* Allocate buffers */
+		Replace(screen_buf,
+		    (unsigned char *)Calloc(sizeof(unsigned char),
+					    maxROWS * maxCOLS));
+		Replace(ea_buf, (struct ea *)Calloc(sizeof(struct ea),
+						    maxROWS * maxCOLS));
+		Replace(ascreen_buf,
+		    (unsigned char *)Calloc(sizeof(unsigned char),
+					    maxROWS * maxCOLS));
+		Replace(aea_buf, (struct ea *)Calloc(sizeof(struct ea),
+						     maxROWS * maxCOLS));
+		Replace(zero_buf, (unsigned char *)Calloc(sizeof(struct ea),
+							  maxROWS * maxCOLS));
 		cursor_addr = 0;
 		buffer_addr = 0;
 	}
@@ -163,9 +168,7 @@ Boolean model_changed;
  * Deal with the relationships between model numbers and rows/cols.
  */
 void
-set_rows_cols(mn, ovc, ovr)
-int mn;
-int ovc, ovr;
+set_rows_cols(int mn, int ovc, int ovr)
 {
 	int defmod;
 
@@ -183,7 +186,7 @@ int ovc, ovr;
 	case 4:
 #if defined(RESTRICT_3279) /*[*/
 		if (appres.m3279) {
-			XtWarning("No 3279 Model 4, defaulting to Model 3");
+			popup_an_error("No 3279 Model 4\nDefaulting to model 3");
 			set_rows_cols("3", ovc, ovr);
 			return;
 		}
@@ -195,7 +198,7 @@ int ovc, ovr;
 	case 5:
 #if defined(RESTRICT_3279) /*[*/
 		if (appres.m3279) {
-			XtWarning("No 3279 Model 5, defaulting to Model 3");
+			popup_an_error("No 3279 Model 5\nDefaulting to model 3");
 			set_rows_cols(3, ovc, ovr);
 			return;
 		}
@@ -210,13 +213,8 @@ int ovc, ovr;
 #else /*][*/
 		defmod = 4;
 #endif
-		{
-			char mnb[2];
-
-			mnb[0] = defmod + '0';
-			mnb[1] = '\0';
-			xs_warning("Unknown model, defaulting to %s", mnb);
-		}
+		popup_an_error("Unknown model: %d\nDefaulting to %d", mn,
+		    defmod);
 		set_rows_cols(defmod, ovc, ovr);
 		return;
 	}
@@ -256,7 +254,7 @@ int ovc, ovr;
  * has at least one field somewhere on it.
  */
 static void
-set_formatted()
+set_formatted(void)
 {
 	register int	baddr;
 
@@ -271,16 +269,33 @@ set_formatted()
 	} while (baddr != 0);
 }
 
+/*
+ * Called when a host is half connected.
+ */
+static void
+ctlr_half_connect(Boolean ignored unused)
+{
+	ticking_start(True);
+}
+
+
 /*
  * Called when a host connects, disconnects, or changes ANSI/3270 modes.
  */
-void
-ctlr_connect()
+static void
+ctlr_connect(Boolean ignored unused)
 {
+	ticking_stop();
+	status_untiming();
+
 	if (ever_3270)
 		fake_fa = 0xE0;
 	else
 		fake_fa = 0xC4;
+	if (!IN_3270 || (IN_SSCP && (kybdlock & KL_OIA_TWAIT))) {
+		kybdlock_clr(KL_OIA_TWAIT, "ctlr_connect");
+		status_reset();
+	}
 
 	default_fg = 0x00;
 	default_gr = 0x00;
@@ -295,8 +310,7 @@ ctlr_connect()
  * rather than its value.
  */
 unsigned char *
-get_field_attribute(baddr)
-register int	baddr;
+get_field_attribute(register int baddr)
 {
 	int	sbaddr;
 
@@ -319,10 +333,7 @@ register int	baddr;
  * Returns True if an attribute is found, False if boundary hit.
  */
 Boolean
-get_bounded_field_attribute(baddr, bound, fa_out)
-register int	baddr;
-register int	bound;
-unsigned char	*fa_out;
+get_bounded_field_attribute(register int baddr, register int bound, unsigned char *fa_out)
 {
 	int	sbaddr;
 
@@ -355,8 +366,7 @@ unsigned char	*fa_out;
  * extended attribute structure.
  */
 struct ea *
-fa2ea(fa)
-unsigned char *fa;
+fa2ea(unsigned char *fa)
 {
 	if (fa == &fake_fa)
 		return &fake_ea;
@@ -370,8 +380,7 @@ unsigned char *fa;
  * can be found.
  */
 int
-next_unprotected(baddr0)
-int baddr0;
+next_unprotected(int baddr0)
 {
 	register int baddr, nbaddr;
 
@@ -392,13 +401,15 @@ int baddr0;
  * size.
  */
 void
-ctlr_erase(alt)
-Boolean alt;
+ctlr_erase(Boolean alt)
 {
 	kybd_inhibit(False);
 
 	ctlr_clear(True);
 
+	/* Let a script go. */
+	sms_host_output();
+
 	if (alt == screen_alt)
 		return;
 
@@ -428,13 +439,11 @@ Boolean alt;
 /*
  * Interpret an incoming 3270 command.
  */
-int
-process_ds(buf, buflen)
-unsigned char	*buf;
-int	buflen;
+enum pds
+process_ds(unsigned char *buf, int buflen)
 {
 	if (!buflen)
-		return 0;
+		return PDS_OKAY_NO_OUTPUT;
 
 	scroll_to_bottom();
 
@@ -445,66 +454,68 @@ int	buflen;
 	case SNA_CMD_EAU:
 		trace_ds("EraseAllUnprotected\n");
 		ctlr_erase_all_unprotected();
+		return PDS_OKAY_NO_OUTPUT;
 		break;
 	case CMD_EWA:	/* erase/write alternate */
 	case SNA_CMD_EWA:
 		trace_ds("EraseWriteAlternate");
 		ctlr_erase(True);
 		ctlr_write(buf, buflen, True);
+		return PDS_OKAY_NO_OUTPUT;
 		break;
 	case CMD_EW:	/* erase/write */
 	case SNA_CMD_EW:
 		trace_ds("EraseWrite");
 		ctlr_erase(False);
 		ctlr_write(buf, buflen, True);
+		return PDS_OKAY_NO_OUTPUT;
 		break;
 	case CMD_W:	/* write */
 	case SNA_CMD_W:
 		trace_ds("Write");
 		ctlr_write(buf, buflen, False);
+		return PDS_OKAY_NO_OUTPUT;
 		break;
 	case CMD_RB:	/* read buffer */
 	case SNA_CMD_RB:
 		trace_ds("ReadBuffer\n");
 		ctlr_read_buffer(aid);
+		return PDS_OKAY_OUTPUT;
 		break;
 	case CMD_RM:	/* read modifed */
 	case SNA_CMD_RM:
 		trace_ds("ReadModified\n");
 		ctlr_read_modified(aid, False);
+		return PDS_OKAY_OUTPUT;
 		break;
 	case CMD_RMA:	/* read modifed all */
 	case SNA_CMD_RMA:
 		trace_ds("ReadModifiedAll\n");
 		ctlr_read_modified(aid, True);
+		return PDS_OKAY_OUTPUT;
 		break;
 	case CMD_WSF:	/* write structured field */
 	case SNA_CMD_WSF:
 		trace_ds("WriteStructuredField");
-		write_structured_field(buf, buflen);
+		return write_structured_field(buf, buflen);
 		break;
 	case CMD_NOP:	/* no-op */
 		trace_ds("NoOp\n");
+		return PDS_OKAY_NO_OUTPUT;
 		break;
 	default:
 		/* unknown 3270 command */
 		popup_an_error("Unknown 3270 Data Stream command: 0x%X\n",
 		    buf[0]);
-		return -1;
+		return PDS_BAD_CMD;
 	}
-
-	return 0;
 }
 
 /*
  * Functions to insert SA attributes into the inbound data stream.
  */
 static void
-insert_sa1(attr, value, currentp, anyp)
-unsigned char attr;
-unsigned char value;
-unsigned char *currentp;
-Boolean *anyp;
+insert_sa1(unsigned char attr, unsigned char value, unsigned char *currentp, Boolean *anyp)
 {
 	if (value == *currentp)
 		return;
@@ -520,12 +531,7 @@ Boolean *anyp;
 }
 
 static void
-insert_sa(baddr, current_fgp, current_grp, current_csp, anyp)
-int baddr;
-unsigned char *current_fgp;
-unsigned char *current_grp;
-unsigned char *current_csp;
-Boolean *anyp;
+insert_sa(int baddr, unsigned char *current_fgp, unsigned char *current_grp, unsigned char *current_csp, Boolean *anyp)
 {
 	if (reply_mode != SF_SRM_CHAR)
 		return;
@@ -548,7 +554,6 @@ Boolean *anyp;
 			cs |= 0xf0;
 		insert_sa1(XA_CHARSET, cs, current_csp, anyp);
 	}
-
 }
 
 
@@ -557,9 +562,7 @@ Boolean *anyp;
  * host.
  */
 void
-ctlr_read_modified(aid_byte, all)
-unsigned char aid_byte;
-Boolean all;
+ctlr_read_modified(unsigned char aid_byte, Boolean all)
 {
 	register int	baddr, sbaddr;
 	Boolean		send_data = True;
@@ -568,6 +571,9 @@ Boolean all;
 	unsigned char	current_gr = 0x00;
 	unsigned char	current_cs = 0x00;
 
+	if (IN_SSCP && aid_byte != AID_ENTER)
+		return;
+
 	trace_ds("> ");
 	obptr = obuf;
 
@@ -586,7 +592,8 @@ Boolean all;
 	    case AID_PA2:
 	    case AID_PA3:
 	    case AID_CLEAR:
-		short_read = True;
+		if (!all)
+			short_read = True;
 		/* fall through... */
 
 	    case AID_SELECT:			/* No data on READ MODIFIED */
@@ -595,13 +602,17 @@ Boolean all;
 		/* fall through... */
 
 	    default:				/* ordinary AID */
-		space3270out(3);
-		*obptr++ = aid_byte;
-		trace_ds(see_aid(aid_byte));
-		if (short_read)
-		    goto rm_done;
-		ENCODE_BADDR(obptr, cursor_addr);
-		trace_ds(rcba(cursor_addr));
+		if (!IN_SSCP) {
+			space3270out(3);
+			*obptr++ = aid_byte;
+			trace_ds(see_aid(aid_byte));
+			if (short_read)
+			    goto rm_done;
+			ENCODE_BADDR(obptr, cursor_addr);
+			trace_ds(rcba(cursor_addr));
+		} else {
+			space3270out(1);	/* just in case */
+		}
 		break;
 	}
 
@@ -643,7 +654,7 @@ Boolean all;
 						*obptr++ = cg2ebc[screen_buf[baddr]];
 						if (!any)
 							trace_ds(" '");
-						trace_ds(see_ebc(cg2ebc[screen_buf[baddr]]));
+						trace_ds("%s", see_ebc(cg2ebc[screen_buf[baddr]]));
 						any = True;
 					}
 					INC_BA(baddr);
@@ -659,6 +670,14 @@ Boolean all;
 		} while (baddr != sbaddr);
 	} else {
 		Boolean	any = False;
+		int nbytes = 0;
+
+		/*
+		 * If we're in SSCP-LU mode, the starting point is where the
+		 * host left the cursor.
+		 */
+		if (IN_SSCP)
+			baddr = sscp_start;
 
 		do {
 			if (screen_buf[baddr]) {
@@ -681,8 +700,16 @@ Boolean all;
 					trace_ds("'");
 				trace_ds(see_ebc(cg2ebc[screen_buf[baddr]]));
 				any = True;
+				nbytes++;
 			}
 			INC_BA(baddr);
+
+			/*
+			 * If we're in SSCP-LU mode, end the return value at
+			 * 255 bytes, or where the screen wraps.
+			 */
+			if (IN_SSCP && (nbytes >= 255 || !baddr))
+				break;
 		} while (baddr != 0);
 		if (any)
 			trace_ds("'");
@@ -697,8 +724,7 @@ Boolean all;
  * Calculate the proper 3270 DS value for an internal field attribute.
  */
 static unsigned char
-calc_fa(fa)
-unsigned char fa;
+calc_fa(unsigned char fa)
 {
 	register unsigned char r = 0x00;
 
@@ -717,13 +743,12 @@ unsigned char fa;
  * host.
  */
 void
-ctlr_read_buffer(aid_byte)
-unsigned char aid_byte;
+ctlr_read_buffer(unsigned char aid_byte)
 {
 	register int	baddr;
 	unsigned char	fa;
 	Boolean		any = False;
-	int		attr_count;
+	int		attr_count = 0;
 	unsigned char	current_fg = 0x00;
 	unsigned char	current_gr = 0x00;
 	unsigned char	current_cs = 0x00;
@@ -810,7 +835,7 @@ unsigned char aid_byte;
 			} else {
 				if (!any)
 					trace_ds(" '");
-				trace_ds(see_ebc(cg2ebc[screen_buf[baddr]]));
+				trace_ds("%s", see_ebc(cg2ebc[screen_buf[baddr]]));
 				any = True;
 			}
 		}
@@ -823,11 +848,12 @@ unsigned char aid_byte;
 	net_output();
 }
 
+#if defined(X3270_TRACE) /*[*/
 /*
  * Construct a 3270 command to reproduce the current state of the display.
  */
 void
-ctlr_snap_buffer()
+ctlr_snap_buffer(void)
 {
 	register int	baddr = 0;
 	int		attr_count;
@@ -836,7 +862,6 @@ ctlr_snap_buffer()
 	unsigned char	current_cs = 0x00;
 	unsigned char   av;
 
-	obptr = obuf;
 	space3270out(2);
 	*obptr++ = screen_alt ? CMD_EWA : CMD_EW;
 	*obptr++ = code_table[0];
@@ -917,14 +942,13 @@ ctlr_snap_buffer()
  * Returns a Boolean indicating if one is necessary.
  */
 Boolean
-ctlr_snap_modes()
+ctlr_snap_modes(void)
 {
 	int i;
 
 	if (!IN_3270 || reply_mode == SF_SRM_FIELD)
 		return False;
 
-	obptr = obuf;
 	space3270out(6 + crm_nattr);
 	*obptr++ = CMD_WSF;
 	*obptr++ = 0x00;	/* implicit length */
@@ -937,13 +961,14 @@ ctlr_snap_modes()
 			*obptr++ = crm_attr[i];
 	return True;
 }
+#endif /*]*/
 
 
 /*
  * Process a 3270 Erase All Unprotected command.
  */
 void
-ctlr_erase_all_unprotected()
+ctlr_erase_all_unprotected(void)
 {
 	register int	baddr, sbaddr;
 	unsigned char	fa;
@@ -998,10 +1023,7 @@ ctlr_erase_all_unprotected()
  * Process a 3270 Write command.
  */
 void
-ctlr_write(buf, buflen, erase)
-unsigned char	buf[];
-int	buflen;
-Boolean erase;
+ctlr_write(unsigned char buf[], int buflen, Boolean erase)
 {
 	register unsigned char	*cp;
 	register int	baddr;
@@ -1017,7 +1039,7 @@ Boolean erase;
 	unsigned char	efa_fg;
 	unsigned char	efa_gr;
 	unsigned char	efa_cs;
-	char		*paren = "(";
+	const char	*paren = "(";
 	enum { NONE, ORDER, SBA, TEXT, NULLCH } previous = NONE;
 
 #define END_TEXT0	{ if (previous == TEXT) trace_ds("'"); }
@@ -1113,6 +1135,8 @@ Boolean erase;
 			trace_ds(rcba(buffer_addr));
 			if (buffer_addr >= COLS * ROWS) {
 				trace_ds(" [invalid address, write command terminated]\n");
+				/* Let a script go. */
+				sms_host_output();
 				return;
 			}
 			current_fa = get_field_attribute(buffer_addr);
@@ -1165,7 +1189,6 @@ Boolean erase;
 				}
 				if (baddr == 0)
 					last_zpt = True;
-
 			} else
 				last_zpt = False;
 			buffer_addr = baddr;
@@ -1186,11 +1209,13 @@ Boolean erase;
 			previous = ORDER;
 			if (*cp)
 				trace_ds("'");
-			trace_ds(see_ebc(*cp));
+			trace_ds("%s", see_ebc(*cp));
 			if (*cp)
 				trace_ds("'");
 			if (baddr >= COLS * ROWS) {
 				trace_ds(" [invalid address, write command terminated]\n");
+				/* Let a script go. */
+				sms_host_output();
 				return;
 			}
 			do {
@@ -1218,6 +1243,8 @@ Boolean erase;
 			previous = ORDER;
 			if (baddr >= COLS * ROWS) {
 				trace_ds(" [invalid address, write command terminated]\n");
+				/* Let a script go. */
+				sms_host_output();
 				return;
 			}
 			do {
@@ -1238,7 +1265,7 @@ Boolean erase;
 			previous = ORDER;
 			if (*cp)
 				trace_ds("'");
-			trace_ds(see_ebc(*cp));
+			trace_ds("%s", see_ebc(*cp));
 			if (*cp)
 				trace_ds("'");
 			ctlr_add(buffer_addr, ebc2cg0[*cp], CS_GE);
@@ -1257,52 +1284,48 @@ Boolean erase;
 			cp++;
 			na = *cp;
 			if (IS_FA(screen_buf[buffer_addr])) {
-				if (na == 0) {
-					INC_BA(buffer_addr);
-				} else {
-					for (i = 0; i < (int)na; i++) {
+				for (i = 0; i < (int)na; i++) {
+					cp++;
+					if (*cp == XA_3270) {
+						trace_ds(" 3270");
 						cp++;
-						if (*cp == XA_3270) {
-							trace_ds(" 3270");
-							cp++;
-							new_attr = ATTR2FA(*cp);
-							ctlr_add(buffer_addr,
-							    new_attr,
-							    0);
-							trace_ds(see_attr(new_attr));
-						} else if (*cp == XA_FOREGROUND) {
-							trace_ds("%s",
-							    see_efa(*cp,
-								*(cp + 1)));
-							cp++;
-							if (appres.m3279)
-								ctlr_add_fg(buffer_addr, *cp);
-						} else if (*cp == XA_HIGHLIGHTING) {
-							trace_ds("%s",
-							    see_efa(*cp,
-								*(cp + 1)));
-							cp++;
-							ctlr_add_gr(buffer_addr, *cp & 0x07);
-						} else if (*cp == XA_CHARSET) {
-							int cs = 0;
+						new_attr = ATTR2FA(*cp);
+						ctlr_add(buffer_addr,
+						    new_attr,
+						    0);
+						trace_ds(see_attr(new_attr));
+					} else if (*cp == XA_FOREGROUND) {
+						trace_ds("%s",
+						    see_efa(*cp,
+							*(cp + 1)));
+						cp++;
+						if (appres.m3279)
+							ctlr_add_fg(buffer_addr, *cp);
+					} else if (*cp == XA_HIGHLIGHTING) {
+						trace_ds("%s",
+						    see_efa(*cp,
+							*(cp + 1)));
+						cp++;
+						ctlr_add_gr(buffer_addr, *cp & 0x07);
+					} else if (*cp == XA_CHARSET) {
+						int cs = 0;
 
-							trace_ds("%s",
-							    see_efa(*cp,
-								*(cp + 1)));
-							cp++;
-							if (*cp == 0xf1)
-								cs = 1;
-							ctlr_add(buffer_addr,
-							    screen_buf[buffer_addr], cs);
-						} else if (*cp == XA_ALL) {
-							trace_ds("%s",
-							    see_efa(*cp,
-								*(cp + 1)));
-							cp++;
-						} else {
-							trace_ds("%s[unsupported]", see_efa(*cp, *(cp + 1)));
-							cp++;
-						}
+						trace_ds("%s",
+						    see_efa(*cp,
+							*(cp + 1)));
+						cp++;
+						if (*cp == 0xf1)
+							cs = 1;
+						ctlr_add(buffer_addr,
+						    screen_buf[buffer_addr], cs);
+					} else if (*cp == XA_ALL) {
+						trace_ds("%s",
+						    see_efa(*cp,
+							*(cp + 1)));
+						cp++;
+					} else {
+						trace_ds("%s[unsupported]", see_efa(*cp, *(cp + 1)));
+						cp++;
 					}
 				}
 				INC_BA(buffer_addr);
@@ -1416,7 +1439,7 @@ Boolean erase;
 		default:	/* enter character */
 			if (*cp <= 0x3F) {
 				END_TEXT("ILLEGAL_ORDER");
-				trace_ds(see_ebc(*cp));
+				trace_ds("%s", see_ebc(*cp));
 				last_cmd = True;
 				last_zpt = False;
 				break;
@@ -1424,7 +1447,7 @@ Boolean erase;
 			if (previous != TEXT)
 				trace_ds(" '");
 			previous = TEXT;
-			trace_ds(see_ebc(*cp));
+			trace_ds("%s", see_ebc(*cp));
 			ctlr_add(buffer_addr, ebc2cg[*cp], default_cs);
 			ctlr_add_fg(buffer_addr, default_fg);
 			ctlr_add_gr(buffer_addr, default_gr);
@@ -1450,6 +1473,9 @@ Boolean erase;
 	trace_primed = False;
 
 	ps_process();
+
+	/* Let a script go. */
+	sms_host_output();
 }
 
 #undef START_FIELDx
@@ -1458,35 +1484,139 @@ Boolean erase;
 #undef END_TEXT0
 #undef END_TEXT
 
+/*
+ * Write SSCP-LU data, which is quite a bit dumber than regular 3270
+ * output.
+ */
+void
+ctlr_write_sscp_lu(unsigned char buf[], int buflen)
+{
+	int i;
+	unsigned char *cp = buf;
+	int s_row;
+	unsigned char c;
+	int baddr;
+	unsigned char fa;
+
+	/*
+	 * The 3174 Functionl Description says that anything but NL, NULL, FM,
+	 * or DUP is to be displayed as a graphic.  However, to deal with
+	 * badly-behaved hosts, we filter out SF, IC and SBA sequences, and
+	 * we display other control codes as spaces.
+	 */
+
+	trace_ds("SSCP-LU data\n");
+	for (i = 0; i < buflen; cp++, i++) {
+		switch (*cp) {
+		case FCORDER_NL:
+			/*
+			 * Insert NULLs to the end of the line and advance to
+			 * the beginning of the next line.
+			 */
+			s_row = buffer_addr / COLS;
+			while ((buffer_addr / COLS) == s_row) {
+				ctlr_add(buffer_addr, ebc2cg[0], default_cs);
+				ctlr_add_fg(buffer_addr, default_fg);
+				ctlr_add_gr(buffer_addr, default_gr);
+				INC_BA(buffer_addr);
+			}
+			break;
+
+		case ORDER_SF:	/* some hosts forget their talking SSCP-LU */
+			cp++;
+			i++;
+			fa = ATTR2FA(*cp);
+			trace_ds(" StartField%s %s [translated to space]\n",
+			    rcba(buffer_addr), see_attr(fa));
+			ctlr_add(buffer_addr, CG_space, default_cs);
+			ctlr_add_fg(buffer_addr, default_fg);
+			ctlr_add_gr(buffer_addr, default_gr);
+			INC_BA(buffer_addr);
+			break;
+		case ORDER_IC:
+			trace_ds(" InsertCursor%s [ignored]\n",
+			    rcba(buffer_addr));
+			break;
+		case ORDER_SBA:
+			baddr = DECODE_BADDR(*(cp+1), *(cp+2));
+			trace_ds(" SetBufferAddress%s [ignored]\n", rcba(baddr));
+			cp += 2;
+			i += 2;
+			break;
+
+		case ORDER_GE:
+			cp++;
+			if (++i >= buflen)
+				break;
+			if (*cp <= 0x40)
+				c = CG_space;
+			else
+				c = ebc2cg0[*cp];
+			ctlr_add(buffer_addr, c, CS_GE);
+			ctlr_add_fg(buffer_addr, default_fg);
+			ctlr_add_gr(buffer_addr, default_gr);
+			INC_BA(buffer_addr);
+			break;
+
+		default:
+			if (*cp == FCORDER_NULL)
+				c = CG_space;
+			else if (*cp == FCORDER_FM)
+				c = CG_asterisk;
+			else if (*cp == FCORDER_DUP)
+				c = CG_semicolon;
+			else if (*cp < 0x40) {
+				trace_ds(" X'%02x') [translated to space]\n",
+				    *cp);
+				c = CG_space; /* technically not necessary */
+			} else
+				c = ebc2cg[*cp];
+			ctlr_add(buffer_addr, c, default_cs);
+			ctlr_add_fg(buffer_addr, default_fg);
+			ctlr_add_gr(buffer_addr, default_gr);
+			INC_BA(buffer_addr);
+			break;
+		}
+	}
+	cursor_move(buffer_addr);
+	sscp_start = buffer_addr;
+
+	/* Unlock the keyboard. */
+	aid = AID_NO;
+	do_reset(False);
+
+	/* Let a script go. */
+	sms_host_output();
+}
 
 /*
  * Process pending input.
  */
 void
-ps_process()
+ps_process(void)
 {
-	/* Process typeahead. */
 	while (run_ta())
 		;
-
-	/* Process pending scripts and macros. */
 	sms_continue();
 
+#if defined(X3270_FT) /*[*/
 	/* Process file transfers. */
-	if (ft_state != FT_NONE &&	/* transfer in progress */
-	    formatted &&		/* screen is formatted */
-	    !screen_alt &&		/* 24x80 screen */
-	    !kybdlock &&		/* keyboard not locked */
-	    				/* magic field */
-	    IS_FA(screen_buf[1919]) && FA_IS_SKIP(screen_buf[1919]))
+	if (ft_state != FT_NONE &&      /* transfer in progress */
+	    formatted &&                /* screen is formatted */
+	    !screen_alt &&              /* 24x80 screen */
+	    !kybdlock &&                /* keyboard not locked */
+	    /* magic field */
+	    IS_FA(screen_buf[1919]) && FA_IS_SKIP(screen_buf[1919])) {
 		ft_cut_data();
+	}
+#endif /*]*/
 }
 
 /*
  * Tell me if there is any data on the screen.
  */
 Boolean
-ctlr_any_data()
+ctlr_any_data(void)
 {
 	register unsigned char *c = screen_buf;
 	register int i;
@@ -1505,18 +1635,19 @@ ctlr_any_data()
  * and buffer addresses and extended attributes.
  */
 void
-ctlr_clear(can_snap)
-Boolean can_snap;
+ctlr_clear(Boolean can_snap)
 {
-	extern Boolean trace_skipping;
-
 	/* Snap any data that is about to be lost into the trace file. */
 	if (ctlr_any_data()) {
+#if defined(X3270_TRACE) /*[*/
 		if (can_snap && !trace_skipping && toggled(SCREEN_TRACE))
 			trace_screen();
+#endif /*]*/
 		scroll_save(maxROWS, ever_3270 ? False : True);
 	}
+#if defined(X3270_TRACE) /*[*/
 	trace_skipping = False;
+#endif /*]*/
 
 	/* Clear the screen. */
 	(void) memset((char *)screen_buf, 0, ROWS*COLS);
@@ -1524,23 +1655,24 @@ Boolean can_snap;
 	ALL_CHANGED;
 	cursor_move(0);
 	buffer_addr = 0;
-	(void) unselect(0, ROWS*COLS);
+	unselect(0, ROWS*COLS);
 	formatted = False;
 	default_fg = 0;
 	default_gr = 0;
+	sscp_start = 0;
 }
 
 /*
  * Fill the screen buffer with blanks.
  */
 static void
-ctlr_blanks()
+ctlr_blanks(void)
 {
 	(void) memset((char *)screen_buf, CG_space, ROWS*COLS);
 	ALL_CHANGED;
 	cursor_move(0);
 	buffer_addr = 0;
-	(void) unselect(0, ROWS*COLS);
+	unselect(0, ROWS*COLS);
 	formatted = False;
 }
 
@@ -1549,22 +1681,21 @@ ctlr_blanks()
  * Change a character in the 3270 buffer.
  */
 void
-ctlr_add(baddr, c, cs)
-int	baddr;		/* buffer address */
-unsigned char	c;	/* character */
-unsigned char	cs;	/* character set */
+ctlr_add(int baddr, unsigned char c, unsigned char cs)
 {
 	unsigned char oc;
 
 	if ((oc = screen_buf[baddr]) != c || ea_buf[baddr].cs != cs) {
 		if (trace_primed && !IsBlank(oc)) {
+#if defined(X3270_TRACE) /*[*/
 			if (toggled(SCREEN_TRACE))
 				trace_screen();
+#endif /*]*/
 			scroll_save(maxROWS, False);
 			trace_primed = False;
 		}
 		if (SELECTED(baddr))
-			(void) unselect(baddr, 1);
+			unselect(baddr, 1);
 		ONE_CHANGED(baddr);
 		screen_buf[baddr] = c;
 		ea_buf[baddr].cs = cs;
@@ -1575,13 +1706,11 @@ unsigned char	cs;	/* character set */
  * Change the graphic rendition of a character in the 3270 buffer.
  */
 void
-ctlr_add_gr(baddr, gr)
-int	baddr;
-unsigned char	gr;
+ctlr_add_gr(int baddr, unsigned char gr)
 {
 	if (ea_buf[baddr].gr != gr) {
 		if (SELECTED(baddr))
-			(void) unselect(baddr, 1);
+			unselect(baddr, 1);
 		ONE_CHANGED(baddr);
 		ea_buf[baddr].gr = gr;
 		if (gr & GR_BLINK)
@@ -1593,9 +1722,7 @@ unsigned char	gr;
  * Change the foreground color for a character in the 3270 buffer.
  */
 void
-ctlr_add_fg(baddr, color)
-int	baddr;
-unsigned char	color;
+ctlr_add_fg(int baddr, unsigned char color)
 {
 	if (!appres.m3279)
 		return;
@@ -1603,19 +1730,18 @@ unsigned char	color;
 		color = 0;
 	if (ea_buf[baddr].fg != color) {
 		if (SELECTED(baddr))
-			(void) unselect(baddr, 1);
+			unselect(baddr, 1);
 		ONE_CHANGED(baddr);
 		ea_buf[baddr].fg = color;
 	}
 }
 
+#if defined(X3270_ANSI) /*[*/
 /*
  * Change the background color for a character in the 3270 buffer.
  */
 void
-ctlr_add_bg(baddr, color)
-int	baddr;
-unsigned char	color;
+ctlr_add_bg(int baddr, unsigned char color)
 {
 	if (!appres.m3279)
 		return;
@@ -1623,11 +1749,12 @@ unsigned char	color;
 		color = 0;
 	if (ea_buf[baddr].bg != color) {
 		if (SELECTED(baddr))
-			(void) unselect(baddr, 1);
+			unselect(baddr, 1);
 		ONE_CHANGED(baddr);
 		ea_buf[baddr].bg = color;
 	}
 }
+#endif /*]*/
 
 /*
  * Copy a block of characters in the 3270 buffer, optionally including all of
@@ -1635,18 +1762,13 @@ unsigned char	color;
  * extended attributes, is considered part of the characters here.)
  */
 void
-ctlr_bcopy(baddr_from, baddr_to, count, move_ea)
-int	baddr_from;
-int	baddr_to;
-int	count;
-int	move_ea;
+ctlr_bcopy(int baddr_from, int baddr_to, int count, int move_ea)
 {
 	/* Move the characters. */
 	if (memcmp((char *) &screen_buf[baddr_from],
 	           (char *) &screen_buf[baddr_to],
 		   count)) {
-		(void) MEMORY_MOVE((char *) &screen_buf[baddr_to],
-			           (char *) &screen_buf[baddr_from],
+		(void) memmove(&screen_buf[baddr_to], &screen_buf[baddr_from],
 			           count);
 		REGION_CHANGED(baddr_to, baddr_to + count);
 		/*
@@ -1656,7 +1778,7 @@ int	move_ea;
 		 * selected text moves.
 		 */
 		if (area_is_selected(baddr_to, count))
-			(void) unselect(baddr_to, count);
+			unselect(baddr_to, count);
 	}
 
 	/*
@@ -1688,35 +1810,32 @@ int	move_ea;
 			}
 		}
 		if (any && area_is_selected(baddr_to, count))
-			(void) unselect(baddr_to, count);
+			unselect(baddr_to, count);
 	}
 
 	/* Move extended attributes. */
 	if (move_ea && memcmp((char *) &ea_buf[baddr_from],
 			      (char *) &ea_buf[baddr_to],
 			      count*sizeof(struct ea))) {
-		(void) MEMORY_MOVE((char *) &ea_buf[baddr_to],
-		                   (char *) &ea_buf[baddr_from],
+		(void) memmove(&ea_buf[baddr_to], &ea_buf[baddr_from],
 			           count*sizeof(struct ea));
 		REGION_CHANGED(baddr_to, baddr_to + count);
 	}
 }
 
+#if defined(X3270_ANSI) /*[*/
 /*
  * Erase a region of the 3270 buffer, optionally clearing extended attributes
  * as well.
  */
 void
-ctlr_aclear(baddr, count, clear_ea)
-int	baddr;
-int	count;
-int	clear_ea;
+ctlr_aclear(int baddr, int count, int clear_ea)
 {
 	if (memcmp((char *) &screen_buf[baddr], (char *) zero_buf, count)) {
 		(void) memset((char *) &screen_buf[baddr], 0, count);
 		REGION_CHANGED(baddr, baddr + count);
 		if (area_is_selected(baddr, count))
-			(void) unselect(baddr, count);
+			unselect(baddr, count);
 	}
 	if (clear_ea && memcmp((char *) &ea_buf[baddr], (char *) zero_buf, count*sizeof(struct ea))) {
 		(void) memset((char *) &ea_buf[baddr], 0, count*sizeof(struct ea));
@@ -1731,13 +1850,13 @@ int	clear_ea;
  * operation is common enough to warrant a separate path.
  */
 void
-ctlr_scroll()
+ctlr_scroll(void)
 {
 	int qty = (ROWS - 1) * COLS;
 	Boolean obscured;
 
 	/* Make sure nothing is selected. (later this can be fixed) */
-	(void) unselect(0, ROWS*COLS);
+	unselect(0, ROWS*COLS);
 
 	/* Synchronize pending changes prior to this. */
 	obscured = screen_obscured();
@@ -1745,11 +1864,8 @@ ctlr_scroll()
 		screen_disp();
 
 	/* Move screen_buf and ea_buf. */
-	(void) MEMORY_MOVE((char *) &screen_buf[0],
-	    (char *) &screen_buf[COLS],
-	    qty);
-	(void) MEMORY_MOVE((char *) &ea_buf[0],
-	    (char *) &ea_buf[COLS],
+	(void) memmove(&screen_buf[0], &screen_buf[COLS], qty);
+	(void) memmove(&ea_buf[0], &ea_buf[COLS],
 	    qty * sizeof(struct ea));
 
 	/* Clear the last line. */
@@ -1759,27 +1875,27 @@ ctlr_scroll()
 	/* Update the screen. */
 	if (obscured) {
 		ALL_CHANGED;
-	} else
+	} else {
 		screen_scroll();
+	}
 }
+#endif /*]*/
 
 /*
  * Note that a particular region of the screen has changed.
  */
 void
-ctlr_changed(bstart, bend)
-int bstart;	/* first changed location */
-int bend;	/* last changed location, plus 1 */
+ctlr_changed(int bstart, int bend)
 {
 	REGION_CHANGED(bstart, bend);
 }
 
+#if defined(X3270_ANSI) /*[*/
 /*
  * Swap the regular and alternate screen buffers
  */
 void
-ctlr_altbuffer(alt)
-Boolean	alt;
+ctlr_altbuffer(Boolean alt)
 {
 	unsigned char *stmp;
 	struct ea *etmp;
@@ -1796,7 +1912,7 @@ Boolean	alt;
 
 		is_altbuffer = alt;
 		ALL_CHANGED;
-		(void) unselect(0, ROWS*COLS);
+		unselect(0, ROWS*COLS);
 
 		/*
 		 * There may be blinkers on the alternate screen; schedule one
@@ -1805,15 +1921,14 @@ Boolean	alt;
 		blink_start();
 	}
 }
-#undef SWAP
+#endif /*]*/
 
 
 /*
  * Set or clear the MDT on an attribute
  */
 void
-mdt_set(fa)
-unsigned char *fa;
+mdt_set(unsigned char *fa)
 {
 	if (*fa & FA_MODIFY)
 		return;
@@ -1823,8 +1938,7 @@ unsigned char *fa;
 }
 
 void
-mdt_clear(fa)
-unsigned char *fa;
+mdt_clear(unsigned char *fa)
 {
 	if (!(*fa & FA_MODIFY))
 		return;
@@ -1838,7 +1952,7 @@ unsigned char *fa;
  * Support for screen-size swapping for scrolling
  */
 void
-ctlr_shrink()
+ctlr_shrink(void)
 {
 	(void) memset((char *)screen_buf,
 	    *debugging_font ? CG_space : CG_null,
@@ -1856,23 +1970,19 @@ ctlr_shrink()
  */
 static struct timeval t_start;
 static Boolean ticking = False;
-static XtIntervalId tick_id;
+static unsigned long tick_id;
 static struct timeval t_want;
 
 /* Return the difference in milliseconds between two timevals. */
 static long
-delta_msec(t1, t0)
-struct timeval *t1, *t0;
+delta_msec(struct timeval *t1, struct timeval *t0)
 {
 	return (t1->tv_sec - t0->tv_sec) * 1000 +
 	       (t1->tv_usec - t0->tv_usec + 500) / 1000;
 }
 
-/*ARGSUSED*/
 static void
-keep_ticking(closure, id)
-XtPointer closure;
-XtIntervalId *id;
+keep_ticking(void)
 {
 	struct timeval t1;
 	long msec;
@@ -1882,43 +1992,39 @@ XtIntervalId *id;
 		t_want.tv_sec++;
 		msec = delta_msec(&t_want, &t1);
 	} while (msec <= 0);
-	tick_id = XtAppAddTimeOut(appcontext, msec, keep_ticking, 0);
+	tick_id = AddTimeOut(msec, keep_ticking);
 	status_timing(&t_start, &t1);
 }
 
 void
-ticking_start(anyway)
-Boolean anyway;
+ticking_start(Boolean anyway)
 {
 	if (!toggled(SHOW_TIMING) && !anyway)
 		return;
 	status_untiming();
 	if (ticking)
-		XtRemoveTimeOut(tick_id);
+		RemoveTimeOut(tick_id);
 	ticking = True;
 	(void) gettimeofday(&t_start, (struct timezone *) 0);
-	tick_id = XtAppAddTimeOut(appcontext, 1000, keep_ticking, 0);
+	tick_id = AddTimeOut(1000, keep_ticking);
 	t_want = t_start;
 }
 
-void
-ticking_stop()
+static void
+ticking_stop(void)
 {
 	struct timeval t1;
 
 	if (!ticking)
 		return;
-	XtRemoveTimeOut(tick_id);
+	RemoveTimeOut(tick_id);
 	(void) gettimeofday(&t1, (struct timezone *) 0);
 	ticking = False;
 	status_timing(&t_start, &t1);
 }
 
-/*ARGSUSED*/
 void
-toggle_showTiming(t, tt)
-struct toggle *t;
-enum toggle_type tt;
+toggle_showTiming(struct toggle *t unused, enum toggle_type tt unused)
 {
 	if (!toggled(SHOW_TIMING))
 		status_untiming();
@@ -1928,10 +2034,7 @@ enum toggle_type tt;
 /*
  * No-op toggle.
  */
-/*ARGSUSED*/
 void
-toggle_nop(t, tt)
-struct toggle *t;
-enum toggle_type tt;
+toggle_nop(struct toggle *t unused, enum toggle_type tt unused)
 {
 }
