@@ -1,6 +1,6 @@
 /*
- * Modifications Copyright 1996 by Paul Mattes.
- * Copyright Octover 1995 by Dick Altenbern.
+ * Modifications Copyright 1996, 1999, 2000, 2001 by Paul Mattes.
+ * Copyright October 1995 by Dick Altenbern.
  * Based in part on code Copyright 1993, 1994, 1995 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -16,6 +16,9 @@
 
 #include "globals.h"
 
+#if defined(X3270_FT) /*[*/
+
+#if defined(X3270_DISPLAY) /*[*/
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Toggle.h>
 #include <X11/Xaw/Command.h>
@@ -26,18 +29,17 @@
 #include <X11/Xaw/TextSink.h>
 #include <X11/Xaw/AsciiSrc.h>
 #include <X11/Xaw/AsciiSink.h>
-#include <errno.h>
-
-#if XtSpecificationRelease < 5 /*[*/
-#define TOGGLE_HACK 1
 #endif /*]*/
+#include <errno.h>
 
 #include "appres.h"
 #include "actionsc.h"
 #include "ft_cutc.h"
 #include "ft_dftc.h"
 #include "ftc.h"
+#include "hostc.h"
 #include "kybdc.h"
+#include "macrosc.h"
 #include "objects.h"
 #include "popupsc.h"
 #include "telnetc.h"
@@ -46,18 +48,24 @@
 /* Macros. */
 #define eos(s)	strchr((s), '\0')
 
+#if defined(X3270_DISPLAY) /*[*/
 #define FILE_WIDTH	300	/* width of file name widgets */
 #define MARGIN		3	/* distance from margins to widgets */
 #define CLOSE_VGAP	0	/* distance between paired toggles */
 #define FAR_VGAP	10	/* distance between single toggles and groups */
 #define BUTTON_GAP	5	/* horizontal distance between buttons */
 #define COLUMN_GAP	40	/* distance between columns */
+#endif /*]*/
+
+#define BN	(Boolean *)NULL
 
 /* Externals. */
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 extern Pixmap diamond;
 extern Pixmap no_diamond;
 extern Pixmap null;
 extern Pixmap dot;
+#endif /*]*/
 
 /* Globals. */
 enum ft_state ft_state = FT_NONE;	/* File transfer state */
@@ -65,9 +73,11 @@ char *ft_local_filename;		/* Local file to transfer to/from */
 FILE *ft_local_file = (FILE *)NULL;	/* File descriptor for local file */
 Boolean ascii_flag = True;		/* Convert to ascii */
 Boolean cr_flag = True;			/* Add crlf to each line */
+Boolean remap_flag = True;		/* Remap ASCII<->EBCDIC */
 unsigned long ft_length = 0;		/* Length of transfer */
 
 /* Statics. */
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 static Widget ft_dialog, ft_shell, local_file, host_file;
 static Widget lrecl_widget, blksize_widget;
 static Widget primspace_widget, secspace_widget;
@@ -75,11 +85,15 @@ static Widget send_toggle, receive_toggle;
 static Widget vm_toggle, tso_toggle;
 static Widget ascii_toggle, binary_toggle;
 static Widget cr_widget;
+static Widget remap_widget;
+#endif /*]*/
 
+static char *ft_host_filename;		/* Host file to transfer to/from */
 static Boolean receive_flag = True;	/* Current transfer is receive */
 static Boolean append_flag = False;	/* Append transfer */
 static Boolean vm_flag = False;		/* VM Transfer flag */
 
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 struct toggle_list {			/* List of toggle widgets */
 	Widget *widgets;
 };
@@ -87,33 +101,43 @@ static Widget recfm_options[5];
 static Widget units_options[5];
 static struct toggle_list recfm_toggles = { recfm_options };
 static struct toggle_list units_toggles = { units_options };
+#endif /*]*/
 
 static enum recfm {
 	DEFAULT_RECFM, FIXED, VARIABLE, UNDEFINED
 } recfm = DEFAULT_RECFM;
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 static Boolean recfm_default = True;
 static enum recfm r_default_recfm = DEFAULT_RECFM;
 static enum recfm r_fixed = FIXED;
 static enum recfm r_variable = VARIABLE;
 static enum recfm r_undefined = UNDEFINED;
+#endif /*]*/
 
 static enum units {
 	DEFAULT_UNITS, TRACKS, CYLINDERS, AVBLOCK
 } units = DEFAULT_UNITS;
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 static Boolean units_default = True;
 static enum units u_default_units = DEFAULT_UNITS;
 static enum units u_tracks = TRACKS;
 static enum units u_cylinders = CYLINDERS;
 static enum units u_avblock = AVBLOCK;
+#endif /*]*/
 
 typedef enum { T_NUMERIC, T_HOSTFILE, T_UNIXFILE } text_t;
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 static text_t t_numeric = T_NUMERIC;
 static text_t t_hostfile = T_HOSTFILE;
 static text_t t_unixfile = T_UNIXFILE;
+#endif /*]*/
 
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 static Boolean s_true = True;
 static Boolean s_false = False;
+#endif /*]*/
 static Boolean allow_overwrite = False;
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 typedef struct sr {
 	struct sr *next;
 	Widget w;
@@ -126,48 +150,77 @@ typedef struct sr {
 	Boolean is_value;
 	Boolean has_focus;
 } sr_t;
-sr_t *sr = (sr_t *)NULL;
-sr_t *sr_last = (sr_t *)NULL;
+static sr_t *sr = (sr_t *)NULL;
+static sr_t *sr_last = (sr_t *)NULL;
 
 static Widget progress_shell, from_file, to_file;
 static Widget ft_status, waiting, aborting;
 static String status_string;
+#endif /*]*/
 static struct timeval t0;		/* Starting time */
 static Boolean ft_is_cut;		/* File transfer is CUT-style */
 
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 static Widget overwrite_shell;
+#endif /*]*/
+static Boolean ft_is_action;
 
-static void apply_bitmap();
-static void check_sensitivity();
-static void flip_toggles();
-static void focus_next();
-static void ft_cancel();
-static void ft_popup_callback();
-static void ft_popup_init();
-static int ft_start();
-static void ft_start_callback();
-static void match_dimension();
-static void overwrite_cancel_callback();
-static void overwrite_okay_callback();
-static void overwrite_popdown();
-static void overwrite_popup_init();
-static void popup_overwrite();
-static void popup_progress();
-static void progress_cancel_callback();
-static void progress_popup_callback();
-static void progress_popup_init();
-static void recfm_callback();
-static void register_sensitivity();
-static void text_callback();
-static void toggle_append();
-static void toggle_ascii();
-static void toggle_cr();
-static void toggle_receive();
-static void toggle_vm();
-static void units_callback();
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
+static void apply_bitmap(Widget w, Pixmap p);
+static void check_sensitivity(Boolean *bvar);
+static void flip_toggles(struct toggle_list *toggle_list, Widget w);
+static void focus_next(sr_t *s);
+static void ft_cancel(Widget w, XtPointer client_data, XtPointer call_data);
+static void ft_popup_callback(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void ft_popup_init(void);
+static int ft_start(void);
+static void ft_start_callback(Widget w, XtPointer call_parms,
+    XtPointer call_data);
+static void match_dimension(Widget w1, Widget w2, const char *n);
+static void overwrite_cancel_callback(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void overwrite_okay_callback(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void overwrite_popdown(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void overwrite_popup_init(void);
+static void popup_overwrite(void);
+static void popup_progress(void);
+static void progress_cancel_callback(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void progress_popup_callback(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void progress_popup_init(void);
+static void recfm_callback(Widget w, XtPointer user_data, XtPointer call_data);
+static void register_sensitivity(Widget w, Boolean *bvar1, Boolean bval1,
+    Boolean *bvar2, Boolean bval2, Boolean *bvar3, Boolean bval3);
+static void text_callback(Widget w, XtPointer client_data, XtPointer call_data);
+static void toggle_append(Widget w, XtPointer client_data, XtPointer call_data);
+static void toggle_ascii(Widget w, XtPointer client_data, XtPointer call_data);
+static void toggle_cr(Widget w, XtPointer client_data, XtPointer call_data);
+static void toggle_remap(Widget w, XtPointer client_data, XtPointer call_data);
+static void toggle_receive(Widget w, XtPointer client_data,
+    XtPointer call_data);
+static void toggle_vm(Widget w, XtPointer client_data, XtPointer call_data);
+static void units_callback(Widget w, XtPointer user_data, XtPointer call_data);
+#endif /*]*/
+static void ft_connected(Boolean ignored);
+static void ft_in3270(Boolean ignored);
 
 /* Main external entry point. */
 
+#if !defined(X3270_DISPLAY) || !defined(X3270_MENUS) /*[*/
+void
+ft_init(void)
+{
+	/* Register for state changes. */
+	register_schange(ST_CONNECT, ft_connected);
+	register_schange(ST_3270_MODE, ft_in3270);
+}
+#endif /*]*/
+
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 /* "File Transfer" dialog. */
 
 /*
@@ -175,10 +228,8 @@ static void units_callback();
  * Called back from the "File Transfer" option on the File menu.
  */
 void
-popup_ft(w, call_parms, call_data)
-Widget w;
-XtPointer call_parms;
-XtPointer call_data;
+popup_ft(Widget w unused, XtPointer call_parms unused,
+	XtPointer call_data unused)
 {
 	/* Initialize it. */
 	if (ft_shell == (Widget)NULL)
@@ -188,10 +239,9 @@ XtPointer call_data;
 	popup_popup(ft_shell, XtGrabNone);
 }
 
-
 /* Initialize the transfer pop-up. */
 static void
-ft_popup_init()
+ft_popup_init(void)
 {
 	Widget w;
 	Widget cancel_button;
@@ -204,6 +254,10 @@ ft_popup_init()
 	Widget recfm_label, units_label;
 	Widget start_button;
 
+	/* Register for state changes. */
+	register_schange(ST_CONNECT, ft_connected);
+	register_schange(ST_3270_MODE, ft_in3270);
+
 	/* Create the menu shell. */
 	ft_shell = XtVaCreatePopupShell(
 	    "ftPopup", transientShellWidgetClass, toplevel,
@@ -215,7 +269,7 @@ ft_popup_init()
 
 	/* Create the form within the shell. */
 	ft_dialog = XtVaCreateManagedWidget(
-	    "dialog", formWidgetClass, ft_shell,
+	    ObjDialog, formWidgetClass, ft_shell,
 	    NULL);
 
 	/* Create the file name widgets. */
@@ -241,9 +295,9 @@ ft_popup_init()
 		XtAddCallback(w, XtNcallback, text_callback,
 		    (XtPointer)&t_unixfile);
 	register_sensitivity(local_file,
-	    NULL, NULL,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False,
+	    BN, False);
 
 	host_label = XtVaCreateManagedWidget(
 	    "host", labelWidgetClass, ft_dialog,
@@ -271,9 +325,9 @@ ft_popup_init()
 		XtAddCallback(w, XtNcallback, text_callback,
 		    (XtPointer)&t_hostfile);
 	register_sensitivity(host_file,
-	    NULL, NULL,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False,
+	    BN, False);
 
 	/* Create the left column. */
 
@@ -342,8 +396,8 @@ ft_popup_init()
 	    NULL);
 	register_sensitivity(recfm_label,
 	    &receive_flag, False,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False);
 
 	recfm_options[0] = XtVaCreateManagedWidget(
 	    "recfmDefault", commandWidgetClass, ft_dialog,
@@ -358,8 +412,8 @@ ft_popup_init()
 	    (XtPointer)&r_default_recfm);
 	register_sensitivity(recfm_options[0],
 	    &receive_flag, False,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False);
 
 	recfm_options[1] = XtVaCreateManagedWidget(
 	    "fixed", commandWidgetClass, ft_dialog,
@@ -374,8 +428,8 @@ ft_popup_init()
 	    (XtPointer)&r_fixed);
 	register_sensitivity(recfm_options[1],
 	    &receive_flag, False,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False);
 
 	recfm_options[2] = XtVaCreateManagedWidget(
 	    "variable", commandWidgetClass, ft_dialog,
@@ -390,8 +444,8 @@ ft_popup_init()
 	    (XtPointer)&r_variable);
 	register_sensitivity(recfm_options[2],
 	    &receive_flag, False,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False);
 
 	recfm_options[3] = XtVaCreateManagedWidget(
 	    "undefined", commandWidgetClass, ft_dialog,
@@ -407,7 +461,7 @@ ft_popup_init()
 	register_sensitivity(recfm_options[3],
 	    &receive_flag, False,
 	    &vm_flag, False,
-	    NULL, NULL);
+	    BN, False);
 
 	lrecl_label = XtVaCreateManagedWidget(
 	    "lrecl", labelWidgetClass, ft_dialog,
@@ -419,7 +473,7 @@ ft_popup_init()
 	register_sensitivity(lrecl_label,
 	    &receive_flag, False,
 	    &recfm_default, False,
-	    NULL, NULL);
+	    BN, False);
 	lrecl_widget = XtVaCreateManagedWidget(
 	    "value", asciiTextWidgetClass, ft_dialog,
 	    XtNfromVert, recfm_options[3],
@@ -440,7 +494,7 @@ ft_popup_init()
 	register_sensitivity(lrecl_widget,
 	    &receive_flag, False,
 	    &recfm_default, False,
-	    NULL, NULL);
+	    BN, False);
 
 	blksize_label = XtVaCreateManagedWidget(
 	    "blksize", labelWidgetClass, ft_dialog,
@@ -453,7 +507,7 @@ ft_popup_init()
 	register_sensitivity(blksize_label,
 	    &receive_flag, False,
 	    &recfm_default, False,
-	    NULL, NULL);
+	    BN, False);
 	blksize_widget = XtVaCreateManagedWidget(
 	    "value", asciiTextWidgetClass, ft_dialog,
 	    XtNfromVert, lrecl_widget,
@@ -474,7 +528,7 @@ ft_popup_init()
 	register_sensitivity(blksize_widget,
 	    &receive_flag, False,
 	    &recfm_default, False,
-	    NULL, NULL);
+	    BN, False);
 
 
 	/* Find the widest widget in the left column. */
@@ -530,8 +584,24 @@ ft_popup_init()
 	XtAddCallback(cr_widget, XtNcallback, toggle_cr, 0);
 	register_sensitivity(cr_widget,
 	    &ascii_flag, True,
-	    NULL, NULL,
-	    NULL, NULL);
+	    BN, False,
+	    BN, False);
+
+	/* Create remap toggle. */
+	remap_widget = XtVaCreateManagedWidget(
+	    "remap", commandWidgetClass, ft_dialog,
+	    XtNfromVert, cr_widget,
+	    XtNfromHoriz, h_ref,
+	    XtNvertDistance, CLOSE_VGAP,
+	    XtNhorizDistance, COLUMN_GAP,
+	    XtNborderWidth, 0,
+	    NULL);
+	apply_bitmap(remap_widget, remap_flag ? dot : null);
+	XtAddCallback(remap_widget, XtNcallback, toggle_remap, NULL);
+	register_sensitivity(remap_widget,
+	    &ascii_flag, True,
+	    BN, False,
+	    BN, False);
 
 	/* Set up the Units group. */
 	units_label = XtVaCreateManagedWidget(
@@ -545,7 +615,7 @@ ft_popup_init()
 	register_sensitivity(units_label,
 	    &receive_flag, False,
 	    &vm_flag, False,
-	    NULL, NULL);
+	    BN, False);
 
 	units_options[0] = XtVaCreateManagedWidget(
 	    "spaceDefault", commandWidgetClass, ft_dialog,
@@ -562,7 +632,7 @@ ft_popup_init()
 	register_sensitivity(units_options[0],
 	    &receive_flag, False,
 	    &vm_flag, False,
-	    NULL, NULL);
+	    BN, False);
 
 	units_options[1] = XtVaCreateManagedWidget(
 	    "tracks", commandWidgetClass, ft_dialog,
@@ -579,7 +649,7 @@ ft_popup_init()
 	register_sensitivity(units_options[1],
 	    &receive_flag, False,
 	    &vm_flag, False,
-	    NULL, NULL);
+	    BN, False);
 
 	units_options[2] = XtVaCreateManagedWidget(
 	    "cylinders", commandWidgetClass, ft_dialog,
@@ -596,7 +666,7 @@ ft_popup_init()
 	register_sensitivity(units_options[2],
 	    &receive_flag, False,
 	    &vm_flag, False,
-	    NULL, NULL);
+	    BN, False);
 
 	units_options[3] = XtVaCreateManagedWidget(
 	    "avblock", commandWidgetClass, ft_dialog,
@@ -613,7 +683,7 @@ ft_popup_init()
 	register_sensitivity(units_options[3],
 	    &receive_flag, False,
 	    &vm_flag, False,
-	    NULL, NULL);
+	    BN, False);
 
 	primspace_label = XtVaCreateManagedWidget(
 	    "primspace", labelWidgetClass, ft_dialog,
@@ -704,19 +774,15 @@ ft_popup_init()
 	XtAddCallback(cancel_button, XtNcallback, ft_cancel, 0);
 }
 
-
-
 /* Callbacks for all the transfer widgets. */
 
 /* Transfer pop-up popping up. */
 static void
-ft_popup_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+ft_popup_callback(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	/* Set the focus to the local file widget. */
-	dialog_focus_action(local_file, (XEvent *)NULL, (String *)NULL,
+	PA_dialog_focus_action(local_file, (XEvent *)NULL, (String *)NULL,
 	    (Cardinal *)NULL);
 
 	/* Disallow overwrites. */
@@ -725,20 +791,15 @@ XtPointer call_data;
 
 /* Cancel button pushed. */
 static void
-ft_cancel(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+ft_cancel(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	XtPopdown(ft_shell);
 }
 
 /* recfm options. */
 static void
-recfm_callback(w, user_data, call_data)
-Widget w;
-XtPointer user_data;
-XtPointer call_data;
+recfm_callback(Widget w, XtPointer user_data, XtPointer call_data unused)
 {
 	recfm = *(enum recfm *)user_data;
 	recfm_default = (recfm == DEFAULT_RECFM);
@@ -748,10 +809,7 @@ XtPointer call_data;
 
 /* Units options. */
 static void
-units_callback(w, user_data, call_data)
-Widget w;
-XtPointer user_data;
-XtPointer call_data;
+units_callback(Widget w, XtPointer user_data, XtPointer call_data unused)
 {
 	units = *(enum units *)user_data;
 	units_default = (units == DEFAULT_UNITS);
@@ -761,10 +819,8 @@ XtPointer call_data;
 
 /* OK button pushed. */
 static void
-ft_start_callback(w, call_parms, call_data)
-Widget w;
-XtPointer call_parms;
-XtPointer call_data;
+ft_start_callback(Widget w unused, XtPointer call_parms unused,
+	XtPointer call_data unused)
 {
 	if (ft_start()) {
 		XtPopdown(ft_shell);
@@ -774,36 +830,15 @@ XtPointer call_data;
 
 /* Mark a toggle. */
 static void
-mark_toggle(w, p)
-Widget w;
-Pixmap p;
+mark_toggle(Widget w, Pixmap p)
 {
-#if defined(TOGGLE_HACK) /*[*/
-	String l, nl;
-
-	XtVaGetValues(w, XtNlabel, &l, NULL);
-	nl = XtNewString(l);
-	if (p == diamond)
-		nl[0] = '+';
-	else if (p == no_diamond)
-		nl[0] = '-';
-	else if (p == dot)
-		nl[0] = '*';
-	else if (p == null)
-		nl[0] = ' ';
-	XtVaSetValues(w, XtNlabel, nl, NULL);
-	XtFree(nl);
-#else /*][*/
 	XtVaSetValues(w, XtNleftBitmap, p, NULL);
-#endif /*]*/
 }
 
 /* Send/receive options. */
 static void
-toggle_receive(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+toggle_receive(Widget w unused, XtPointer client_data,
+	XtPointer call_data unused)
 {
 	/* Toggle the flag */
 	receive_flag = *(Boolean *)client_data;
@@ -816,10 +851,7 @@ XtPointer call_data;
 
 /* Ascii/binary options. */
 static void
-toggle_ascii(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+toggle_ascii(Widget w unused, XtPointer client_data, XtPointer call_data unused)
 {
 	/* Toggle the flag. */
 	ascii_flag = *(Boolean *)client_data;
@@ -828,16 +860,15 @@ XtPointer call_data;
 	mark_toggle(ascii_toggle, ascii_flag ? diamond : no_diamond);
 	mark_toggle(binary_toggle, ascii_flag ? no_diamond : diamond);
 	cr_flag = ascii_flag;
+	remap_flag = ascii_flag;
 	mark_toggle(cr_widget, cr_flag ? dot : null);
+	mark_toggle(remap_widget, remap_flag ? dot : null);
 	check_sensitivity(&ascii_flag);
 }
 
 /* CR option. */
 static void
-toggle_cr(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+toggle_cr(Widget w, XtPointer client_data unused, XtPointer call_data unused)
 {
 	/* Toggle the cr flag */
 	cr_flag = !cr_flag;
@@ -847,10 +878,8 @@ XtPointer call_data;
 
 /* Append option. */
 static void
-toggle_append(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+toggle_append(Widget w, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	/* Toggle Append Flag */
 	append_flag = !append_flag;
@@ -858,12 +887,21 @@ XtPointer call_data;
 	mark_toggle(w, append_flag ? dot : null);
 }
 
+/* Remap option. */
+static void
+toggle_remap(Widget w, XtPointer client_data unused,
+	XtPointer call_data unused)
+{
+	/* Toggle Remap Flag */
+	remap_flag = !remap_flag;
+
+	mark_toggle(w, remap_flag ? dot : null);
+}
+
 /* TSO/VM option. */
 static void
-toggle_vm(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+toggle_vm(Widget w unused, XtPointer client_data unused,
+    XtPointer call_data unused)
 {
 	/* Toggle the flag. */
 	vm_flag = *(Boolean *)client_data;
@@ -888,18 +926,19 @@ XtPointer call_data;
  * Returns 1 if the transfer has started, 0 otherwise.
  */
 static int
-ft_start()
+ft_start(void)
 {
 	char opts[80];
 	char *op = opts + 1;
 	char *cmd;
 	String lrecl, blksize, primspace, secspace;
-	int flen;
-	char *host_filename;
+	unsigned flen;
+
+	ft_is_action = False;
 
 	/* Get the host file from its widget */
-	XtVaGetValues(host_file, XtNstring, &host_filename, NULL);
-	if (!*host_filename)
+	XtVaGetValues(host_file, XtNstring, &ft_host_filename, NULL);
+	if (!*ft_host_filename)
 		return 0;
 	/* XXX: probably more validation to do here */
 
@@ -1031,7 +1070,7 @@ ft_start()
 	/* Build the whole command. */
 	cmd = xs_buffer("%s %s %s%s\\n",
 	    appres.ft_command,
-	    receive_flag ? "get" : "put", host_filename, op);
+	    receive_flag ? "get" : "put", ft_host_filename, op);
 
 	/* Erase the line and enter the command. */
 	flen = kybd_prime();
@@ -1055,7 +1094,7 @@ ft_start()
 
 /* Pop up the "in progress" pop-up. */
 static void
-popup_progress()
+popup_progress(void)
 {
 	/* Initialize it. */
 	if (progress_shell == (Widget)NULL)
@@ -1067,7 +1106,7 @@ popup_progress()
 
 /* Initialize the "in progress" pop-up. */
 static void
-progress_popup_init()
+progress_popup_init(void)
 {
 	Widget progress_pop, from_label, to_label, cancel_button;
 
@@ -1082,7 +1121,7 @@ progress_popup_init()
 
 	/* Create a form structure to contain the other stuff */
 	progress_pop = XtVaCreateManagedWidget(
-	    "dialog", formWidgetClass, progress_shell,
+	    ObjDialog, formWidgetClass, progress_shell,
 	    NULL);
 
 	/* Create the widgets. */
@@ -1164,18 +1203,13 @@ progress_popup_init()
 
 /* In-progress pop-up popped up. */
 static void
-progress_popup_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+progress_popup_callback(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
-	String hf, lf;
-
-	XtVaGetValues(host_file, XtNstring, &hf, NULL);
-	XtVaGetValues(local_file, XtNstring, &lf, NULL);
-
-	XtVaSetValues(from_file, XtNlabel, receive_flag ? hf : lf, NULL);
-	XtVaSetValues(to_file, XtNlabel, receive_flag ? lf : hf, NULL);
+	XtVaSetValues(from_file, XtNlabel,
+	    receive_flag ? ft_host_filename : ft_local_filename, NULL);
+	XtVaSetValues(to_file, XtNlabel,
+	    receive_flag ? ft_local_filename : ft_host_filename, NULL);
 
 	switch (ft_state) {
 	    case FT_AWAIT_ACK:
@@ -1201,10 +1235,8 @@ XtPointer call_data;
 
 /* In-progress "cancel" button. */
 static void
-progress_cancel_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+progress_cancel_callback(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	if (ft_state == FT_RUNNING) {
 		ft_state = FT_ABORT_WAIT;
@@ -1221,7 +1253,7 @@ XtPointer call_data;
 
 /* Pop up the "overwrite" pop-up. */
 static void
-popup_overwrite()
+popup_overwrite(void)
 {
 	/* Initialize it. */
 	if (overwrite_shell == (Widget)NULL)
@@ -1233,7 +1265,7 @@ popup_overwrite()
 
 /* Initialize the "overwrite" pop-up. */
 static void
-overwrite_popup_init()
+overwrite_popup_init(void)
 {
 	Widget overwrite_pop, overwrite_name, okay_button, cancel_button;
 	String overwrite_string, label, lf;
@@ -1250,7 +1282,7 @@ overwrite_popup_init()
 
 	/* Create a form structure to contain the other stuff */
 	overwrite_pop = XtVaCreateManagedWidget(
-	    "dialog", formWidgetClass, overwrite_shell,
+	    ObjDialog, formWidgetClass, overwrite_shell,
 	    NULL);
 
 	/* Create the widgets. */
@@ -1273,7 +1305,7 @@ overwrite_popup_init()
 		d += 20;
 	XtVaSetValues(overwrite_name, XtNwidth, d, NULL);
 	XtVaGetValues(overwrite_name, XtNheight, &d, NULL);
-	XtVaSetValues(overwrite_name, XtNheight, d + 10, NULL);
+	XtVaSetValues(overwrite_name, XtNheight, d + 20, NULL);
 
 	okay_button = XtVaCreateManagedWidget(
 	    ObjConfirmButton, commandWidgetClass, overwrite_pop,
@@ -1297,10 +1329,8 @@ overwrite_popup_init()
 
 /* Overwrite "okay" button. */
 static void
-overwrite_okay_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+overwrite_okay_callback(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	XtPopdown(overwrite_shell);
 
@@ -1313,31 +1343,27 @@ XtPointer call_data;
 
 /* Overwrite "cancel" button. */
 static void
-overwrite_cancel_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+overwrite_cancel_callback(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	XtPopdown(overwrite_shell);
 }
 
 /* Overwrite pop-up popped down. */
 static void
-overwrite_popdown(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+overwrite_popdown(Widget w unused, XtPointer client_data unused,
+	XtPointer call_data unused)
 {
 	XtDestroyWidget(overwrite_shell);
 	overwrite_shell = (Widget)NULL;
 }
+#endif /*]*/
 
 /* External entry points called by ft_dft and ft_cut. */
 
 /* Pop up a message, end the transfer. */
 void
-ft_complete(errmsg)
-String errmsg;
+ft_complete(const char *errmsg)
 {
 	/* Close the local file. */
 	if (ft_local_file != (FILE *)NULL && fclose(ft_local_file) < 0)
@@ -1347,22 +1373,26 @@ String errmsg;
 	/* Clean up the state. */
 	ft_state = FT_NONE;
 
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 	/* Pop down the in-progress shell. */
 	XtPopdown(progress_shell);
+#endif /*]*/
 
 	/* Pop up the text. */
-	if (errmsg != (String)NULL) {
-		/* Make sure the error message will fit on the display. */
-		if (strlen(errmsg) > 50 && strchr(errmsg, '\n') == CN) {
-			char *s = errmsg + 50;
+	if (errmsg != CN) {
+		char *msg_copy = NewString(errmsg);
 
-			while (s > errmsg && *s != ' ')
+		/* Make sure the error message will fit on the display. */
+		if (strlen(msg_copy) > 50 && strchr(msg_copy, '\n') == CN) {
+			char *s = msg_copy + 50;
+
+			while (s > msg_copy && *s != ' ')
 				s--;
-			if (s > errmsg)
-				*s = '\n';
+			if (s > msg_copy)
+				*s = '\n';	/* yikes! */
 		}
-		popup_an_error(errmsg);
-		XtFree(errmsg);
+		popup_an_error(msg_copy);
+		Free(msg_copy);
 	} else {
 		struct timeval t1;
 		double kbytes_sec;
@@ -1372,30 +1402,38 @@ String errmsg;
 		kbytes_sec = (double)ft_length / 1024.0 /
 			((double)(t1.tv_sec - t0.tv_sec) + 
 			 (double)(t1.tv_usec - t0.tv_usec) / 1.0e6);
-		buf = XtMalloc(256);
+		buf = Malloc(256);
 		(void) sprintf(buf, get_message("ftComplete"), ft_length,
 		    kbytes_sec, ft_is_cut ? "CUT" : "DFT");
-		popup_an_info(buf);
-		XtFree(buf);
+		if (ft_is_action) {
+			sms_info("%s", buf);
+			sms_continue();
+		}
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
+		else
+			popup_an_info(buf);
+#endif /*]*/
+		Free(buf);
 	}
 }
 
 /* Update the bytes-transferred count on the progress pop-up. */
 void
-ft_update_length()
+ft_update_length(void)
 {
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 	char text_string[80];
 
 	/* Format the message */
 	sprintf(text_string, status_string, ft_length);
 
 	XtVaSetValues(ft_status, XtNlabel, text_string, NULL);
+#endif /*]*/
 }
 
 /* Process a transfer acknowledgement. */
 void
-ft_running(is_cut)
-Boolean is_cut;
+ft_running(Boolean is_cut)
 {
 	if (ft_state == FT_AWAIT_ACK)
 		ft_state = FT_RUNNING;
@@ -1403,47 +1441,51 @@ Boolean is_cut;
 	(void) gettimeofday(&t0, (struct timezone *)NULL);
 	ft_length = 0;
 
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 	XtUnmapWidget(waiting);
+#endif /*]*/
 	ft_update_length();
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 	XtMapWidget(ft_status);
+#endif /*]*/
 }
 
 /* Process a protocol-generated abort. */
 void
-ft_aborting()
+ft_aborting(void)
 {
 	if (ft_state == FT_RUNNING || ft_state == FT_ABORT_WAIT) {
 		ft_state = FT_ABORT_SENT;
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 		XtUnmapWidget(waiting);
 		XtUnmapWidget(ft_status);
 		XtMapWidget(aborting);
+#endif /*]*/
 	}
 }
 
 /* Process a disconnect abort. */
-void
-ft_disconnected()
+static void
+ft_connected(Boolean ignored unused)
 {
-	if (ft_state != FT_NONE)
+	if (!CONNECTED && ft_state != FT_NONE)
 		ft_complete(get_message("ftDisconnected"));
 }
 
 /* Process an abort from no longer being in 3270 mode. */
-void
-ft_not3270()
+static void
+ft_in3270(Boolean ignored unused)
 {
-	if (ft_state != FT_NONE)
+	if (!IN_3270 && ft_state != FT_NONE)
 		ft_complete(get_message("ftNot3270"));
 }
 
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
 /* Support functions for dialogs. */
 
 /* Match one dimension of two widgets. */
 static void
-match_dimension(w1, w2, n)
-Widget w1;
-Widget w2;
-String n;
+match_dimension(Widget w1, Widget w2, const char *n)
 {
 	Dimension h1, h2;
 	Dimension b1, b2;
@@ -1460,43 +1502,19 @@ String n;
 
 /* Apply a bitmap to a widget. */
 static void
-apply_bitmap(w, p)
-Widget w;
-Pixmap p;
+apply_bitmap(Widget w, Pixmap p)
 {
 	Dimension d1;
-#if defined(TOGGLE_HACK) /*[*/
-	String l, nl;
-#endif /*]*/
 
 	XtVaGetValues(w, XtNheight, &d1, NULL);
 	if (d1 < 10)
 		XtVaSetValues(w, XtNheight, 10, NULL);
-#if defined(TOGGLE_HACK) /*[*/
-	XtVaGetValues(w, XtNlabel, &l, NULL);
-	nl = XtMalloc(strlen(l) + 6);
-	if (p == diamond)
-		(void) sprintf(nl, "+  %s", l);
-	else if (p == dot)
-		(void) sprintf(nl, "*  %s", l);
-	else if (p == no_diamond)
-		(void) sprintf(nl, "-  %s", l);
-	else if (p == null)
-		(void) sprintf(nl, "   %s", l);
-	XtVaSetValues(w,
-	    XtNlabel, nl,
-	    NULL);
-	XtFree(nl);
-#else /*][*/
 	XtVaSetValues(w, XtNleftBitmap, p, NULL);
-#endif /*]*/
 }
 
 /* Flip a multi-valued toggle. */
 static void
-flip_toggles(toggle_list, w)
-struct toggle_list *toggle_list;
-Widget w;
+flip_toggles(struct toggle_list *toggle_list, Widget w)
 {
 	int i;
 
@@ -1513,10 +1531,7 @@ Widget w;
  * specified criteria.
  */
 static void
-text_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+text_callback(Widget w, XtPointer client_data, XtPointer call_data unused)
 {
 	XawTextBlock b;		/* firstPos, length, ptr, format */
 	static XawTextBlock nullb = { 0, 0, NULL };
@@ -1577,14 +1592,8 @@ XtPointer call_data;
 
 /* Register widget sensitivity, based on zero to three Booleans. */
 static void
-register_sensitivity(w, bvar1, bval1, bvar2, bval2, bvar3, bval3)
-Widget w;
-Boolean *bvar1;
-Boolean bval1;
-Boolean *bvar2;
-Boolean bval2;
-Boolean *bvar3;
-Boolean bval3;
+register_sensitivity(Widget w, Boolean *bvar1, Boolean bval1, Boolean *bvar2,
+    Boolean bval2, Boolean *bvar3, Boolean bval3)
 {
 	sr_t *s;
 	Boolean f;
@@ -1624,8 +1633,7 @@ Boolean bval3;
 
 /* Scan the list of registered widgets for a sensitivity change. */
 static void
-check_sensitivity(bvar)
-Boolean *bvar;
+check_sensitivity(Boolean *bvar)
 {
 	sr_t *s;
 
@@ -1650,8 +1658,7 @@ Boolean *bvar;
 
 /* Move the input focus to the next sensitive value field. */
 static void
-focus_next(s)
-sr_t *s;
+focus_next(sr_t *s)
 {
 	sr_t *t;
 	Boolean sen;
@@ -1691,11 +1698,8 @@ sr_t *s;
 
 /* Proceed to the next input field. */
 void
-dialog_next_action(w, event, parms, num_parms)
-Widget w;
-XEvent *event;
-String *parms;
-Cardinal *num_parms;
+PA_dialog_next_action(Widget w, XEvent *event unused, String *parms unused,
+	Cardinal *num_parms unused)
 {
 	sr_t *s;
 
@@ -1709,11 +1713,8 @@ Cardinal *num_parms;
 
 /* Set keyboard focus to an input field. */
 void
-dialog_focus_action(w, event, parms, num_parms)
-Widget w;
-XEvent *event;
-String *parms;
-Cardinal *num_parms;
+PA_dialog_focus_action(Widget w, XEvent *event unused, String *parms unused,
+	Cardinal *num_parms unused)
 {
 	sr_t *s;
 
@@ -1741,3 +1742,323 @@ Cardinal *num_parms;
 	XawTextDisplayCaret(w, True);
 	XtSetKeyboardFocus(ft_dialog, w);
 }
+#endif /*]*/
+
+/*
+ * Script/macro action for file transfer.
+ *  Transfer(option=value[,...])
+ *  Options are:
+ *   Direction=send|receive	default receive
+ *   HostFile=name		required
+ *   LocalFile=name			required
+ *   Host=[tso|vm]		default tso
+ *   Mode=[ascii|binary]	default ascii
+ *   Cr=[add|remove|keep]	default add/remove
+ *   Exist=[keep|replace|append]	default keep
+ *   Recfm=[default|fixed|variable|undefined] default default
+ *   Lrecl=n			no default
+ *   Blksize=n			no default
+ *   Allocation=[default|tracks|cylinders|avblock] default default
+ *   PrimarySpace=n		no default
+ *   SecondarySpace=n		no default
+ */
+static struct {
+	const char *name;
+	char *value;
+	const char *keyword[4];
+} tp[] = {
+	{ "Direction",		CN, { "receive", "send" } },
+	{ "HostFile" },
+	{ "LocalFile" },
+	{ "Host",		CN, { "tso", "vm" } },
+	{ "Mode",		CN, { "ascii", "binary" } },
+	{ "Cr",			CN, { "remove",	"add", "keep" } },
+	{ "Exist",		CN, { "keep", "replace", "append" } },
+	{ "Recfm",		CN, { "default", "fixed", "variable",
+				      "undefined" } },
+	{ "Lrecl" },
+	{ "Blksize" },
+	{ "Allocation",		CN, { "default", "tracks", "cylinders",
+				      "avblock" } },
+	{ "PrimarySpace" },
+	{ "SecondarySpace" },
+	{ CN }
+};
+#define PARM_DIRECTION		0
+#define PARM_HOST_FILE		1
+#define PARM_LOCAL_FILE		2
+#define PARM_HOST		3
+#define PARM_MODE		4
+#define PARM_CR			5
+#define PARM_EXIST		6
+#define PARM_RECFM		7
+#define PARM_LRECL		8
+#define PARM_BLKSIZE		9
+#define PARM_ALLOCATION		10
+#define PARM_PRIMARY_SPACE	11
+#define PARM_SECONDARY_SPACE	12
+#define N_PARMS			(PARM_SECONDARY_SPACE + 1)
+
+void  
+Transfer_action(Widget w unused, XEvent *event, String *params,
+    Cardinal *num_params)
+{
+	int i, k;
+	Cardinal j;
+	long l;
+	char *ptr;
+
+	char opts[80];
+	char *op = opts + 1;
+	char *cmd;
+	unsigned flen;
+
+        action_debug(Transfer_action, event, params, num_params);
+
+	ft_is_action = True;
+
+	/* Make sure we're connected. */
+	if (!IN_3270) {
+		popup_an_error("Not connected");
+		return;
+	}
+
+	/* Set everything to the default. */
+	for (i = 0; i < N_PARMS; i++) {
+		Free(tp[i].value);
+		if (tp[i].keyword[0] != CN)
+			tp[i].value =
+				NewString(tp[i].keyword[0]);
+		else
+			tp[i].value = CN;
+	}
+
+	/* See what they specified. */
+	for (j = 0; j < *num_params; j++) {
+		for (i = 0; i < N_PARMS; i++) {
+			char *eq;
+			int kwlen;
+
+			eq = strchr(params[j], '=');
+			if (eq == CN || eq == params[j] || !*(eq + 1)) {
+				popup_an_error("Invalid option syntax: '%s'",
+					params[j]);
+				return;
+			}
+			kwlen = eq - params[j];
+			if (!strncasecmp(params[j], tp[i].name, kwlen)
+					&& !tp[i].name[kwlen]) {
+				if (tp[i].keyword[0]) {
+					for (k = 0;
+					     tp[i].keyword[k] != CN && k < 4;
+					     k++) {
+						if (!strcasecmp(eq + 1,
+							tp[i].keyword[k])) {
+							break;
+						}
+					}
+					if (k >= 4 ||
+					    tp[i].keyword[k] == CN) {
+						popup_an_error("Invalid option "
+							"value: '%s'", eq + 1);
+						return;
+					}
+				} else switch (i) {
+				    case PARM_LRECL:
+				    case PARM_BLKSIZE:
+				    case PARM_PRIMARY_SPACE:
+				    case PARM_SECONDARY_SPACE:
+					l = strtol(eq + 1, &ptr, 10);
+					if (ptr == eq + 1 || *ptr) {
+						popup_an_error("Invalid option "
+							"value: '%s'", eq + 1);
+						return;
+					}
+					break;
+				    default:
+					break;
+				}
+				tp[i].value = NewString(eq + 1);
+				break;
+			}
+		}
+		if (i >= N_PARMS) {
+			popup_an_error("Unknown option: %s", params[j]);
+			return;
+		}
+	}
+
+	/* Check for required values. */
+	if (tp[PARM_HOST_FILE].value == CN) {
+		popup_an_error("Missing 'HostFile' option");
+		return;
+	}
+	if (tp[PARM_LOCAL_FILE].value == CN) {
+		popup_an_error("Missing 'LocalFile' option");
+		return;
+	}
+
+	/*
+	 * Start the transfer.  Much of this is duplicated from ft_start()
+	 * and should be made common.
+	 */
+
+	receive_flag = !strcasecmp(tp[PARM_DIRECTION].value, "receive");
+	append_flag = !strcasecmp(tp[PARM_EXIST].value, "append");
+	allow_overwrite = !strcasecmp(tp[PARM_EXIST].value, "replace");
+	ascii_flag = !strcasecmp(tp[PARM_MODE].value, "ascii");
+	cr_flag = !strcasecmp(tp[PARM_CR].value, "remove") ||
+		  !strcasecmp(tp[PARM_CR].value, "add");
+	vm_flag = !strcasecmp(tp[PARM_HOST].value, "vm");
+	recfm = DEFAULT_RECFM;
+	for (k = 0; tp[PARM_RECFM].keyword[k] != CN && k < 4; k++) {
+		if (!strcasecmp(tp[PARM_RECFM].value,
+			    tp[PARM_RECFM].keyword[k]))  {
+			recfm = (enum recfm)k;
+			break;
+		}
+	}
+	units = DEFAULT_UNITS;
+	for (k = 0; tp[PARM_ALLOCATION].keyword[k] != CN && k < 4; k++) {
+		if (!strcasecmp(tp[PARM_ALLOCATION].value,
+			    tp[PARM_ALLOCATION].keyword[k]))  {
+			units = (enum units)k;
+			break;
+		}
+	}
+
+	ft_host_filename = tp[PARM_HOST_FILE].value;
+	ft_local_filename = tp[PARM_LOCAL_FILE].value;
+
+	/* See if the local file can be overwritten. */
+	if (receive_flag && !append_flag && !allow_overwrite) {
+		ft_local_file = fopen(ft_local_filename, "r");
+		if (ft_local_file != (FILE *)NULL) {
+			(void) fclose(ft_local_file);
+			popup_an_error("File exists");
+			return;
+		}
+	}
+
+	/* Open the local file. */
+	ft_local_file = fopen(ft_local_filename,
+	    receive_flag ?
+		(append_flag ? "a" : "w" ) :
+		"r");
+	if (ft_local_file == (FILE *)NULL) {
+		popup_an_errno(errno, "Open(%s)", ft_local_filename);
+		return;
+	}
+
+	/* Build the ind$file command */
+	op[0] = '\0';
+	if (ascii_flag)
+		strcat(op, " ascii");
+	if (ascii_flag && cr_flag)
+		strcat(op, " crlf");
+	if (append_flag && !receive_flag)
+		strcat(op, " append");
+	if (!receive_flag) {
+		if (!vm_flag) {
+			if (recfm != DEFAULT_RECFM) {
+				/* RECFM Entered, process */
+				strcat(op, " recfm(");
+				switch (recfm) {
+				    case FIXED:
+					strcat(op, "f");
+					break;
+				    case VARIABLE:
+					strcat(op, "v");
+					break;
+				    case UNDEFINED:
+					strcat(op, "u");
+					break;
+				    default:
+					break;
+				};
+				strcat(op, ")");
+				if (tp[PARM_LRECL].value != CN)
+					sprintf(eos(op), " lrecl(%s)",
+					    tp[PARM_LRECL].value);
+				if (tp[PARM_BLKSIZE].value != CN)
+					sprintf(eos(op), " blksize(%s)",
+					    tp[PARM_BLKSIZE].value);
+			}
+			if (units != DEFAULT_UNITS) {
+				/* Space Entered, processs it */
+				switch (units) {
+				    case TRACKS:
+					strcat(op, " tracks");
+					break;
+				    case CYLINDERS:
+					strcat(op, " cylinders");
+					break;
+				    case AVBLOCK:
+					strcat(op, " avblock");
+					break;
+				    default:
+					break;
+				};
+				if (tp[PARM_PRIMARY_SPACE].value != CN) {
+					sprintf(eos(op), " space(%s",
+					    tp[PARM_PRIMARY_SPACE].value);
+					if (tp[PARM_SECONDARY_SPACE].value)
+						sprintf(eos(op), ",%s",
+						    tp[PARM_SECONDARY_SPACE].value);
+					strcat(op, ")");
+				}
+			}
+		} else {
+			if (recfm != DEFAULT_RECFM) {
+				strcat(op, " recfm ");
+				switch (recfm) {
+				    case FIXED:
+					strcat(op, "f");
+					break;
+				    case VARIABLE:
+					strcat(op, "v");
+					break;
+				    default:
+					break;
+				};
+
+				if (tp[PARM_LRECL].value)
+					sprintf(eos(op), " lrecl %s",
+					    tp[PARM_LRECL].value);
+			}
+		}
+	}
+
+	/* Insert the '(' for VM options. */
+	if (strlen(op) > 0 && vm_flag) {
+		opts[0] = ' ';
+		opts[1] = '(';
+		op = opts;
+	}
+
+	/* Build the whole command. */
+	cmd = xs_buffer("%s %s %s%s\\n",
+	    appres.ft_command,
+	    receive_flag ? "get" : "put", ft_host_filename, op);
+
+	/* Erase the line and enter the command. */
+	flen = kybd_prime();
+	if (!flen || flen < strlen(cmd) - 1) {
+		Free(cmd);
+		popup_an_error(get_message("ftUnable"));
+		return;
+	}
+	(void) emulate_input(cmd, strlen(cmd), False);
+	Free(cmd);
+
+	/* Get this thing started. */
+	ft_state = FT_AWAIT_ACK;
+	ft_is_cut = False;
+
+	/* Start tracking it. */
+#if defined(X3270_DISPLAY) && defined(X3270_MENUS) /*[*/
+	popup_progress();
+#endif /*]*/
+}
+
+#endif /*]*/
