@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1999 by Paul Mattes.
+ * Copyright 1996, 1999, 2000, 2001 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -13,9 +13,12 @@
  */
 
 #include <errno.h>
-#include <string.h>
 
 #include "globals.h"
+
+#if defined(X3270_FT) /*[*/
+
+#include "appres.h"
 #include "ctlr.h"
 #include "3270ds.h"
 
@@ -31,7 +34,7 @@
 #include "trace_dsc.h"
 #include "utilc.h"
 
-Boolean cut_xfer_in_progress = False;
+static Boolean cut_xfer_in_progress = False;
 
 /* Data stream conversion tables. */
 
@@ -96,26 +99,24 @@ static int xlate_buffered = 0;			/* buffer count */
 static int xlate_buf_ix = 0;			/* buffer index */
 static unsigned char xlate_buf[XLATE_NBUF];	/* buffer */
 
-static void cut_control_code();
-static void cut_data_request();
-static void cut_retransmit();
-static void cut_data();
+static void cut_control_code(void);
+static void cut_data_request(void);
+static void cut_retransmit(void);
+static void cut_data(void);
 
-static void cut_ack();
-static void cut_abort();
+static void cut_ack(void);
+static void cut_abort(const char *s, unsigned short reason);
 
-static unsigned from6();
-static int xlate_getc();
+static unsigned from6(unsigned char c);
+static int xlate_getc(void);
 
 /*
  * Convert a buffer for uploading (host->local).  Overwrites the buffer.
  * Returns the length of the converted data.
  * If there is a conversion error, calls cut_abort() and returns -1.
  */
-int
-upload_convert(buf, len)
-unsigned char *buf;
-int len;
+static int
+upload_convert(unsigned char *buf, int len)
 {
 	unsigned char *ob0 = buf;
 	unsigned char *ob = ob0;
@@ -174,6 +175,8 @@ int len;
 		c = conv[quadrant].xlate[ix];
 		if (ascii_flag && cr_flag && (c == '\r' || c == 0x1a))
 			continue;
+		if (ascii_flag && remap_flag)
+			c = ft2asc[c];
 		*ob++ = c;
 	}
 
@@ -181,13 +184,10 @@ int len;
 }
 
 /* Convert a buffer for downloading (local->host). */
-int
-download_convert(buf, len, obuf)
-unsigned char *buf;
-int len;
-unsigned char *obuf;
+static int
+download_convert(unsigned const char *buf, unsigned len, unsigned char *xobuf)
 {
-	unsigned char *ob0 = obuf;
+	unsigned char *ob0 = xobuf;
 	unsigned char *ob = ob0;
 
 	while (len--) {
@@ -206,6 +206,10 @@ unsigned char *obuf;
 			continue;
 		}
 
+		/* Translate. */
+		if (ascii_flag && remap_flag)
+			c = asc2ft[c];
+
 		/* Quadrant already defined. */
 		if (quadrant >= 0) {
 			ixp = (unsigned char *)memchr(conv[quadrant].xlate, c,
@@ -245,7 +249,7 @@ unsigned char *obuf;
  * We have received what looks like an appropriate message from the host.
  */
 void
-ft_cut_data()
+ft_cut_data(void)
 {
 	switch (cg2ebc[screen_buf[O_FRAME_TYPE]]) {
 	    case FT_CONTROL_CODE:
@@ -272,7 +276,7 @@ ft_cut_data()
  * Process a control code from the host.
  */
 static void
-cut_control_code()
+cut_control_code(void)
 {
 	unsigned short code;
 	char *buf;
@@ -307,7 +311,7 @@ cut_control_code()
 			buf = saved_errmsg;
 			saved_errmsg = CN;
 		} else {
-			bp = buf = XtMalloc(81);
+			bp = buf = Malloc(81);
 			for (i = 0; i < 80; i++)
 				*bp++ = cg2asc[screen_buf[O_CC_MESSAGE + i]];
 			*bp-- = '\0';
@@ -321,6 +325,7 @@ cut_control_code()
 				strcpy(buf, get_message("ftHostCancel"));
 		}
 		ft_complete(buf);
+		Free(buf);
 		break;
 	    default:
 		trace_ds("unknown 0x%04x\n", code);
@@ -333,7 +338,7 @@ cut_control_code()
  * Process a data request from the host.
  */
 static void
-cut_data_request()
+cut_data_request(void)
 {
 	unsigned char seq = screen_buf[O_DR_FRAME_SEQ];
 	int count;
@@ -357,18 +362,18 @@ cut_data_request()
 
 	/* Check for errors. */
 	if (ferror(ft_local_file)) {
-		int i;
+		int j;
 		char *msg;
 
 		/* Clean out any data we may have written. */
-		for (i = 0; i < count; i++)
-			ctlr_add(O_UP_DATA + i, 0, 0);
+		for (j = 0; j < count; j++)
+			ctlr_add(O_UP_DATA + j, 0, 0);
 
 		/* Abort the transfer. */
 		msg = xs_buffer("read(%s): %s", ft_local_filename,
 		    strerror(errno));
 		cut_abort(msg, SC_ABORT_FILE);
-		XtFree(msg);
+		Free(msg);
 		return;
 	}
 
@@ -404,7 +409,7 @@ cut_data_request()
  * (Improperly) process a retransmit from the host.
  */
 static void
-cut_retransmit()
+cut_retransmit(void)
 {
 	trace_ds("< FT RETRANSMIT\n");
 	cut_abort(get_message("ftCutRetransmit"), SC_ABORT_XMIT);
@@ -414,8 +419,7 @@ cut_retransmit()
  * Convert an encoded integer.
  */
 static unsigned
-from6(c)
-unsigned char c;
+from6(unsigned char c)
 {
 	char *p;
 
@@ -430,7 +434,7 @@ unsigned char c;
  * Process data from the host.
  */
 static void
-cut_data()
+cut_data(void)
 {
 	static unsigned char cvbuf[O_RESPONSE - O_DT_DATA];
 	unsigned short raw_length;
@@ -469,7 +473,7 @@ cut_data()
 		msg = xs_buffer("write(%s): %s", ft_local_filename,
 		    strerror(errno));
 		cut_abort(msg, SC_ABORT_FILE);
-		XtFree(msg);
+		Free(msg);
 	} else {
 		ft_length += conv_length;
 		ft_update_length();
@@ -481,7 +485,7 @@ cut_data()
  * Acknowledge a host command.
  */
 static void
-cut_ack()
+cut_ack(void)
 {
 	trace_ds("> FT ACK\n");
 	action_internal(Enter_action, IA_FT, CN, CN);
@@ -491,14 +495,10 @@ cut_ack()
  * Abort a transfer in progress.
  */
 static void
-cut_abort(s, reason)
-char *s;
-unsigned short reason;
+cut_abort(const char *s, unsigned short reason)
 {
 	/* Save the error message. */
-	if (saved_errmsg != CN)
-		XtFree(saved_errmsg);
-	saved_errmsg = XtNewString(s);
+	Replace(saved_errmsg, NewString(s));
 
 	/* Send the abort sequence. */
 	ctlr_add(RO_FRAME_TYPE, ebc2cg[RFT_CONTROL_CODE], 0);
@@ -517,7 +517,7 @@ unsigned short reason;
  * Returns the character (in EBCDIC), or EOF.
  */
 static int
-xlate_getc()
+xlate_getc(void)
 {
 	int r;
 	int c;
@@ -541,7 +541,7 @@ xlate_getc()
 
 	/* Expand it. */
 	if (ascii_flag && cr_flag && c == '\n') {
-		nc = download_convert("\r", 1, cbuf);
+		nc = download_convert((unsigned const char *)"\r", 1, cbuf);
 	} else
 		nc = 0;
 
@@ -560,3 +560,5 @@ xlate_getc()
 	}
 	return r;
 }
+
+#endif /*]*/
