@@ -1,5 +1,5 @@
 /*
- * Copyright 1996 by Paul Mattes.
+ * Copyright 1996, 1999 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -7,4 +7,4 @@
  *  supporting documentation.
  */
 
-extern void ft_cut_data();
+extern void ft_cut_data(void);
