@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1996, 1999 by Paul Mattes.
+ * Modifications Copyright 1996, 1999, 2000, 2001 by Paul Mattes.
  * Copyright Octover 1995 by Dick Altenbern.
  * Based in part on code Copyright 1993, 1994, 1995 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
@@ -15,14 +15,18 @@
  */
 
 #include "globals.h"
-#include <string.h>
 
+#if defined(X3270_FT) /*[*/
+
+#include "appres.h"
 #include "3270ds.h"
 #include "ft_dft_ds.h"
+
 #include "actionsc.h"
 #include "kybdc.h"
 #include "ft_dftc.h"
 #include "ftc.h"
+#include "tablesc.h"
 #include "telnetc.h"
 #include "trace_dsc.h"
 #include "utilc.h"
@@ -61,7 +65,7 @@ struct error_response {			/* Buffer to build for an error resp. */
 	char sf_err_hdr[2];		/* 0x6904 */
 	char sf_err_code[2];		/* The error code */
 };
-#define UPLOAD_LENGTH 2048-2
+#define UPLOAD_LENGTH (DFT_INBUF - 2)
 struct upload_buffer_hdr {
 	char sf_length[2];		/* SF length */
 	char sf_d0;			/* 0xd0 */
@@ -80,25 +84,24 @@ struct upload_buffer {
 
 /* Statics. */
 static Boolean message_flag = False;	/* Open Request for msg received */
-static int at_eof;
+static int eof;
 static unsigned long recnum;
 static char *abort_string = CN;
 
-static void dft_abort();
-static void dft_close_request();
-static void dft_data_insert();
-static void dft_get_request();
-static void dft_insert_request();
-static void dft_open_request();
-static void dft_set_cur_req();
-static int filter_len();
+static void dft_abort(const char *s, unsigned short code);
+static void dft_close_request(void);
+static void dft_data_insert(struct data_buffer *data_bufr);
+static void dft_get_request(void);
+static void dft_insert_request(void);
+static void dft_open_request(struct open_buffer *open_buf);
+static void dft_set_cur_req(void);
+static int filter_len(char *s, register int len);
 
 /* Process a Transfer Data structured field from the host. */
 void
-ft_dft_data(data_bufr, length)
-struct data_buffer *data_bufr;
-int length;
+ft_dft_data(unsigned char *data, int length unused)
 {
+	struct data_buffer *data_bufr = (struct data_buffer *)data;
 	unsigned short data_type;
 	unsigned char *cp;
 
@@ -141,8 +144,7 @@ int length;
 
 /* Process an Open request. */
 static void
-dft_open_request(open_buf)
-struct open_buffer *open_buf;
+dft_open_request(struct open_buffer *open_buf)
 {
 	trace_ds(" Open %*.*s\n", 7, 7, open_buf->name);
 
@@ -152,7 +154,7 @@ struct open_buffer *open_buf;
 		message_flag = False;
 		ft_running(False);
 	}
-	at_eof = False;
+	eof = False;
 	recnum = 1;
 
 	trace_ds("> WriteStructuredField FileTransferData OpenAck\n");
@@ -167,7 +169,7 @@ struct open_buffer *open_buf;
 
 /* Process an Insert request. */
 static void
-dft_insert_request()
+dft_insert_request(void)
 {
 	trace_ds(" Insert\n");
 	/* Doesn't currently do anything. */
@@ -175,8 +177,7 @@ dft_insert_request()
 
 /* Process a Data Insert request. */
 static void
-dft_data_insert(data_bufr)
-struct data_buffer *data_bufr;
+dft_data_insert(struct data_buffer *data_bufr)
 {
 	/* Received a data buffer, get the length and process it */
 	int my_length;
@@ -207,7 +208,7 @@ struct data_buffer *data_bufr;
 		unsigned char *dollarp;
 
 		/* Get storage to copy the message. */
-		msgp = (unsigned char *)XtMalloc(my_length + 1);
+		msgp = (unsigned char *)Malloc(my_length + 1);
 
 		/* Copy the message. */
 		memcpy(msgp, data_bufr->data, my_length);
@@ -221,25 +222,37 @@ struct data_buffer *data_bufr;
 
 		/* If transfer completed ok, use our msg. */
 		if (memcmp(msgp, END_TRANSFER, strlen(END_TRANSFER)) == 0) {
-			XtFree(msgp);
+			Free(msgp);
 			ft_complete((String)NULL);
 		} else if (ft_state == FT_ABORT_SENT && abort_string != CN) {
-			XtFree(msgp);
+			Free(msgp);
 			ft_complete(abort_string);
-			abort_string = CN;
-		} else
-			ft_complete(msgp);
+			Replace(abort_string, CN);
+		} else {
+			ft_complete((char *)msgp);
+			Free(msgp);
+		}
 	} else if (my_length > 0) {
 		/* Write the data out to the file. */
 		int rv = 1;
 
+		if (ascii_flag && remap_flag) {
+			/* Filter. */
+			unsigned char *s = (unsigned char *)data_bufr->data;
+			unsigned len = my_length;
+
+			while (len--) {
+				*s = ft2asc[*s];
+				s++;
+			}
+		}
 		if (ascii_flag && cr_flag) {
 			char *s = (char *)data_bufr->data;
 			unsigned len = my_length;
 
 			/* Delete CRs and ^Zs. */
 			while (len) {
-				int l = filter_len(s, len);
+				unsigned l = filter_len(s, len);
 
 				if (l) {
 					rv = fwrite(s, l, (size_t)1,
@@ -267,7 +280,7 @@ struct data_buffer *data_bufr;
 			    strerror(errno));
 
 			dft_abort(buf, TR_DATA_INSERT);
-			XtFree(buf);
+			Free(buf);
 		}
 
 		/* Add up amount transferred. */
@@ -291,7 +304,7 @@ struct data_buffer *data_bufr;
 
 /* Process a Set Cursor request. */
 static void
-dft_set_cur_req()
+dft_set_cur_req(void)
 {
 	trace_ds(" SetCursor\n");
 	/* Currently doesn't do anything. */
@@ -299,7 +312,7 @@ dft_set_cur_req()
 
 /* Process a Get request. */
 static void
-dft_get_request()
+dft_get_request(void)
 {
 	int numbytes;
 	size_t numread;
@@ -315,9 +328,9 @@ dft_get_request()
 
 	/*
 	 * This is a request to send an upload buffer.
-	 * First check to see if we are finished (at_eof = True).
+	 * First check to see if we are finished (eof = True).
 	 */
-	if (at_eof) {
+	if (eof) {
 		/* We are done, send back the eof error. */
 		trace_ds("> WriteStructuredField FileTransferData EOF\n");
 		space3270out(sizeof(struct error_response) + 1);
@@ -360,6 +373,16 @@ dft_get_request()
 					    ft_local_file) != CN) {
 					/* We got a line. */
 
+					/* Filter. */
+					{
+						unsigned char *s = bufptr;
+
+						while (*s) {
+							*s = asc2ft[*s];
+							s++;
+						}
+					}
+
 					/* Decrement left. */
 					numbytes -= (strlen((char *)bufptr)+1);
 
@@ -389,6 +412,16 @@ dft_get_request()
 			    	/* Not crlf, do binary read. */
 				numread = fread(bufptr, 1, numbytes,
 				    ft_local_file);
+				if (ascii_flag && remap_flag) {
+					unsigned char *s = bufptr;
+					int i = numread;
+
+					while (i) {
+						*s = asc2ft[*s];
+						s++;
+						i--;
+					}
+				}
 				bufptr += numread;
 				numbytes -= numread;
 			}
@@ -396,13 +429,14 @@ dft_get_request()
 				/* End of file. */
 
 				/* Set that we are out of data. */
-				at_eof = True;
+				eof = True;
 				break;
 			} else if (ferror(ft_local_file)) {
 				char *buf;
 
 				buf = xs_buffer("read(%s): %s",
-				    ft_local_filename, strerror(errno));
+					ft_local_filename,
+					strerror(errno));
 				dft_abort(buf, TR_GET_REQ);
 			}
 		}
@@ -432,7 +466,7 @@ dft_get_request()
 
 /* Process a Close request. */
 static void
-dft_close_request()
+dft_close_request(void)
 {
 	/*
 	 * Recieved a close request from the system.
@@ -451,13 +485,9 @@ dft_close_request()
 
 /* Abort a transfer. */
 static void
-dft_abort(s, code)
-char *s;
-unsigned short code;
+dft_abort(const char *s, unsigned short code)
 {
-	if (abort_string != CN)
-		XtFree(abort_string);
-	abort_string = XtNewString(s);
+	Replace(abort_string, NewString(s));
 
 	trace_ds("> WriteStructuredField FileTransferData Error\n");
 
@@ -478,9 +508,7 @@ unsigned short code;
 
 /* Returns the number of bytes in s, limited by len, that aren't CRs or ^Zs. */
 static int
-filter_len(s, len)
-char *s;
-register int len;
+filter_len(char *s, register int len)
 {
 	register char *t = s;
 
@@ -490,3 +518,5 @@ register int len;
 	}
 	return t - s;
 }
+
+#endif /*]*/
