@@ -1,5 +1,5 @@
 /*
- * Modifications Copyright 1993, 1994, 1995, 1996, 1999 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
  * Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -13,43 +13,42 @@
  *		Common definitions for x3270.
  */
 
+/* Autoconf settings. */
+#include "conf.h"			/* autoconf settings */
+#if defined(X3270_TN3270E) && !defined(X3270_ANSI) /*[*/
+#define X3270_ANSI	1	/* RFC2355 requires NVT mode */
+#endif /*]*/
+
 /*
- * OS-specific #defines.
+ * OS-specific #defines.  Except for the blocking-connect workarounds, these
+ * should be replaced with autoconf probes as soon as possible.
  */
 
 /*
- * SEPARATE_SELECT_H
- *   The definitions of the data structures for select() are in <select.h>.
- * NO_SYS_TIME_H
- *   Don't #include <sys/time.h>.
- * NO_MEMORY_H
- *   Don't #include <memory.h>.
- * LOCAL_TELNET_H
- *   #include a local copy of "telnet.h" rather then <arpa/telnet.h>
- * SELECT_INT
- *   select() takes (int *) arguments rather than (fd_set *) arguments.
  * BLOCKING_CONNECT_ONLY
  *   Use only blocking sockets.
  */
 #if defined(sco) /*[*/
 #define BLOCKING_CONNECT_ONLY	1
-#define NO_SYS_TIME_H		1
-#endif /*]*/
-
-#if defined(_IBMR2) || defined(_SEQUENT_) || defined(__QNX__) /*[*/
-#define SEPARATE_SELECT_H	1
 #endif /*]*/
 
 #if defined(apollo) /*[*/
 #define BLOCKING_CONNECT_ONLY	1
-#define NO_MEMORY_H		1
 #endif /*]*/
 
-#if defined(hpux) /*[*/
-#define SELECT_INT		1
-#define LOCAL_TELNET_H		1
+/*
+ * Compiler-specific #defines.
+ */
+
+/* 'unused' explicitly flags an unused parameter */
+#if defined(__GNUC__) /*[*/
+#define unused __attribute__((__unused__))
+#else /*][*/
+#define unused /* nothing */
 #endif /*]*/
 
+
+
 /*
  * Prerequisite #includes.
  */
@@ -58,80 +57,117 @@
 #include <unistd.h>			/* Unix system calls */
 #include <ctype.h>			/* Character classes */
 #include <string.h>			/* String manipulations */
-#if !defined(NO_MEMORY_H) /*[*/
-#include <memory.h>			/* Block moves and compares */
-#endif /*]*/
 #include <sys/types.h>			/* Basic system data types */
-#if !defined(NO_SYS_TIME_H) /*[*/
 #include <sys/time.h>			/* System time-related data types */
+#include "localdefs.h"			/* {s,tcl,c}3270-specific defines */
+
+/*
+ * Cancel out contradictory parts.
+ */
+#if !defined(X3270_DISPLAY) /*[*/
+#undef X3270_KEYPAD
+#undef X3270_MENUS
+#endif /*]*/
+
+/* Local process (-e) header files. */
+#if defined(X3270_LOCAL_PROCESS) && defined(HAVE_LIBUTIL) /*[*/
+#define LOCAL_PROCESS	1
+#include <termios.h>
+#if defined(HAVE_PTY_H) /*[*/
+#include <pty.h>
+#endif /*]*/
+#if defined(HAVE_LIBUTIL_H) /*[*/
+#include <libutil.h>
+#endif /*]*/
+#if defined(HAVE_UTIL_H) /*[*/
+#include <util.h>
+#endif /*]*/
 #endif /*]*/
-#include <X11/Intrinsic.h>
 
 /* Simple global variables */
 
 extern int		COLS;
 extern int		ROWS;
+#if defined(X3270_DISPLAY) /*[*/
+extern Atom		a_3270, a_registry, a_encoding;
 extern XtAppContext	appcontext;
-extern Atom		a_delete_me;
-extern Atom		a_save_yourself;
-extern Atom		a_state;
-extern Pixel		colorbg_pixel;
+#endif /*]*/
+extern const char	*build;
+extern int		children;
+extern char		*connected_lu;
+extern char		*connected_type;
 extern char		*current_host;
 extern unsigned short	current_port;
 extern Boolean		*debugging_font;
-extern int		depth;
-extern Display		*display;
-extern XFontStruct      **efontinfo;
 extern char		*efontname;
-extern Boolean		error_popup_visible;
 extern Boolean		ever_3270;
 extern Boolean		exiting;
 extern Boolean		*extended_3270font;
+extern Boolean		*font_8bit;
 extern Boolean		flipped;
 extern char		*full_current_host;
+extern char		*full_efontname;
 extern char		full_model_name[];
-extern Pixmap		gray;
-extern Pixel		keypadbg_pixel;
-extern Boolean		*latin1_font;
+extern char		*funky_font;
+extern char		*hostname;
 extern char		luname[];
+#if defined(LOCAL_PROCESS) /*[*/
+extern Boolean		local_process;
+#endif /*]*/
 extern int		maxCOLS;
 extern int		maxROWS;
 extern char		*model_name;
 extern int		model_num;
+extern Boolean		non_tn3270e_host;
 extern int		ov_cols, ov_rows;
 extern Boolean		passthru_host;
-extern char		*programname;
-extern Boolean		reconnect_disabled;
-extern XrmDatabase	rdb;
-extern Window		root_window;
+extern const char	*programname;
+extern char		*reconnect_host;
+extern int		screen_depth;
 extern Boolean		scroll_initted;
 extern Boolean		shifted;
 extern Boolean		*standard_font;
 extern Boolean		std_ds_host;
 extern char		*termtype;
 extern Widget		toplevel;
-extern char		*user_icon_name;
-extern char		*user_title;
+
+#if defined(X3270_DISPLAY) /*[*/
+extern Atom		a_delete_me;
+extern Atom		a_save_yourself;
+extern Atom		a_state;
+extern Display		*display;
+extern Pixmap		gray;
+extern Pixel		keypadbg_pixel;
+extern XrmDatabase	rdb;
+extern Window		root_window;
+#endif /*]*/
 
 /* Data types and complex global variables */
 
 /*   connection state */
 enum cstate {
 	NOT_CONNECTED,		/* no socket, unknown mode */
+	RESOLVING,		/* resolving hostname */
 	PENDING,		/* connection pending */
-	CONNECTED_INITIAL,	/* connected in ANSI mode */
-	CONNECTED_3270		/* connected in 3270 mode */
+	CONNECTED_INITIAL,	/* connected, no mode yet */
+	CONNECTED_ANSI,		/* connected in NVT ANSI mode */
+	CONNECTED_3270,		/* connected in old-style 3270 mode */
+	CONNECTED_INITIAL_E,	/* connected in TN3270E mode, unnegotiated */
+	CONNECTED_NVT,		/* connected in TN3270E mode, NVT mode */
+	CONNECTED_SSCP,		/* connected in TN3270E mode, SSCP-LU mode */
+	CONNECTED_TN3270E	/* connected in TN3270E mode, 3270 mode */
 };
 extern enum cstate cstate;
 
-#define PCONNECTED	((int)cstate >= (int)PENDING)
-#define HALF_CONNECTED	(cstate == PENDING)
+#define PCONNECTED	((int)cstate >= (int)RESOLVING)
+#define HALF_CONNECTED	(cstate == RESOLVING || cstate == PENDING)
 #define CONNECTED	((int)cstate >= (int)CONNECTED_INITIAL)
-#define IN_3270		(cstate == CONNECTED_3270)
-#define IN_ANSI		(cstate == CONNECTED_INITIAL)
-
-/*   mouse-cursor state */
-enum mcursor_state { LOCKED, NORMAL, WAIT };
+#define IN_NEITHER	(cstate == CONNECTED_INITIAL)
+#define IN_ANSI		(cstate == CONNECTED_ANSI || cstate == CONNECTED_NVT)
+#define IN_3270		(cstate == CONNECTED_3270 || cstate == CONNECTED_TN3270E || cstate == CONNECTED_SSCP)
+#define IN_SSCP		(cstate == CONNECTED_SSCP)
+#define IN_TN3270E	(cstate == CONNECTED_TN3270E)
+#define IN_E		(cstate >= CONNECTED_INITIAL_E)
 
 /*   keyboard modifer bitmap */
 #define ShiftKeyDown	0x01
@@ -140,7 +176,7 @@ enum mcursor_state { LOCKED, NORMAL, WAIT };
 
 /*   toggle names */
 struct toggle_name {
-	char *name;
+	const char *name;
 	int index;
 };
 extern struct toggle_name toggle_names[];
@@ -163,6 +199,9 @@ struct ea {
 /*   translation lists */
 struct trans_list {
 	char			*name;
+	char			*pathname;
+	Boolean			is_temp;
+	Boolean			from_server;
 	struct trans_list	*next;
 };
 extern struct trans_list *trans_list;
@@ -179,25 +218,38 @@ extern int font_count;
 /*   input key type */
 enum keytype { KT_STD, KT_GE };
 
+/*   state changes */
+#define ST_RESOLVING	1
+#define ST_HALF_CONNECT	2
+#define ST_CONNECT	3
+#define ST_3270_MODE	4
+#define ST_LINE_MODE	5
+#define ST_REMODEL	6
+#define ST_PRINTER	7
+#define ST_EXITING	8
+#define ST_CHARSET	9
+#define N_ST		10
+
+/* Naming convention for private actions. */
+#define PA_PFX	"PA-"
+
 /* Shorthand macros */
 
 #define CN	((char *) NULL)
 #define PN	((XtPointer) NULL)
+#define Replace(var, value) { Free(var); var = (value); }
+
+/* Configuration change masks. */
+#define NO_CHANGE	0x0000	/* no change */
+#define MODEL_CHANGE	0x0001	/* screen dimensions changed */
+#define FONT_CHANGE	0x0002	/* emulator font changed */
+#define COLOR_CHANGE	0x0004	/* color scheme or 3278/9 mode changed */
+#define SCROLL_CHANGE	0x0008	/* scrollbar snapped on or off */
+#define CHARSET_CHANGE	0x0010	/* character set changed */
+#define ALL_CHANGE	0xffff	/* everything changed */
 
 /* Portability macros */
 
-/*   Replacement for memcpy that handles overlaps */
-
-#if XtSpecificationRelease >= 5 /*[*/
-#include <X11/Xfuncs.h>
-#undef MEMORY_MOVE
-#define MEMORY_MOVE(to,from,cnt)	bcopy(from,to,cnt)
-#else /*][*/
-#if !defined(MEMORY_MOVE) /*[*/
-extern char *MEMORY_MOVE();
-#endif /*]*/
-#endif /*]*/
-
 /*   Equivalent of setlinebuf */
 
 #if defined(_IOLBF) /*[*/
@@ -211,3 +263,9 @@ extern char *MEMORY_MOVE();
 #if defined(MOTOROLA)
 #define gettimeofday(tp,tz)	gettimeofday(tp)
 #endif
+
+/* DFT file transfer buffer sizes. */
+#if defined(X3270_FT) /*[*/
+#define DFT_INBUF	(16*1024)
+#define DFT_OUTBUF	(31*1024)
+#endif /*]*/
