@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995, 1996 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -14,6 +14,9 @@
  */
 
 #include "globals.h"
+
+#if defined(X3270_KEYPAD) /*[*/
+
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Command.h>
@@ -30,6 +33,8 @@
 
 #include "keypad.bm"
 
+enum kp_placement kp_placement;
+
 #define NUM_ROWS	4
 #define NUM_VROWS	15
 #define BORDER		1
@@ -61,15 +66,17 @@
  * Table of 3278 key labels and actions
  */
 struct button_list {
-	char *label;
-	char *name;
-	char *bits;
+	const char *label;
+	const char *name;
+	const char *bits;
 	int width;
 	int height;
-	void (*fn)();
-	char *parm;
+	XtActionProc action;
+	const char *parm;
 };
 
+Boolean keypad_changed = False;
+
 static char Lg[] = "large";
 static char Bm[] = "bm";
 static char Sm[] = "small";
@@ -239,31 +246,40 @@ static XtTranslations keypad_t0 = (XtTranslations) NULL;
 static XtTranslations saved_xt = (XtTranslations) NULL;
 
 
+/* Initialize the keypad placement from the keypad resource. */
+void
+keypad_placement_init(void)
+{
+	if (!strcmp(appres.keypad, KpLeft))
+		kp_placement = kp_left;
+	else if (!strcmp(appres.keypad, KpRight))
+		kp_placement = kp_right;
+	else if (!strcmp(appres.keypad, KpBottom))
+		kp_placement = kp_bottom;
+	else if (!strcmp(appres.keypad, KpIntegral))
+		kp_placement = kp_integral;
+	else
+		xs_error("Unknown value for %s", ResKeypad);
+}
+
 /*
  * Callback for keypad buttons.  Simply calls the function pointed to by the
  * client data.
  */
-/*ARGSUSED*/
 static void
-callfn(w, client_data, call_data)
-	Widget w;
-	XtPointer client_data;
-	XtPointer call_data;
+callfn(Widget w unused, XtPointer client_data, XtPointer call_data unused)
 {
 	struct button_list *keyd = (struct button_list *) client_data;
 
-	action_internal(keyd->fn, IA_KEYPAD, keyd->parm, CN);
+	action_internal(keyd->action, IA_KEYPAD, keyd->parm, CN);
 }
 
 /*
  * Create a button.
  */
 static Widget
-make_a_button(container, x, y, w, h, keyd)
-	Widget container;
-	Position x, y;
-	Dimension w, h;
-	struct button_list *keyd;
+make_a_button(Widget container, Position x, Position y, Dimension w,
+    Dimension h, struct button_list *keyd)
 {
 	Widget command;
 	Pixmap pixmap;
@@ -293,10 +309,9 @@ make_a_button(container, x, y, w, h, keyd)
  * Create the keys for a horizontal keypad
  */
 static void
-keypad_keys_horiz(container)
-	Widget container;
+keypad_keys_horiz(Widget container)
 {
-	int i;
+	unsigned i;
 	Position row, col;
 	Position x0, y0;
 
@@ -361,10 +376,9 @@ static Widget spf_container;
  * Create the keys for a vertical keypad.
  */
 static void
-keypad_keys_vert(container)
-	Widget container;
+keypad_keys_vert(Widget container)
 {
-	int i;
+	unsigned i;
 	Position row, col;
 	Position x0, y0;
 	Widget c1, c2;
@@ -445,24 +459,20 @@ keypad_keys_vert(container)
 }
 
 static Dimension
-get_keypad_dimension(name)
-char *name;
+get_keypad_dimension(const char *name)
 {
-	static char rname[64];
 	char *d;
 	long v;
 
-	(void) sprintf(rname, "%s.%s", ResKeypad, name);
-	if (!(d = get_resource(rname)))
+	if ((d = get_fresource("%s.%s", ResKeypad, name)) == CN)
 		xs_error("Cannot find %s resource", ResKeypad);
 	if ((v = strtol(d, (char **)0, 0)) <= 0)
 		xs_error("Illegal %s resource", ResKeypad);
-	XtFree(d);
 	return (Dimension)v;
 }
 
 static void
-init_keypad_dimensions()
+init_keypad_dimensions(void)
 {
 	static Boolean done = False;
 
@@ -477,14 +487,14 @@ init_keypad_dimensions()
 }
 
 Dimension
-min_keypad_width()
+min_keypad_width(void)
 {
 	init_keypad_dimensions();
 	return HORIZ_WIDTH;
 }
 
 Dimension
-keypad_qheight()
+keypad_qheight(void)
 {
 	init_keypad_dimensions();
 	return TOP_MARGIN +
@@ -496,14 +506,9 @@ keypad_qheight()
  * Create a keypad.
  */
 Widget
-keypad_init(container, voffset, screen_width, floating, vert)
-	Widget container;
-	Dimension voffset;
-	Dimension screen_width;
-	Boolean floating;
-	Boolean vert;
+keypad_init(Widget container, Dimension voffset, Dimension screen_width, Boolean floating, Boolean vert)
 {
-	Widget key_pad;
+	static Widget key_pad;
 	Dimension height;
 	Dimension width = screen_width;
 	Dimension hoffset;
@@ -530,24 +535,33 @@ keypad_init(container, voffset, screen_width, floating, vert)
 		hoffset = 0;
 	if (voffset & 1)
 		voffset++;
-	key_pad = XtVaCreateManagedWidget(
-		"keyPad", compositeWidgetClass, container,
-		XtNx, hoffset,
-		XtNy, voffset,
-		XtNborderWidth, (Dimension) (floating ? 1 : 0),
-		XtNwidth, width,
-		XtNheight, height,
-		NULL);
-	if (appres.mono)
-		XtVaSetValues(key_pad, XtNbackgroundPixmap, gray, NULL);
-	else
-		XtVaSetValues(key_pad, XtNbackground, keypadbg_pixel, NULL);
+	if (key_pad == (Widget)NULL) {
+		key_pad = XtVaCreateManagedWidget(
+		    "keyPad", compositeWidgetClass, container,
+		    XtNx, hoffset,
+		    XtNy, voffset,
+		    XtNborderWidth, (Dimension) (floating ? 1 : 0),
+		    XtNwidth, width,
+		    XtNheight, height,
+		    NULL);
+		if (appres.mono)
+			XtVaSetValues(key_pad, XtNbackgroundPixmap, gray,
+			    NULL);
+		else
+			XtVaSetValues(key_pad, XtNbackground, keypadbg_pixel,
+			    NULL);
 
-	/* Create the keys */
-	if (vert)
-		keypad_keys_vert(key_pad);
-	else
-		keypad_keys_horiz(key_pad);
+		/* Create the keys */
+		if (vert)
+			keypad_keys_vert(key_pad);
+		else
+			keypad_keys_horiz(key_pad);
+	} else {
+		XtVaSetValues(key_pad,
+		    XtNx, hoffset,
+		    XtNy, voffset,
+		    NULL);
+	}
 
 	return key_pad;
 }
@@ -557,7 +571,7 @@ keypad_init(container, voffset, screen_width, floating, vert)
  * unmapping the window containing the shifted keys.
  */
 void
-keypad_shift()
+keypad_shift(void)
 {
 	if (!vert_keypad || !XtIsRealized(spf_container))
 		return;
@@ -574,18 +588,18 @@ keypad_shift()
  */
 Widget keypad_shell = NULL;
 Boolean keypad_popped = False;
-Boolean up_once = False;
+static Boolean up_once = False;
 static Position kset_x, kset_y;
 static Position kmove_x, kmove_y;
 static Dimension decor_width, decor_height;
-enum { WA_UNKNOWN, WA_ALL, WA_PARTIAL, WA_NONE } wa_mode = WA_UNKNOWN;
+static enum { WA_UNKNOWN, WA_ALL, WA_PARTIAL, WA_NONE } wa_mode = WA_UNKNOWN;
 
 /*
  * Called when the main screen is first exposed, to pop up the keypad the
  * first time
  */
 void
-keypad_first_up()
+keypad_first_up(void)
 {
 	if (!appres.keypad_on || kp_placement == kp_integral || up_once)
 		return;
@@ -594,12 +608,9 @@ keypad_first_up()
 }
 
 /* Called when the keypad popup pops up or down */
-/*ARGSUSED*/
 static void
-keypad_updown(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+keypad_updown(Widget w unused, XtPointer client_data,
+		XtPointer call_data unused)
 {
 	appres.keypad_on = keypad_popped = *(Boolean *)client_data;
 	if (keypad_popped) {
@@ -619,11 +630,11 @@ static enum placement *pp;
 
 /* Create the pop-up keypad */
 void
-keypad_popup_init()
+keypad_popup_init(void)
 {
 	Widget w;
 	Dimension height, width, border;
-	Boolean vert;
+	Boolean vert = False;
 
 	if (keypad_shell != NULL)
 		return;
@@ -703,8 +714,7 @@ keypad_popup_init()
 
 /* Set a temporary keymap. */
 void
-keypad_set_temp_keymap(trans)
-XtTranslations trans;
+keypad_set_temp_keymap(XtTranslations trans)
 {
 	if (keypad_container != (Widget) NULL) {
 		if (trans == (XtTranslations)NULL) {
@@ -719,7 +729,7 @@ XtTranslations trans;
 
 /* Change the baseleve keymap. */
 void
-keypad_set_keymap()
+keypad_set_keymap(void)
 {
 	if (keypad_container == (Widget)NULL)
 		return;
@@ -728,9 +738,7 @@ keypad_set_keymap()
 }
 
 static void
-measure_move(closure, id)
-XtPointer closure;
-XtIntervalId *id;
+measure_move(XtPointer closure unused, XtIntervalId *id unused)
 {
 	Position x, y;
 
@@ -770,7 +778,7 @@ XtIntervalId *id;
 
 /* Move the keypad. */
 void
-keypad_move()
+keypad_move(void)
 {
 	Position x, y;
 	Dimension w, h;
@@ -861,11 +869,8 @@ keypad_move()
  * window manager decorations and stores them in decor_width and decor_height.
  */
 void
-ReparentNotify_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+PA_ReparentNotify_action(Widget w unused, XEvent *event, String *params unused,
+    Cardinal *num_params unused)
 {
 	XReparentEvent *e = (XReparentEvent *)event;
 
@@ -879,3 +884,5 @@ Cardinal *num_params;
 		}
 	}
 }
+
+#endif /*]*/
