@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995, 1999 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000, 2001 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -14,7 +14,6 @@
  */
 
 #include "globals.h"
-#include <string.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/Command.h>
@@ -22,22 +21,22 @@
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Label.h>
 #include <X11/Xaw/Text.h>
-#if defined(__STDC__)
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 #include "objects.h"
+#include "appres.h"
 
 #include "actionsc.h"
+#include "hostc.h"
 #include "macrosc.h"
-#include "mainc.h"
 #include "popupsc.h"
 #include "screenc.h"
 #include "utilc.h"
+#include "xioc.h"
 
 static char vmsgbuf[4096];
 
+static enum form_type forms[] = { FORM_NO_WHITE, FORM_NO_CC, FORM_AS_IS };
+
 
 /*
  * General popup support
@@ -45,14 +44,13 @@ static char vmsgbuf[4096];
 
 /* Find the parent of a window */
 static Window
-parent_of(w)
-Window w;
+parent_of(Window w)
 {
-	Window root, parent, *children;
+	Window root, parent, *wchildren;
 	unsigned int nchildren;
 
-	XQueryTree(display, w, &root, &parent, &children, &nchildren);
-	XFree((char *) children);
+	XQueryTree(display, w, &root, &parent, &wchildren, &nchildren);
+	XFree((char *)wchildren);
 	return parent;
 }
 
@@ -61,8 +59,8 @@ Window w;
  * root, so we can pop up a window relative to them.
  */
 void
-toplevel_geometry(x, y, width, height)
-Dimension *x, *y, *width, *height;
+toplevel_geometry(Position *x, Position *y, Dimension *width,
+    Dimension *height)
 {
 	Window tlw = XtWindow(toplevel);
 	Window win;
@@ -116,9 +114,7 @@ Dimension *x, *y, *width, *height;
 
 /* Pop up a popup shell */
 void
-popup_popup(shell, grab)
-Widget shell;
-XtGrabKind grab;
+popup_popup(Widget shell, XtGrabKind grab)
 {
 	XtPopup(shell, grab);
 	XSetWMProtocols(display, XtWindow(shell), &a_delete_me, 1);
@@ -134,19 +130,15 @@ static enum placement RightD = Right;
 enum placement *RightP = &RightD;
 
 /* Place a popped-up shell */
-/*ARGSUSED*/
 void
-place_popup(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+place_popup(Widget w, XtPointer client_data, XtPointer call_data unused)
 {
 	Dimension width, height;
-	Dimension x = 0, y = 0;
+	Position x = 0, y = 0;
+	Position xnew, ynew;
 	Dimension win_width, win_height;
 	Dimension popup_width, popup_height;
 	enum placement p = *(enum placement *)client_data;
-	extern Dimension main_width;
 
 	/* Get and fix the popup's dimensions */
 	XtRealizeWidget(w);
@@ -180,9 +172,15 @@ XtPointer call_data;
 		    XtNwidth, &popup_width,
 		    XtNheight, &popup_height,
 		    NULL);
+		xnew = x + (win_width-popup_width) / (unsigned) 2;
+		if (xnew < 0)
+			xnew = 0;
+		ynew = y + (win_height-popup_height) / (unsigned) 2;
+		if (ynew < 0)
+			ynew = 0;
 		XtVaSetValues(w,
-		    XtNx, x + (win_width-popup_width) / (unsigned) 2,
-		    XtNy, y + (win_height-popup_height) / (unsigned) 2,
+		    XtNx, xnew,
+		    XtNy, ynew,
 		    NULL);
 		break;
 	    case Bottom:
@@ -211,16 +209,14 @@ XtPointer call_data;
 
 /* Action called when "Return" is pressed in data entry popup */
 void
-confirm_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+PA_confirm_action(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
 	Widget w2;
 
 	/* Find the Confirm or Okay button */
 	w2 = XtNameToWidget(XtParent(w), ObjConfirmButton);
+	if (w2 == NULL)
+		w2 = XtNameToWidget(XtParent(w), ObjConfirmButton);
 	if (w2 == NULL)
 		w2 = XtNameToWidget(w, ObjConfirmButton);
 	if (w2 == NULL) {
@@ -235,12 +231,9 @@ Cardinal *num_params;
 }
 
 /* Callback for "Cancel" button in data entry popup */
-/*ARGSUSED*/
 static void
-cancel_button_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+cancel_button_callback(Widget w unused, XtPointer client_data,
+    XtPointer call_data unused)
 {
 	XtPopdown((Widget) client_data);
 }
@@ -249,12 +242,9 @@ XtPointer call_data;
  * Callback for text source changes.  Ensures that the dialog text does not
  * contain white space -- especially newlines.
  */
-/*ARGSUSED*/
 static void
-popup_dialog_callback(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+popup_dialog_callback(Widget w, XtPointer client_data,
+    XtPointer call_data unused)
 {
 	static Boolean called_back = False;
 	XawTextBlock b, nullb;	/* firstPos, length, ptr, format */
@@ -264,6 +254,10 @@ XtPointer call_data;
 	int end_pos = 0;
 	int i;
 	enum { FRONT, MIDDLE, END } place = FRONT;
+	enum form_type *ftp = (enum form_type *)client_data;
+
+	if (*ftp == FORM_AS_IS)
+		return;
 
 	if (called_back)
 		return;
@@ -287,8 +281,11 @@ XtPointer call_data;
 			end_len += b.length;
 			continue;
 		}
-		for (i = 0; i < b.length; i++)
-			if (isspace(*(b.ptr + i))) {
+		for (i = 0; i < b.length; i++) {
+			char c;
+
+			c = *(b.ptr + i);
+			if (isspace(c) && (*ftp != FORM_NO_CC || c != ' ')) {
 				if (place == FRONT) {
 					front_len++;
 					continue;
@@ -300,6 +297,7 @@ XtPointer call_data;
 				}
 			} else
 				place = MIDDLE;
+		}
 		pos += b.length;
 		if (b.length < 1024)
 			break;
@@ -314,11 +312,8 @@ XtPointer call_data;
 
 /* Create a simple data entry popup */
 Widget
-create_form_popup(name, callback, callback2, no_spaces)
-char *name;
-XtCallbackProc callback;
-XtCallbackProc callback2;
-Boolean no_spaces;
+create_form_popup(const char *name, XtCallbackProc callback,
+    XtCallbackProc callback2, enum form_type form_type)
 {
 	char *widgetname;
 	Widget shell;
@@ -334,7 +329,7 @@ Boolean no_spaces;
 	    widgetname, transientShellWidgetClass, toplevel,
 	    NULL);
 	XtFree(widgetname);
-	XtAddCallback(shell, XtNpopupCallback, place_popup, (XtPointer) CenterP);
+	XtAddCallback(shell, XtNpopupCallback, place_popup, (XtPointer)CenterP);
 
 	/* Create a dialog in the popup */
 
@@ -346,7 +341,6 @@ Boolean no_spaces;
 	    NULL);
 
 	/* Add "Confirm" and "Cancel" buttons to the dialog */
-
 	w = XtVaCreateManagedWidget(
 	    ObjConfirmButton, commandWidgetClass, dialog,
 	    NULL);
@@ -362,7 +356,7 @@ Boolean no_spaces;
 	    NULL);
 	XtAddCallback(w, XtNcallback, cancel_button_callback, (XtPointer) shell);
 
-	if (!no_spaces)
+	if (form_type == FORM_AS_IS)
 		return shell;
 
 	/* Modify the translations for the objects in the dialog */
@@ -376,140 +370,248 @@ Boolean no_spaces;
 	if (w == NULL)
 		XtWarning("Cannot find text source in dialog");
 	else
-		XtAddCallback(w, XtNcallback, popup_dialog_callback, NULL);
+		XtAddCallback(w, XtNcallback, popup_dialog_callback,
+		    &forms[(int)form_type]);
 
 	return shell;
 }
 
 
 /*
- * Error popup
+ * Read-only popups.
  */
+struct rsm {
+	struct rsm *next;
+	char *text;
+};
+struct rop {
+	const char *name;			/* resource name */
+	XtGrabKind grab;			/* grab kind */
+	Boolean is_error;			/* is it? */
+	Boolean overwrites;			/* does it? */
+	const char *itext;			/* initial text */
+	Widget shell;				/* pop-up shell */
+	Widget form;				/* dialog form */
+	Widget cancel_button;			/* cancel button */
+	abort_callback_t *cancel_callback;	/* callback for cancel button */
+	Boolean visible;			/* visibility flag */
+	struct rsm *rsms;			/* stored messages */
+};
 
-static Widget error_shell = NULL;
-static Widget error_form;
-Boolean error_popup_visible;
+static struct rop error_popup = {
+	"errorPopup", XtGrabExclusive, True, False,
+	"first line\nsecond line\nthird line",
+	NULL, NULL, NULL, NULL,
+	False, NULL
+};
+static struct rop info_popup = {
+	"infoPopup", XtGrabNonexclusive, False, False,
+	"first line\nsecond line\nthird line",
+	NULL, NULL, NULL, NULL,
+	False, NULL
+};
 
-/* Called when OK is pressed on the error popup */
-/*ARGSUSED*/
+static struct rop printer_error_popup = {
+	"printerErrorPopup", XtGrabExclusive, True, True,
+	"first line\nsecond line\nthird line\nfourth line",
+	NULL, NULL, NULL, NULL, False, NULL
+};
+static struct rop printer_info_popup = {
+	"printerInfoPopup", XtGrabNonexclusive, False, True,
+	"first line\nsecond line\nthird line\nfourth line",
+	NULL,
+	NULL, NULL, NULL, False, NULL
+};
+
+static struct rop child_error_popup = {
+	"childErrorPopup", XtGrabNonexclusive, True, True,
+	"first line\nsecond line\nthird line\nfourth line",
+	NULL, NULL, NULL, NULL, False, NULL
+};
+static struct rop child_info_popup = {
+	"childInfoPopup", XtGrabNonexclusive, False, True,
+	"first line\nsecond line\nthird line\nfourth line",
+	NULL,
+	NULL, NULL, NULL, False, NULL
+};
+
+/* Called when OK is pressed in a read-only popup */
 static void
-saw_error(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+rop_ok(Widget w unused, XtPointer client_data, XtPointer call_data unused)
 {
-	XtPopdown(error_shell);
+	struct rop *rop = (struct rop *)client_data;
+	struct rsm *r;
+
+	if ((r = rop->rsms) != NULL) {
+		XtVaSetValues(rop->form, XtNlabel, r->text, NULL);
+		rop->rsms = r->next;
+		Free(r->text);
+		Free(r);
+	} else {
+		XtPopdown(rop->shell);
+	}
 }
 
-/* Called when the error popup is closed */
-/*ARGSUSED*/
+/* Called when Cancel is pressed in a read-only popup */
 static void
-error_popdown(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
+rop_cancel(Widget w unused, XtPointer client_data, XtPointer call_data unused)
 {
-	error_popup_visible = False;
-	if (exiting)
+	struct rop *rop = (struct rop *)client_data;
+
+	XtPopdown(rop->shell);
+	if (rop->cancel_callback != NULL)
+		(*rop->cancel_callback)();
+}
+
+/* Called when a read-only popup is closed */
+static void
+rop_popdown(Widget w unused, XtPointer client_data, XtPointer call_data unused)
+{
+	struct rop *rop = (struct rop *)client_data;
+
+	rop->visible = False;
+	if (exiting && rop->is_error)
 		x3270_exit(1);
 }
 
-void
-error_popup_init()
+/* Initialize a read-only pop-up. */
+static void
+rop_init(struct rop *rop)
 {
 	Widget w;
+	struct rsm *r;
 
-	if (error_shell != NULL)
+	if (rop->shell != NULL)
 		return;
 
-	error_shell = XtVaCreatePopupShell(
-	    "errorPopup", transientShellWidgetClass, toplevel,
+	rop->shell = XtVaCreatePopupShell(
+	    rop->name, transientShellWidgetClass, toplevel,
 	    NULL);
-	XtAddCallback(error_shell, XtNpopupCallback, place_popup,
+	XtAddCallback(rop->shell, XtNpopupCallback, place_popup,
 	    (XtPointer) CenterP);
-	XtAddCallback(error_shell, XtNpopdownCallback, error_popdown, PN);
+	XtAddCallback(rop->shell, XtNpopdownCallback, rop_popdown, rop);
 
 	/* Create a dialog in the popup */
-
-	error_form = XtVaCreateManagedWidget(
-	    "dialog", dialogWidgetClass, error_shell,
+	rop->form = XtVaCreateManagedWidget(
+	    ObjDialog, dialogWidgetClass, rop->shell,
 	    NULL);
-	XtVaSetValues(XtNameToWidget(error_form, XtNlabel),
-	    XtNlabel, "first line\nsecond line",
+	XtVaSetValues(XtNameToWidget(rop->form, XtNlabel),
+	    XtNlabel, rop->itext,
 	    NULL);
 
 	/* Add "OK" button to the dialog */
-
 	w = XtVaCreateManagedWidget(
-	    ObjConfirmButton, commandWidgetClass, error_form,
+	    ObjConfirmButton, commandWidgetClass, rop->form,
 	    NULL);
-	XtAddCallback(w, XtNcallback, saw_error, 0);
+	XtAddCallback(w, XtNcallback, rop_ok, rop);
 
-	/* Force it into existence so it sizes itself with 2-line text */
+	/* Add an unmapped "Cancel" button to the dialog */
+	rop->cancel_button = XtVaCreateManagedWidget(
+	    ObjCancelButton, commandWidgetClass, rop->form,
+	    XtNright, w,
+	    XtNmappedWhenManaged, False,
+	    NULL);
+	XtAddCallback(rop->cancel_button, XtNcallback, rop_cancel, rop);
 
-	XtRealizeWidget(error_shell);
+	/* Force it into existence so it sizes itself with 4-line text */
+	XtRealizeWidget(rop->shell);
+
+	/* If there's a pending message, pop it up now. */
+	if ((r = rop->rsms) != NULL) {
+		if (rop->is_error)
+			popup_an_error("%s", r->text);
+		else
+			popup_an_info("%s", r->text);
+		rop->rsms = r->next;
+		Free(r->text);
+		Free(r);
+	}
 }
 
-/* Pop up an error dialog. */
-void
-#if defined(__STDC__)
-popup_an_error(char *fmt, ...)
-#else
-popup_an_error(va_alist)
-va_dcl
-#endif
+/* Pop up a dialog.  Common logic for all forms. */
+static void
+popup_rop(struct rop *rop, abort_callback_t *a, const char *fmt, va_list args)
 {
-	va_list args;
-
-#if defined(__STDC__)
-	va_start(args, fmt);
-#else
-	char *fmt;
-	va_start(args);
-	fmt = va_arg(args, char *);
-#endif
-
 	(void) vsprintf(vmsgbuf, fmt, args);
-	if (!error_shell) {
+	if (!rop->shell || (rop->visible && !rop->overwrites)) {
+#if 0
+		char *nl;
+
+		while ((nl = strchr(vmsgbuf, '\n')) != CN) {
+			*nl = ' ';
+		}
 		(void) fprintf(stderr, "%s: %s\n", programname, vmsgbuf);
-		exit(1);
+		if (rop->is_error)
+			exit(1);
+		else
+			return;
+#endif
+		struct rsm *r, **s;
+
+		r = (struct rsm *)Malloc(sizeof(struct rsm));
+		r->text = NewString(vmsgbuf);
+		r->next = NULL;
+		for (s = &rop->rsms; *s != NULL; s = &(*s)->next)
+			;
+		*s = r;
+		return;
 	}
 
-	if (sms_redirect()) {
+	if (rop->is_error && sms_redirect()) {
 		sms_error(vmsgbuf);
 		return;
 	}
 
-	XtVaSetValues(error_form, XtNlabel, vmsgbuf, NULL);
-	if (!error_popup_visible) {
-		ring_bell();
-		error_popup_visible = True;
-		popup_popup(error_shell, XtGrabExclusive);
+	XtVaSetValues(rop->form, XtNlabel, vmsgbuf, NULL);
+	if (a != NULL)
+		XtMapWidget(rop->cancel_button);
+	else
+		XtUnmapWidget(rop->cancel_button);
+	rop->cancel_callback = a;
+	if (!rop->visible) {
+		if (rop->is_error)
+			ring_bell();
+		rop->visible = True;
+		popup_popup(rop->shell, rop->grab);
+	}
+}
+
+static void
+error_exit(void)
+{
+	x3270_exit(0);
+}
+
+/* Pop up an error dialog. */
+void
+popup_an_error(const char *fmt, ...)
+{
+	va_list args;
+
+	va_start(args, fmt);
+	popup_rop(&error_popup, appres.reconnect? error_exit: NULL, fmt, args);
+	va_end(args);
+}
+
+/* Pop down an error dialog. */
+void
+popdown_an_error(void)
+{
+	if (error_popup.visible) {
+		XtPopdown(error_popup.shell);
 	}
 }
 
 /* Pop up an error dialog, based on an error number. */
 void
-#if defined(__STDC__)
-popup_an_errno(int errn, char *fmt, ...)
-#else
-popup_an_errno(va_alist)
-va_dcl
-#endif
+popup_an_errno(int errn, const char *fmt, ...)
 {
 	va_list args;
 	char *s;
 
-#if defined(__STDC__)
 	va_start(args, fmt);
-#else
-	int errn;
-	char *fmt;
-	va_start(args);
-	errn = va_arg(args, int);
-	fmt = va_arg(args, char *);
-#endif
 	(void) vsprintf(vmsgbuf, fmt, args);
+	va_end(args);
 	s = XtNewString(vmsgbuf);
 
 	if (errn > 0)
@@ -519,116 +621,143 @@ va_dcl
 	XtFree(s);
 }
 
-
-/*
- * Info popup
- */
-
-static Widget info_shell = NULL;
-static Widget info_form;
-Boolean info_popup_visible;
-
-/* Called when OK is pressed on the info popup */
-/*ARGSUSED*/
-static void
-saw_info(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
-{
-	XtPopdown(info_shell);
-}
-
-/* Called when the info popup is closed */
-/*ARGSUSED*/
-static void
-info_popdown(w, client_data, call_data)
-Widget w;
-XtPointer client_data;
-XtPointer call_data;
-{
-	info_popup_visible = False;
-}
-
-void
-info_popup_init()
-{
-	Widget w;
-
-	if (info_shell != NULL)
-		return;
-
-	info_shell = XtVaCreatePopupShell(
-	    "infoPopup", transientShellWidgetClass, toplevel,
-	    NULL);
-	XtAddCallback(info_shell, XtNpopupCallback, place_popup,
-	    (XtPointer) CenterP);
-	XtAddCallback(info_shell, XtNpopdownCallback, info_popdown, PN);
-
-	/* Create a dialog in the popup */
-
-	info_form = XtVaCreateManagedWidget(
-	    "dialog", dialogWidgetClass, info_shell,
-	    NULL);
-	XtVaSetValues(XtNameToWidget(info_form, XtNlabel),
-	    XtNlabel, "first line\nsecond line",
-	    NULL);
-
-	/* Add "OK" button to the dialog */
-
-	w = XtVaCreateManagedWidget(
-	    ObjConfirmButton, commandWidgetClass, info_form,
-	    NULL);
-	XtAddCallback(w, XtNcallback, saw_info, 0);
-
-	/* Force it into existence so it sizes itself with 2-line text */
-
-	XtRealizeWidget(info_shell);
-}
-
 /* Pop up an info dialog. */
 void
-#if defined(__STDC__)
-popup_an_info(char *fmt, ...)
-#else
-popup_an_info(va_alist)
-va_dcl
-#endif
+popup_an_info(const char *fmt, ...)
 {
 	va_list args;
 
-#if defined(__STDC__)
 	va_start(args, fmt);
-#else
-	char *fmt;
-	va_start(args);
-	fmt = va_arg(args, char *);
-#endif
+	popup_rop(&info_popup, NULL, fmt, args);
+	va_end(args);
+}
 
-	(void) vsprintf(vmsgbuf, fmt, args);
-	if (!info_shell) {
-		(void) printf("%s: %s\n", programname, vmsgbuf);
-		return;
+/*
+ * Produce a result of some sort.  If there is a script running, return it
+ * as the value; otherwise, pop it up as an info.
+ */
+void
+action_output(const char *fmt, ...)
+{
+	va_list args;
+
+	va_start(args, fmt);
+	if (sms_redirect()) {
+		char buf[4096];
+
+		(void) vsprintf(buf, fmt, args);
+		sms_info("%s", buf);
+	} else
+		popup_rop(&info_popup, NULL, fmt, args);
+	va_end(args);
+}
+
+/* Callback for x3270 exit.  Dumps any undisplayed error messages to stderr. */
+static void
+dump_errmsgs(Boolean b unused)
+{
+	while (error_popup.rsms != NULL) {
+		fprintf(stderr, "Error: %s\n", error_popup.rsms->text);
+		error_popup.rsms = error_popup.rsms->next;
 	}
-
-	XtVaSetValues(info_form, XtNlabel, vmsgbuf, NULL);
-	if (!info_popup_visible) {
-		info_popup_visible = True;
-		popup_popup(info_shell, XtGrabNonexclusive);
+	while (info_popup.rsms != NULL) {
+		fprintf(stderr, "%s\n", info_popup.rsms->text);
+		info_popup.rsms = info_popup.rsms->next;
 	}
 }
 
+/* Initialization. */
+
+void
+error_init(void)
+{
+	register_schange(ST_EXITING, dump_errmsgs);
+}
+
+void
+error_popup_init(void)
+{
+	rop_init(&error_popup);
+}
+
+void
+info_popup_init(void)
+{
+	rop_init(&info_popup);
+}
+
+void
+printer_popup_init(void)
+{
+	if (printer_error_popup.shell != NULL)
+		return;
+	rop_init(&printer_error_popup);
+	rop_init(&printer_info_popup);
+}
+
+void
+child_popup_init(void)
+{
+	if (child_error_popup.shell != NULL)
+		return;
+	rop_init(&child_error_popup);
+	rop_init(&child_info_popup);
+}
+
+/* Query. */
+Boolean
+error_popup_visible(void)
+{
+	return error_popup.visible;
+}
+
+/*
+ * Printer pop-up.
+ * Allows both error and info popups, and a cancel button.
+ *   is_err	If True, this is an error pop-up.  If false, this is an info
+ *		pop-up.
+ *   a		If non-NULL, the Cancel button is enabled, and this is the
+ *		callback function for it.  If NULL, there will be no Cancel
+ *		button.
+ *   fmt...	printf()-like format and arguments.
+ */
+void
+popup_printer_output(Boolean is_err, abort_callback_t *a, const char *fmt, ...)
+{
+	va_list args;
+
+	va_start(args, fmt);
+	popup_rop(is_err? &printer_error_popup: &printer_info_popup, a, fmt,
+	    args);
+	va_end(args);
+}
+
+/*
+ * Child output pop-up.
+ * Allows both error and info popups, and a cancel button.
+ *   is_err	If True, this is an error pop-up.  If false, this is an info
+ *		pop-up.
+ *   a		If non-NULL, the Cancel button is enabled, and this is the
+ *		callback function for it.  If NULL, there will be no Cancel
+ *		button.
+ *   fmt...	printf()-like format and arguments.
+ */
+void
+popup_child_output(Boolean is_err, abort_callback_t *a, const char *fmt, ...)
+{
+	va_list args;
+
+	va_start(args, fmt);
+	popup_rop(is_err? &child_error_popup: &child_info_popup, a, fmt, args);
+	va_end(args);
+}
 
 /*
  * Script actions
  */
-/*ARGSUSED*/
 void
-Info_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+Info_action(Widget w unused, XEvent *event, String *params,
+    Cardinal *num_params)
 {
 	action_debug(Info_action, event, params, num_params);
 	if (check_usage(Info_action, *num_params, 1, 1) < 0)
