@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000, 2001 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -56,6 +56,9 @@
 #include "tablesc.h"
 #include "utilc.h"
 
+#define Max(x, y)	(((x) > (y))? (x): (y))
+#define Min(x, y)	(((x) < (y))? (x): (y))
+
 /*
  * Mouse side.
  */
@@ -74,7 +77,7 @@ static Dimension down1_x, down1_y;
 static unsigned long up_time = 0;
 static int      saw_motion = 0;
 static int      num_clicks = 0;
-static void     grab_sel();
+static void grab_sel(int start, int end, Boolean really, Time t);
 #define NS		5
 static Atom     want_sel[NS];
 static struct {			/* owned selections */
@@ -83,9 +86,9 @@ static struct {			/* owned selections */
 }               own_sel[NS];
 static Boolean  cursor_moved = False;
 static int      saved_cursor_addr;
-static void     own_sels();
-int             n_owned = -1;
-Boolean         any_selected = False;
+static void own_sels(Time t);
+static int	n_owned = -1;
+static Boolean	any_selected = False;
 
 extern Widget  *screen;
 
@@ -103,7 +106,7 @@ extern Widget  *screen;
 		x = COLS - 1;		\
 	if (flipped)			\
 		x = (COLS - x) - 1;	\
-	y = Y_TO_ROW(event_y(event));	\
+	y = Y_TO_ROW(event_y(event) - *descent);	\
 	if (y <= 0)			\
 		y = 0;			\
 	if (y >= ROWS)			\
@@ -148,8 +151,7 @@ static int char_class[256] = {
 
 /* Parse a charClass string: [low-]high:value[,...] */
 void
-reclass(s)
-char *s;
+reclass(char *s)
 {
 	int n;
 	int low, high, value;
@@ -199,13 +201,11 @@ char *s;
 	}
 
     fail:
-	xs_warning("Error in %s string", ResCharClass);
+	popup_an_error("Error in %s string", ResCharClass);
 }
 
 static void
-select_word(baddr, t)
-int baddr;
-Time t;
+select_word(int baddr, Time t)
 {
 	unsigned char fa = *get_field_attribute(baddr);
 	unsigned char ch;
@@ -250,9 +250,7 @@ Time t;
 }
 
 static void
-select_line(baddr, t)
-int baddr;
-Time t;
+select_line(int baddr, Time t)
 {
 	f_start = baddr - (baddr % COLS);
 	f_end = f_start + COLS - 1;
@@ -266,20 +264,21 @@ Time t;
  * Start a new selection.
  * Usually bound to <Btn1Down>.
  */
-/*ARGSUSED*/
 void
-select_start_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+select_start_action(Widget w, XEvent *event, String *params,
+    Cardinal *num_params)
 {
 	int x, y;
 	register int baddr;
 
+	action_debug(select_start_action, event, params, num_params);
+	if (event == NULL) {
+		popup_an_error("%s can only be used as a keymap action",
+		    action_name(select_start_action));
+		return;
+	}
 	if (w != *screen)
 		return;
-	action_debug(select_start_action, event, params, num_params);
 	BOUNDED_XY(event, x, y);
 	baddr = ROWCOL_TO_BA(y, x);
 	f_start = f_end = v_start = v_end = baddr;
@@ -299,20 +298,21 @@ Cardinal *num_params;
  * Alternate form of select_start, which combines cursor motion with selection.
  * Usually bound to <Btn1Down> in a user-specified keymap.
  */
-/*ARGSUSED*/
 void
-move_select_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+move_select_action(Widget w, XEvent *event, String *params,
+    Cardinal *num_params)
 {
 	int x, y;
 	register int baddr;
 
+	action_debug(move_select_action, event, params, num_params);
+	if (event == NULL) {
+		popup_an_error("%s can only be used as a keymap action",
+		    action_name(move_select_action));
+		return;
+	}
 	if (w != *screen)
 		return;
-	action_debug(move_select_action, event, params, num_params);
 	BOUNDED_XY(event, x, y);
 	baddr = ROWCOL_TO_BA(y, x);
 
@@ -341,34 +341,70 @@ Cardinal *num_params;
  * Begin extending the current selection.
  * Usually bound to <Btn3Down>.
  */
-/*ARGSUSED*/
 void
-start_extend_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+start_extend_action(Widget w, XEvent *event, String *params,
+    Cardinal *num_params)
 {
 	int x, y;
 	int baddr;
+	Boolean continuous = (!ever_3270 && !toggled(RECTANGLE_SELECT));
 
+	action_debug(start_extend_action, event, params, num_params);
+	if (event == NULL) {
+		popup_an_error("%s can only be used as a keymap action",
+		    action_name(start_extend_action));
+		return;
+	}
 	if (w != *screen)
 		return;
-	action_debug(start_extend_action, event, params, num_params);
 
 	down1_time = 0L;
 
 	BOUNDED_XY(event, x, y);
 	baddr = ROWCOL_TO_BA(y, x);
 
-	if (baddr < f_start)
-		v_start = baddr;		/* extend above */
-	else if (baddr > f_end)
-		v_end = baddr;			/* extend below */
-	else if (baddr - f_start > f_end - baddr)
-		v_end = baddr;			/* shrink end */
-	else
-		v_start = baddr;		/* shrink start */
+	if (continuous) {
+		/* Think linearly. */
+		if (baddr < f_start)
+			v_start = baddr;
+		else if (baddr > f_end)
+			v_end = baddr;
+		else if (baddr - f_start > f_end - baddr)
+			v_end = baddr;
+		else
+			v_start = baddr;
+	} else {
+		/* Think rectangularly. */
+		int nrow = baddr / COLS;
+		int ncol = baddr % COLS;
+		int vrow_ul = v_start / COLS;
+		int vrow_lr = v_end / COLS;
+		int vcol_ul = Min(v_start % COLS, v_end % COLS);
+		int vcol_lr = Max(v_start % COLS, v_end % COLS);
+
+		/* Set up the row. */
+		if (nrow <= vrow_ul)
+			vrow_ul = nrow;
+		else if (nrow >= vrow_lr)
+			vrow_lr = nrow;
+		else if (nrow - vrow_ul > vrow_lr - nrow)
+			vrow_lr = nrow;
+		else
+			vrow_ul = nrow;
+
+		/* Set up the column. */
+		if (ncol <= vcol_ul)
+			vcol_ul = ncol;
+		else if (ncol >= vcol_lr)
+			vcol_lr = ncol;
+		else if (ncol - vcol_ul > vcol_lr - ncol)
+			vcol_lr = ncol;
+		else
+			vcol_ul = ncol;
+
+		v_start = (vrow_ul * COLS) + vcol_ul;
+		v_end = (vrow_lr * COLS) + vcol_lr;
+	}
 
 	grab_sel(v_start, v_end, True, event_time(event));
 	saw_motion = 1;
@@ -379,25 +415,26 @@ Cardinal *num_params;
  * Continuously extend the current selection.
  * Usually bound to <Btn1Motion> and <Btn3Motion>.
  */
-/*ARGSUSED*/
 void
-select_extend_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+select_extend_action(Widget w, XEvent *event, String *params,
+    Cardinal *num_params)
 {
 	int x, y;
 	int baddr;
 
+	action_debug(select_extend_action, event, params, num_params);
+	if (event == NULL) {
+		popup_an_error("%s can only be used as a keymap action",
+		    action_name(select_extend_action));
+		return;
+	}
 	if (w != *screen)
 		return;
-	action_debug(select_extend_action, event, params, num_params);
 
 	/* Ignore initial drag events if are too near. */
 	if (down1_time != 0L &&
-	    abs((int) event_x(event) - (int) down1_x) < CHAR_WIDTH &&
-	    abs((int) event_y(event) - (int) down1_y) < CHAR_HEIGHT)
+	    abs((int) event_x(event) - (int) down1_x) < *char_width &&
+	    abs((int) event_y(event) - (int) down1_y) < *char_height)
 		return;
 	else
 		down1_time = 0L;
@@ -445,18 +482,21 @@ Cardinal *num_params;
  * End the selection.
  * Usually bound to <BtnUp>.
  */
-/*ARGSUSED*/
 void
-select_end_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+select_end_action(Widget w unused, XEvent *event, String *params,
+    Cardinal *num_params)
 {
-	int i;
+	Cardinal i;
 	int x, y;
 
 	action_debug(select_end_action, event, params, num_params);
+	if (event == NULL) {
+		popup_an_error("%s can only be used as a keymap action",
+		    action_name(select_end_action));
+		return;
+	}
+	if (w != *screen)
+		return;
 
 	if (n_owned == -1) {
 		for (i = 0; i < NS; i++)
@@ -509,15 +549,11 @@ Cardinal *num_params;
  * Set the selection.
  * Usually bound to the Copy key.
  */
-/*ARGSUSED*/
 void
-set_select_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+set_select_action(Widget w unused, XEvent *event, String *params,
+    Cardinal *num_params)
 {
-	int i;
+	Cardinal i;
 
 	action_debug(set_select_action, event, params, num_params);
 
@@ -539,57 +575,17 @@ Cardinal *num_params;
 }
 
 /*
- * MoveCursor action.  Not a selection operation, but it uses the same macros
- * to map the mouse position onto a cursor position.
- * Usually bound to Shift<Btn1Down>.
+ * Translate the mouse position to a buffer address.
  */
-/*ARGSUSED*/
-void
-MoveCursor_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+int
+mouse_baddr(Widget w, XEvent *event)
 {
 	int x, y;
-	register int baddr;
-	int row, col;
 
-	action_debug(MoveCursor_action, event, params, num_params);
-
-	if (kybdlock) {
-		if (*num_params == 2)
-			enq_ta(MoveCursor_action, params[0], params[1]);
-		return;
-	}
-
-	switch (*num_params) {
-	    case 0:		/* mouse click, presumably */
-		if (w != *screen)
-			return;
-		BOUNDED_XY(event, x, y);
-		baddr = ROWCOL_TO_BA(y, x);
-		cursor_move(baddr);
-		break;
-	    case 2:		/* probably a macro call */
-		row = atoi(params[0]);
-		col = atoi(params[1]);
-		if (!IN_3270) {
-			row--;
-			col--;
-		}
-		if (row < 0)
-			row = 0;
-		if (col < 0)
-			col = 0;
-		baddr = ((row * COLS) + col) % (ROWS * COLS);
-		cursor_move(baddr);
-		break;
-	    default:		/* couln't say */
-		popup_an_error("%s: illegal argument count",
-		    action_name(MoveCursor_action));
-		break;
-	}
+	if (w != *screen)
+		return 0;
+	BOUNDED_XY(event, x, y);
+	return ROWCOL_TO_BA(y, x);
 }
 
 /*
@@ -600,13 +596,8 @@ Cardinal *num_params;
 #define ULS	sizeof(unsigned long)
 #define ULBS	(ULS * 8)
 
-/*ARGSUSED*/
 void
-Cut_action(w, event, params, num_params)
-Widget w;
-XEvent *event;
-String *params;
-Cardinal *num_params;
+Cut_action(Widget w unused, XEvent *event, String *params, Cardinal *num_params)
 {
 	register int baddr;
 	unsigned char fa = *get_field_attribute(0);
@@ -636,7 +627,114 @@ Cardinal *num_params;
 		if (target[baddr/ULBS] & (1 << (baddr%ULBS)))
 			ctlr_add(baddr, repl, 0);
 
-	XtFree((XtPointer)target);
+	Free(target);
+}
+
+/*
+ * KybdSelect action.  Extends the selection area in the indicated direction.
+ */
+void
+KybdSelect_action(Widget w unused, XEvent *event, String *params,
+    Cardinal *num_params)
+{
+	enum { UP, DOWN, LEFT, RIGHT } direction;
+	int x_start, x_end;
+	int i;
+
+	action_debug(KybdSelect_action, event, params, num_params);
+	if (event == NULL) {
+		popup_an_error("%s can only be used as a keymap action",
+		    action_name(select_start_action));
+		return;
+	}
+	if (w != *screen)
+		return;
+
+	if (*num_params < 1) {
+		popup_an_error("%s requires one argument",
+		    action_name(KybdSelect_action));
+		return;
+	}
+	if (!strcasecmp(params[0], "Up")) {
+		direction = UP;
+	} else if (!strcasecmp(params[0], "Down")) {
+		direction = DOWN;
+	} else if (!strcasecmp(params[0], "Left")) {
+		direction = LEFT;
+	} else if (!strcasecmp(params[0], "Right")) {
+		direction = RIGHT;
+	} else {
+		popup_an_error("%s first argument must be Up, Down, Left, or "
+		    "Right", action_name(KybdSelect_action));
+		return;
+	}
+
+	if (!any_selected)
+		x_start = x_end = cursor_addr;
+	else {
+		if (f_start < f_end) {
+			x_start = f_start;
+			x_end = f_end;
+		} else {
+			x_start = f_end;
+			x_end = f_start;
+		}
+	}
+
+	switch (direction) {
+	    case UP:
+		if (!(x_start / COLS))
+			return;
+		x_start -= COLS;
+		break;
+	    case DOWN:
+		if ((x_end / COLS) == ROWS - 1)
+			return;
+		x_end += COLS;
+		break;
+	    case LEFT:
+		if (!(x_start % COLS))
+			return;
+		x_start--;
+		break;
+	    case RIGHT:
+		if ((x_end % COLS) == COLS - 1)
+			return;
+		x_end++;
+		break;
+	}
+
+	/* Figure out the atoms they want. */
+	if (n_owned == -1) {
+		for (i = 0; i < NS; i++)
+			own_sel[i].atom = None;
+		n_owned = 0;
+	}
+	for (i = 1; i < NS; i++)
+		if (i < *num_params)
+			want_sel[i] = XInternAtom(display, params[i], False);
+		else
+			want_sel[i] = None;
+	if (*num_params == 1)
+		want_sel[0] = XA_PRIMARY;
+
+	/* Grab the selection. */
+	f_start = v_start = x_start;
+	f_end = v_end = x_end;
+	grab_sel(f_start, f_end, True, event_time(event));
+}
+
+/*
+ * unselect action.  Removes a selection.
+ */
+void
+Unselect_action(Widget w unused, XEvent *event, String *params,
+    Cardinal *num_params)
+{
+	action_debug(Unselect_action, event, params, num_params);
+
+	/* It's just cosmetic. */
+	unselect(0, ROWS*COLS);
 }
 
 
@@ -652,7 +750,7 @@ static int      sb_size = 0;
 static Time     sel_time;
 
 static void
-init_select_buf()
+init_select_buf(void)
 {
 	if (select_buf == CN)
 		select_buf = XtMalloc(sb_size = SB_CHUNK);
@@ -660,8 +758,7 @@ init_select_buf()
 }
 
 static void
-store_sel(c)
-char c;
+store_sel(char c)
 {
 	if (sb_ptr - select_buf >= sb_size) {
 		sb_size += SB_CHUNK;
@@ -672,12 +769,8 @@ char c;
 }
 
 static Boolean
-convert_sel(w, selection, target, type, value, length, format)
-Widget w;
-Atom *selection, *target, *type;
-XtPointer *value;
-unsigned long *length;
-int *format;
+convert_sel(Widget w, Atom *selection, Atom *target, Atom *type,
+    XtPointer *value, unsigned long *length, int *format)
 {
 	int i;
 
@@ -703,8 +796,7 @@ int *format;
 		*targetP++ = XA_COMPOUND_TEXT(display);
 		*targetP++ = XA_LENGTH(display);
 		*targetP++ = XA_LIST_LENGTH(display);
-		(void) MEMORY_MOVE((char *) targetP,
-				   (char *) std_targets,
+		(void) memmove(targetP,  std_targets,
 				   (int) (sizeof(Atom) * std_length));
 		XtFree((char *) std_targets);
 		*type = XA_ATOM;
@@ -721,7 +813,7 @@ int *format;
 			*type = XA_STRING;
 		*length = strlen(own_sel[i].buffer);
 		*value = XtMalloc(*length);
-		(void) MEMORY_MOVE(*value, own_sel[i].buffer, (int) *length);
+		(void) memmove(*value, own_sel[i].buffer, (int) *length);
 		*format = 8;
 		return True;
 	}
@@ -731,9 +823,9 @@ int *format;
 			*(long *)*value = 1;
 		else {
 			long temp = 1;
-			(void) MEMORY_MOVE((char *) *value,
-			                   ((char*) &temp) + sizeof(long) - 4,
-					   4);
+			(void) memmove(*value,
+					((char*) &temp) + sizeof(long) - 4,
+					4);
 		}
 		*type = XA_INTEGER;
 		*length = 1;
@@ -746,7 +838,7 @@ int *format;
 			*(long*)*value = strlen(own_sel[i].buffer);
 		else {
 			long temp = strlen(own_sel[i].buffer);
-			(void) MEMORY_MOVE((char *) *value,
+			(void) memmove(*value,
 			                   ((char *) &temp) + sizeof(long) - 4,
 					   4);
 		}
@@ -769,11 +861,8 @@ int *format;
 	return False;
 }
 
-/*ARGSUSED*/
 static void
-lose_sel(w, selection)
-Widget w;
-Atom *selection;
+lose_sel(Widget w unused, Atom *selection)
 {
 	int i;
 
@@ -799,15 +888,20 @@ Atom *selection;
 static Boolean osc_valid = False;
 
 static void
-osc_start()
+osc_start(void)
 {
 	osc_valid = False;
 }
 
+/*
+ * Return a 'selection' version of a given character on the screen.
+ * Returns a printable ASCII character, or 0 if the character is a NULL and
+ * shouldn't be included in the selection.
+ * Also returns in 'ge' whether or not an ESC (0x1b) should be included
+ * in front of the character.
+ */
 static unsigned char
-onscreen_char(baddr, ge)
-int baddr;
-Boolean *ge;
+onscreen_char(int baddr, Boolean *ge)
 {
 	static int osc_baddr;
 	static unsigned char fa;
@@ -833,24 +927,43 @@ Boolean *ge;
 		osc_valid = True;
 	}
 
+	/* Assume it isn't a graphic escape. */
 	*ge = False;
+
+	/* If it isn't visible, then make it a blank. */
 	if (FA_IS_ZERO(fa))
 		return ' ';
-	else {
-		switch (ea_buf[baddr].cs) {
-		    case 0:
+
+	switch (ea_buf[baddr].cs) {
+	    case 0:
+	    default:
+		switch (screen_buf[baddr]) {
+		    case CG_null:
+			return 0;
+		    case CG_fm:
+			*ge = True;
+			return ';';
+		    case CG_dup:
+			*ge = True;
+			return '*';
 		    default:
 			return cg2asc[screen_buf[baddr]];
-		    case CS_GE:
-			if (screen_buf[baddr] == CG_Yacute)
-				return '[';
-			else if (screen_buf[baddr] == CG_diaeresis)
-				return ']';
+		}
+	    case CS_GE:
+		switch (screen_buf[baddr]) {
+		    case CG_null:
+			return 0;
+		    case CG_Yacute:
+			return '[';
+		    case CG_diaeresis:
+			return ']';
+		    default:
 			*ge = True;
 			return cg2asc[screen_buf[baddr]];
-		    case 2:
-			return screen_buf[baddr] + 0x5f;
 		}
+	    case 2:
+		/* vt100 line-drawing character */
+		return screen_buf[baddr] + 0x5f;
 	}
 }
 
@@ -858,8 +971,7 @@ Boolean *ge;
  * Attempt to own the selections in want_sel[].
  */
 static void
-own_sels(t)
-Time t;
+own_sels(Time t)
 {
 	register int i, j;
 
@@ -894,10 +1006,9 @@ Time t;
 				n_owned++;
 				own_sel[j].atom = want_sel[i];
 			}
-			if (own_sel[j].buffer != CN)
-				XtFree(own_sel[j].buffer);
-			own_sel[j].buffer = XtMalloc(strlen(select_buf) + 1);
-			MEMORY_MOVE(own_sel[j].buffer, select_buf,
+			Replace(own_sel[j].buffer,
+			    XtMalloc(strlen(select_buf) + 1));
+			(void) memmove(own_sel[j].buffer, select_buf,
 			    strlen(select_buf) + 1);
 		} else {
 			XtWarning("Could not get selection");
@@ -919,15 +1030,13 @@ Time t;
  * own the selections in want_sel[].
  */
 static void
-grab_sel(start, end, really, t)
-int start, end;
-Boolean really;
-Time t;
+grab_sel(int start, int end, Boolean really, Time t)
 {
 	register int i;
 	int start_row, end_row;
 	unsigned char osc;
 	Boolean ge;
+	int nulls = 0;
 
 	unselect(0, ROWS*COLS);
 
@@ -949,27 +1058,39 @@ Time t;
 		for (i = start; i <= end; i++) {
 			SET_SELECT(i);
 			if (really) {
-				if (i != start && !(i % COLS))
+				if (i != start && !(i % COLS)) {
+					nulls = 0;
 					store_sel('\n');
+				}
 				osc = onscreen_char(i, &ge);
-				if (ge)
-					store_sel('\033');
-				store_sel((char)osc);
+				if (osc) {
+					while (nulls) {
+						store_sel(' ');
+						nulls--;
+					}
+					if (ge)
+						store_sel('\033');
+					store_sel((char)osc);
+				} else
+					nulls++;
 			}
 		}
-		/* Check for newline extension */
+		/* Check for newline extension on the last line. */
 		if ((end % COLS) != (COLS - 1)) {
 			Boolean all_blank = True;
 
-			for (i = end; i < end + (COLS - (end % COLS)); i++)
-				if (onscreen_char(i, &ge) != ' ') {
+			for (i = end; i < end + (COLS - (end % COLS)); i++) {
+				if (onscreen_char(i, &ge)) {
 					all_blank = False;
 					break;
 				}
+			}
 			if (all_blank) {
-				for (i = end; i < end + (COLS - (end % COLS)); i++)
+				for (i = end; i < end + (COLS - (end % COLS)); i++) {
 					SET_SELECT(i);
-				store_sel('\n');
+				}
+				if (really)
+					store_sel('\n');
 			}
 		}
 	} else {
@@ -979,9 +1100,16 @@ Time t;
 				SET_SELECT(i);
 				if (really) {
 					osc = onscreen_char(i, &ge);
-					if (ge)
-						store_sel('\033');
-					store_sel((char)osc);
+					if (osc) {
+						while (nulls) {
+							store_sel(' ');
+							nulls--;
+						}
+						if (ge)
+							store_sel('\033');
+						store_sel((char)osc);
+					} else
+						nulls++;
 				}
 			}
 			if (really && (end % COLS == COLS - 1))
@@ -1003,39 +1131,29 @@ Time t;
 					SET_SELECT(row*COLS + col);
 					if (really) {
 						osc = onscreen_char(row*COLS + col, &ge);
-						if (ge)
-							store_sel('\033');
-						store_sel((char)osc);
+						if (osc) {
+							while (nulls) {
+								store_sel(' ');
+								nulls--;
+							}
+							if (ge)
+								store_sel('\033');
+							store_sel((char)osc);
+						} else
+							nulls++;
 					}
 				}
+				nulls = 0;
 				if (really)
 					store_sel('\n');
 			}
 		}
 	}
 
-	/* Trim trailing blanks on each line */
-	if (really) {
-		int k, l;
-		char c;
-		int nb = 0;
-		int iy = 0;
-
+	/* Terminate the result. */
+	if (really)
 		store_sel('\0');
-		for (k = 0; (c = select_buf[k]); k++) {
-			if (c == ' ')
-				nb++;
-			else {
-				if (c != '\n') {
-					for (l = 0; l < nb; l++)
-						select_buf[iy++] = ' ';
-				}
-				select_buf[iy++] = c;
-				nb = 0;
-			}
-		}
-		select_buf[iy] = '\0';
-	}
+
 	any_selected = True;
 	ctlr_changed(0, ROWS*COLS);
 	if (really)
@@ -1046,8 +1164,7 @@ Time t;
  * Check if any character in a given region is selected.
  */
 Boolean
-area_is_selected(baddr, len)
-int baddr, len;
+area_is_selected(int baddr, int len)
 {
 	register int i;
 
@@ -1060,11 +1177,8 @@ int baddr, len;
 /*
  * Unhighlight the region of selected text -- but don't give up the selection.
  */
-/*ARGSUSED*/
 void
-unselect(baddr, len)
-int baddr;
-int len;
+unselect(int baddr unused, int len unused)
 {
 	if (any_selected) {
 		(void) memset((char *) selected, 0, (ROWS*COLS + 7) / 8);
@@ -1072,3 +1186,64 @@ int len;
 		any_selected = False;
 	}
 }
+
+/* Selection insertion. */
+#define NP	5
+static Atom	paste_atom[NP];
+static int	n_pasting = 0;
+static int	pix = 0;
+static Time	paste_time;
+
+static void
+paste_callback(Widget w, XtPointer client_data unused, Atom *selection unused,
+    Atom *type unused, XtPointer value, unsigned long *length,
+    int *format unused)
+{
+	char *s;
+	unsigned long len;
+
+	if ((value == NULL) || (*length == 0)) {
+		XtFree(value);
+
+		/* Try the next one. */
+		if (n_pasting > pix)
+			XtGetSelectionValue(w, paste_atom[pix++], XA_STRING,
+			    paste_callback, NULL, paste_time);
+		return;
+	}
+
+	s = (char *)value;
+	len = *length;
+	(void) emulate_input(s, (int) len, True);
+	n_pasting = 0;
+
+	XtFree(value);
+}
+
+void
+insert_selection_action(Widget w, XEvent *event, String *params,
+    Cardinal *num_params)
+{
+	Cardinal i;
+	Atom	a;
+	XButtonEvent *be = (XButtonEvent *)event;
+
+	action_debug(insert_selection_action, event, params, num_params);
+	n_pasting = 0;
+	for (i = 0; i < *num_params; i++) {
+		a = XInternAtom(display, params[i], True);
+		if (a == None) {
+			popup_an_error("%s: No atom for selection",
+			    action_name(insert_selection_action));
+			continue;
+		}
+		if (n_pasting < NP)
+			paste_atom[n_pasting++] = a;
+	}
+	pix = 0;
+	if (n_pasting > pix) {
+		paste_time = be->time;
+		XtGetSelectionValue(w, paste_atom[pix++], XA_STRING,
+		    paste_callback, NULL, paste_time);
+	}
+}
