@@ -1,5 +1,5 @@
 /*
- * Copyright 1993, 1994, 1995, 199 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
@@ -21,13 +21,13 @@
 #include "cg.h"
 
 #include "kybdc.h"
+#include "hostc.h"
+#include "screenc.h"
 #include "statusc.h"
 #include "tablesc.h"
 #include "utilc.h"
 
 extern Window  *screen_window;
-extern GC       screen_gc();
-extern GC       screen_invgc();
 
 static XChar2b *status_2b;
 static unsigned char *status_1b;
@@ -130,19 +130,21 @@ static Position		status_y;
 
 /* Status line contents (high-level) */
 
-static void     do_disconnected();
-static void     do_connecting();
-static void     do_nonspecific();
-static void     do_inhibit();
-static void     do_blank();
-static void     do_twait();
-static void     do_syswait();
-static void     do_protected();
-static void     do_numeric();
-static void     do_overflow();
-static void     do_scrolled();
+static void do_disconnected(void);
+static void do_resolving(void);
+static void do_connecting(void);
+static void do_nonspecific(void);
+static void do_inhibit(void);
+static void do_blank(void);
+static void do_twait(void);
+static void do_syswait(void);
+static void do_protected(void);
+static void do_numeric(void);
+static void do_overflow(void);
+static void do_scrolled(void);
+static void do_minus(void);
 
-static Boolean  oia_undera = False;
+static Boolean  oia_undera = True;
 static Boolean  oia_boxsolid = False;
 static int      oia_shift = 0;
 static Boolean  oia_typeahead = False;
@@ -151,6 +153,7 @@ static unsigned char oia_compose_char = 0;
 static enum keytype oia_compose_keytype = KT_STD;
 static enum msg {
 	DISCONNECTED,		/* X Not Connected */
+	XRESOLVING,		/* X Resolving */
 	CONNECTING,		/* X Connecting */
 	NONSPECIFIC,		/* X */
 	INHIBIT,		/* X Inhibit */
@@ -160,12 +163,14 @@ static enum msg {
 	PROTECTED,		/* X Protected */
 	NUMERIC,		/* X Numeric */
 	OVERFLOW,		/* X Overflow */
-	SCROLLED		/* X Scrolled */
+	SCROLLED,		/* X Scrolled */
+	MINUS			/* X -f */
 }               oia_msg = DISCONNECTED, saved_msg;
 static Boolean  msg_is_saved = False;
 static int      n_scrolled = 0;
-static void     (*msg_proc[]) () = {
+static void     (*msg_proc[])(void) = {
 	do_disconnected,
+	do_resolving,
 	do_connecting,
 	do_nonspecific,
 	do_inhibit,
@@ -176,6 +181,7 @@ static void     (*msg_proc[]) () = {
 	do_numeric,
 	do_overflow,
 	do_scrolled,
+	do_minus
 };
 static int      msg_color[] = {
 	FA_INT_HIGH_SEL,
@@ -189,6 +195,7 @@ static int      msg_color[] = {
 	FA_INT_NORM_SEL,
 	FA_INT_NORM_SEL,
 	FA_INT_NORM_SEL,
+	FA_INT_NORM_SEL
 };
 static int      msg_color3279[] = {
 	COLOR_WHITE,
@@ -202,6 +209,7 @@ static int      msg_color3279[] = {
 	COLOR_RED,
 	COLOR_RED,
 	COLOR_WHITE,
+	COLOR_RED
 };
 static Boolean  oia_insert = False;
 static Boolean  oia_reverse = False;
@@ -216,6 +224,12 @@ static unsigned char disc_pfx[] = {
 static unsigned char *disc_msg;
 static int      disc_len = sizeof(disc_pfx);
 
+static unsigned char rslv_pfx[] = {
+	CG_lock, CG_space, CG_commhi, CG_commjag, CG_commlo, CG_space
+};
+static unsigned char *rslv_msg;
+static int      rslv_len = sizeof(rslv_pfx);
+
 static unsigned char cnct_pfx[] = {
 	CG_lock, CG_space, CG_commhi, CG_commjag, CG_commlo, CG_space
 };
@@ -223,6 +237,7 @@ static unsigned char *cnct_msg;
 static int      cnct_len = sizeof(cnct_pfx);
 
 static unsigned char *a_not_connected;
+static unsigned char *a_resolving;
 static unsigned char *a_connecting;
 static unsigned char *a_inhibit;
 static unsigned char *a_twait;
@@ -231,77 +246,85 @@ static unsigned char *a_protected;
 static unsigned char *a_numeric;
 static unsigned char *a_overflow;
 static unsigned char *a_scrolled;
+static unsigned char *a_minus;
 
-static unsigned char *make_amsg();
-static unsigned char *make_emsg();
+static unsigned char *make_amsg(const char *key);
+static unsigned char *make_emsg(unsigned char prefix[], const char *key,
+	int *len);
 
-static void     status_render();
-static void     do_ctlr();
-static void     do_msg();
-static void     paint_msg();
-static void     do_insert();
-static void     do_reverse();
-static void     do_kmap();
-static void	do_script();
-static void     do_shift();
-static void     do_typeahead();
-static void     do_compose();
-static void     do_timing();
-static void     do_cursor();
+static void status_render(int region);
+static void do_ctlr(void);
+static void do_msg(enum msg t);
+static void paint_msg(enum msg t);
+static void do_insert(Boolean on);
+static void do_reverse(Boolean on);
+static void do_kmap(Boolean on);
+static void do_script(Boolean on);
+static void do_shift(int state);
+static void do_typeahead(int state);
+static void do_compose(Boolean on, unsigned char c, enum keytype keytype);
+static void do_timing(char *buf);
+static void do_cursor(char *buf);
+
+static void status_connect(Boolean connected);
+static void status_3270_mode(Boolean connected);
+static void status_resolving(Boolean ignored);
+static void status_half_connect(Boolean ignored);
 
 
-/* Initialize, or reinitialize the status line */
+/* Initialize the status line */
 void
-status_init(keep_contents, model_changed, font_changed)
-Boolean keep_contents;
-Boolean model_changed;
-Boolean font_changed;
+status_init(void)
 {
-	int	i;
-	static Boolean	ever = False;
+	a_not_connected = make_amsg("statusNotConnected");
+	disc_msg = make_emsg(disc_pfx, "statusNotConnected",
+	    &disc_len);
+	a_resolving = make_amsg("statusResolving");
+	rslv_msg = make_emsg(rslv_pfx, "statusResolving", &rslv_len);
+	a_connecting = make_amsg("statusConnecting");
+	cnct_msg = make_emsg(cnct_pfx, "statusConnecting", &cnct_len);
+	a_inhibit = make_amsg("statusInhibit");
+	a_twait = make_amsg("statusTwait");
+	a_syswait = make_amsg("statusSyswait");
+	a_protected = make_amsg("statusProtected");
+	a_numeric = make_amsg("statusNumeric");
+	a_overflow = make_amsg("statusOverflow");
+	a_scrolled = make_amsg("statusScrolled");
+	a_minus = make_amsg("statusMinus");
 
-	if (!ever) {
-		a_not_connected = make_amsg("statusNotConnected");
-		disc_msg = make_emsg(disc_pfx, "statusNotConnected",
-		    &disc_len);
-		a_connecting = make_amsg("statusConnecting");
-		cnct_msg = make_emsg(cnct_pfx, "statusConnecting", &cnct_len);
-		a_inhibit = make_amsg("statusInhibit");
-		a_twait = make_amsg("statusTwait");
-		a_syswait = make_amsg("statusSyswait");
-		a_protected = make_amsg("statusProtected");
-		a_numeric = make_amsg("statusNumeric");
-		a_overflow = make_amsg("statusOverflow");
-		a_scrolled = make_amsg("statusScrolled");
-		ever = True;
-	}
-	if (font_changed)
+	register_schange(ST_RESOLVING, status_resolving);
+	register_schange(ST_HALF_CONNECT, status_half_connect);
+	register_schange(ST_CONNECT, status_connect);
+	register_schange(ST_3270_MODE, status_3270_mode);
+}
+
+/* Reinitialize the status line */
+void
+status_reinit(unsigned cmask)
+{
+	unsigned i;
+
+	if (cmask & FONT_CHANGE)
 		nullblank = *standard_font ? ' ' : CG_space;
-	if (font_changed || model_changed) {
+	if (cmask & (FONT_CHANGE | MODEL_CHANGE)) {
 		status_y = STATUS_Y;
-		if (!(*efontinfo)->descent)
+		if (!*descent)
 			++status_y;
 	}
-	if (model_changed) {
-		if (status_line)
-			XtFree((char *)status_line);
-		status_line = (struct status_line *) XtCalloc(sizeof(struct status_line), SSZ);
-		if (status_2b != (XChar2b *)NULL)
-			XtFree((char *)status_2b);
-		status_2b = (XChar2b *)XtCalloc(sizeof(XChar2b), maxCOLS);
-		if (status_1b != (unsigned char *)NULL)
-			XtFree((char *)status_1b);
-		status_1b = (unsigned char *)XtCalloc(sizeof(unsigned char),
-		    maxCOLS);
-		if (display_2b != (XChar2b *)NULL)
-			XtFree((XtPointer)display_2b);
-		display_2b = (XChar2b *)XtCalloc(sizeof(XChar2b), maxCOLS);
+	if (cmask & MODEL_CHANGE) {
+		Replace(status_line,
+		    (struct status_line *)XtCalloc(sizeof(struct status_line),
+						   SSZ));
+		Replace(status_2b,
+		    (XChar2b *)XtCalloc(sizeof(XChar2b), maxCOLS));
+		Replace(status_1b,
+		    (unsigned char *)XtCalloc(sizeof(unsigned char), maxCOLS));
+		Replace(display_2b,
+		    (XChar2b *)XtCalloc(sizeof(XChar2b), maxCOLS));
 		offsets[SSZ] = maxCOLS;
 		if (appres.mono)
 			colors[1] = FA_INT_NORM_NSEL;
 		for (i = 0; i < SSZ; i++) {
-			status_line[i].color = appres.m3279 ?
-			    colors3279[i] : colors[i];
 			status_line[i].start = offsets[i];
 			status_line[i].len = offsets[i+1] - offsets[i];
 			status_line[i].s2b = status_2b + offsets[i];
@@ -310,43 +333,46 @@ Boolean font_changed;
 		}
 	} else
 		(void) memset(display_2b, 0, maxCOLS * sizeof(XChar2b));
+	if (cmask & (COLOR_CHANGE | MODEL_CHANGE)) {
+		for (i = 0; i < SSZ; i++) {
+			status_line[i].color = appres.m3279 ?
+			    colors3279[i] : colors[i];
+		}
+	}
 
 	for (i = 0; i < SSZ; i++)
-		status_line[i].changed = keep_contents;
-	status_changed = keep_contents;
+		status_line[i].changed = True;
+	status_changed = True;
 
 	/*
-	 * If reinitializing, redo the whole line, in case we switched between
-	 * a 3270 font and a standard font.
+	 * Always redraw all the fields; it's easier than keeping track of
+	 * what may have changed and why.
 	 */
-	if (keep_contents && font_changed) {
-		do_ctlr();
-		paint_msg(oia_msg);
-		do_insert(oia_insert);
-		do_reverse(oia_reverse);
-		do_kmap(oia_kmap);
-		do_script(oia_script);
-		do_shift(oia_shift);
-		do_typeahead(oia_typeahead);
-		do_compose(oia_compose, oia_compose_char, oia_compose_keytype);
-		do_cursor(oia_cursor);
-		do_timing(oia_timing);
-	}
+	do_ctlr();
+	paint_msg(oia_msg);
+	do_insert(oia_insert);
+	do_reverse(oia_reverse);
+	do_kmap(oia_kmap);
+	do_script(oia_script);
+	do_shift(oia_shift);
+	do_typeahead(oia_typeahead);
+	do_compose(oia_compose, oia_compose_char, oia_compose_keytype);
+	do_cursor(oia_cursor);
+	do_timing(oia_timing);
 }
 
 /* Render the status line onto the screen */
 void
-status_disp()
+status_disp(void)
 {
-	int	i;
+	unsigned i;
 
 	if (!status_changed)
 		return;
 	for (i = 0; i < SSZ; i++)
 		if (status_line[i].changed) {
 			status_render(i);
-			(void) MEMORY_MOVE(status_line[i].d2b,
-				   (char *) status_line[i].s2b,
+			(void) memmove(status_line[i].d2b, status_line[i].s2b,
 				   status_line[i].len * sizeof(XChar2b));
 			status_line[i].changed = False;
 		}
@@ -355,9 +381,9 @@ status_disp()
 
 /* Mark the entire status line as changed */
 void
-status_touch()
+status_touch(void)
 {
-	int	i;
+	unsigned i;
 
 	for (i = 0; i < SSZ; i++) {
 		status_line[i].changed = True;
@@ -367,40 +393,26 @@ status_touch()
 	status_changed = True;
 }
 
-/* Initialize the controller status */
-void
-status_ctlr_init()
-{
-	oia_undera = True;
-	do_ctlr();
-}
-
 /* Keyboard lock status changed */
 void
-status_kybdlock()
+status_kybdlock(void)
 {
 	/* presently implemented as explicit calls */
 }
 
 /* Connected or disconnected */
-void
-status_connect()
+static void
+status_connect(Boolean connected)
 {
-	if (CONNECTED) {
-		oia_boxsolid = True;
+	if (connected) {
+		oia_boxsolid = IN_3270 && !IN_SSCP;
 		do_ctlr();
 		if (kybdlock & KL_AWAITING_FIRST)
 			do_msg(NONSPECIFIC);
 		else
 			do_msg(BLANK);
 		status_untiming();
-	} else if (HALF_CONNECTED) {
-		oia_boxsolid = False;
-		do_ctlr();
-		do_msg(CONNECTING);
-		status_untiming();
-		status_uncursor_pos();
-	} else {	/* not connected */
+	} else {
 		oia_boxsolid = False;
 		do_ctlr();
 		do_msg(DISCONNECTED);
@@ -408,9 +420,40 @@ status_connect()
 	}
 }
 
+/* Changed 3270 mode */
+static void
+status_3270_mode(Boolean connected)
+{
+	oia_boxsolid = IN_3270 && !IN_SSCP;
+	do_ctlr();
+	status_untiming();
+}
+
+/* Resolving */
+static void
+status_resolving(Boolean ignored unused)
+{
+	oia_boxsolid = False;
+	do_ctlr();
+	do_msg(RESOLVING);
+	status_untiming();
+	status_uncursor_pos();
+}
+
+/* Half connected */
+static void
+status_half_connect(Boolean ignored unused)
+{
+	oia_boxsolid = False;
+	do_ctlr();
+	do_msg(CONNECTING);
+	status_untiming();
+	status_uncursor_pos();
+}
+
 /* Lock the keyboard (twait) */
 void
-status_twait()
+status_twait(void)
 {
 	oia_undera = False;
 	do_ctlr();
@@ -419,7 +462,7 @@ status_twait()
 
 /* Done with controller confirmation */
 void
-status_ctlr_done()
+status_ctlr_done(void)
 {
 	oia_undera = True;
 	do_ctlr();
@@ -427,15 +470,14 @@ status_ctlr_done()
 
 /* Lock the keyboard (X SYSTEM) */
 void
-status_syswait()
+status_syswait(void)
 {
 	do_msg(SYSWAIT);
 }
 
 /* Lock the keyboard (operator error) */
 void
-status_oerr(error_type)
-int error_type;
+status_oerr(int error_type)
 {
 	switch (error_type) {
 	    case KL_OERR_PROTECTED:
@@ -452,8 +494,7 @@ int error_type;
 
 /* Lock the keyboard (X Scrolled) */
 void
-status_scrolled(n)
-int n;
+status_scrolled(int n)
 {
 	if (n != 0) {
 		if (!msg_is_saved) {
@@ -470,9 +511,16 @@ int n;
 	}
 }
 
+/* Lock the keyboard (X -f) */
+void
+status_minus(void)
+{
+	do_msg(MINUS);
+}
+
 /* Unlock the keyboard */
 void
-status_reset()
+status_reset(void)
 {
 	if (kybdlock & KL_ENTER_INHIBIT)
 		do_msg(INHIBIT);
@@ -484,58 +532,49 @@ status_reset()
 
 /* Toggle insert mode */
 void
-status_insert_mode(on)
-Boolean on;
+status_insert_mode(Boolean on)
 {
 	do_insert(oia_insert = on);
 }
 
 /* Toggle reverse mode */
 void
-status_reverse_mode(on)
-Boolean on;
+status_reverse_mode(Boolean on)
 {
 	do_reverse(oia_reverse = on);
 }
 
 /* Toggle kmap mode */
 void
-status_kmap(on)
-Boolean on;
+status_kmap(Boolean on)
 {
 	do_kmap(oia_kmap = on);
 }
 
 /* Toggle script mode */
 void
-status_script(on)
-Boolean on;
+status_script(Boolean on)
 {
 	do_script(oia_script = on);
 }
 
 /* Toggle shift mode */
 void
-status_shift_mode(state)
-int state;
+status_shift_mode(int state)
 {
 	do_shift(oia_shift = state);
 }
 
 /* Toggle typeahead */
 void
-status_typeahead(on)
-Boolean on;
+status_typeahead(Boolean on)
 {
 	do_typeahead(oia_typeahead = on);
 }
 
 /* Set compose character */
 void
-status_compose(on, c, keytype)
-Boolean on;
-unsigned char c;
-enum keytype keytype;
+status_compose(Boolean on, unsigned char c, enum keytype keytype)
 {
 	oia_compose = on;
 	oia_compose_char = c;
@@ -545,8 +584,7 @@ enum keytype keytype;
 
 /* Display timing */
 void
-status_timing(t0, t1)
-struct timeval *t0, *t1;
+status_timing(struct timeval *t0, struct timeval *t1)
 {
 	static char	no_time[] = ":??.?";
 	static char	buf[TCNT+1];
@@ -568,15 +606,14 @@ struct timeval *t0, *t1;
 
 /* Erase timing indication */
 void
-status_untiming()
+status_untiming(void)
 {
 	do_timing(oia_timing = (char *) 0);
 }
 
 /* Update cursor position */
 void
-status_cursor_pos(ca)
-int ca;
+status_cursor_pos(int ca)
 {
 	static char	buf[CCNT+1];
 
@@ -586,7 +623,7 @@ int ca;
 
 /* Erase cursor position */
 void
-status_uncursor_pos()
+status_uncursor_pos(void)
 {
 	do_cursor(oia_cursor = (char *) 0);
 }
@@ -596,12 +633,9 @@ status_uncursor_pos()
 
 /* Update the status line by displaying "symbol" at column "col".  */
 static void
-status_add(col, symbol, keytype)
-	int	col;
-	unsigned char symbol;
-	enum keytype keytype;
+status_add(int col, unsigned char symbol, enum keytype keytype)
 {
-	int	i;
+	unsigned i;
 	XChar2b n2b;
 
 	n2b.byte1 = (keytype == KT_STD) ? 0 : 1;
@@ -628,8 +662,7 @@ status_add(col, symbol, keytype)
  * redundantly draw over B or not?  Right now we don't.
  */
 static void
-status_render(region)
-	int	region;
+status_render(int region)
 {
 	int	i;
 	struct status_line *sl = &status_line[region];
@@ -637,11 +670,27 @@ status_render(region)
 	int	i0 = -1;
 
 	/* The status region may change colors; don't be so clever */
+	if (*funky_font)
+		XFillRectangle(display, *screen_window,
+		    screen_invgc(sl->color),
+		    COL_TO_X(sl->start), status_y - *ascent,
+		    *char_width * sl->len, *char_height);
 	if (region == WAIT_REGION) {
 		XDrawImageString(display, *screen_window, screen_gc(sl->color),
 		    COL_TO_X(sl->start), status_y, (char *) sl->s1b, sl->len);
 	} else {
 		for (i = 0; i < sl->len; i++) {
+			if (*funky_font) {
+				if (!sl->s1b[i])
+					continue;
+				XDrawImageString(display,
+				    *screen_window,
+				    screen_gc(sl->color),
+				    COL_TO_X(sl->start + i),
+				    status_y,
+				    (char *) sl->s1b + i, 1);
+				continue;
+			}
 			if (sl->s2b[i].byte1 == sl->d2b[i].byte1 &&
 			    sl->s2b[i].byte2 == sl->d2b[i].byte2) {
 				if (nd) {
@@ -677,19 +726,33 @@ status_render(region)
 				XDrawImageString(display, *screen_window,
 				    screen_gc(sl->color),
 				    COL_TO_X(sl->start + i0), status_y,
-				    (char *) sl->s1b + i0, nd);
+				    (char *)sl->s1b + i0, nd);
 		}
 	}
 
 	/* Leftmost region has unusual attributes */
 	if (*standard_font && region == CTLR_REGION) {
+		if (*funky_font) {
+			XFillRectangle(display, *screen_window,
+			    screen_invgc(sl->color),
+			    COL_TO_X(sl->start), status_y - *ascent,
+			    *char_width * 3, *char_height);
+			XFillRectangle(display, *screen_window,
+			    screen_gc(sl->color),
+			    COL_TO_X(sl->start + LBOX), status_y - *ascent,
+			    *char_width, *char_height);
+			XFillRectangle(display, *screen_window,
+			    screen_gc(sl->color),
+			    COL_TO_X(sl->start + RBOX), status_y - *ascent,
+			    *char_width, *char_height);
+		}
 		XDrawImageString(display, *screen_window,
 		    screen_invgc(sl->color),
 		    COL_TO_X(sl->start + LBOX), status_y,
 		    (char *) sl->s1b + LBOX, 1);
 		XDrawRectangle(display, *screen_window, screen_gc(sl->color),
 		    COL_TO_X(sl->start + CNCT),
-		    status_y - (*efontinfo)->ascent + *char_height - 1,
+		    status_y - *ascent + *char_height - 1,
 		    *char_width - 1, 0);
 		XDrawImageString(display, *screen_window,
 		    screen_invgc(sl->color),
@@ -700,9 +763,7 @@ status_render(region)
 
 /* Write into the message area of the status line */
 static void
-status_msg_set(msg, len)
-unsigned char *msg;
-int len;
+status_msg_set(unsigned const char *msg, int len)
 {
 	register int	i;
 
@@ -715,26 +776,35 @@ int len;
 
 /* Controller status */
 static void
-do_ctlr()
+do_ctlr(void)
 {
 	if (*standard_font) {
 		status_add(LBOX, '4', KT_STD);
 		if (oia_undera)
-			status_add(CNCT, 'A', KT_STD);
+			status_add(CNCT, (IN_E ? 'B' : 'A'), KT_STD);
 		else
 			status_add(CNCT, ' ', KT_STD);
-		if (oia_boxsolid)
+		if (IN_ANSI)
+			status_add(RBOX, 'N', KT_STD);
+		else if (oia_boxsolid)
 			status_add(RBOX, ' ', KT_STD);
+		else if (IN_SSCP)
+			status_add(RBOX, 'S', KT_STD);
 		else
 			status_add(RBOX, '?', KT_STD);
 	} else {
 		status_add(LBOX, CG_box4, KT_STD);
 		if (oia_undera)
-			status_add(CNCT, CG_underA, KT_STD);
+			status_add(CNCT, (IN_E ? CG_underB : CG_underA),
+				KT_STD);
 		else
 			status_add(CNCT, CG_null, KT_STD);
-		if (oia_boxsolid)
+		if (IN_ANSI)
+			status_add(RBOX, CG_N, KT_STD);
+		else if (oia_boxsolid)
 			status_add(RBOX, CG_boxsolid, KT_STD);
+		else if (IN_SSCP)
+			status_add(RBOX, CG_boxhuman, KT_STD);
 		else
 			status_add(RBOX, CG_boxquestion, KT_STD);
 	}
@@ -744,8 +814,7 @@ do_ctlr()
 
 /* Change the state of the message area, or if scrolled, the saved message */
 static void
-do_msg(t)
-enum msg t;
+do_msg(enum msg t)
 {
 	if (msg_is_saved) {
 		saved_msg = t;
@@ -756,8 +825,7 @@ enum msg t;
 
 /* Paint the message area. */
 static void
-paint_msg(t)
-enum msg t;
+paint_msg(enum msg t)
 {
 	oia_msg = t;
 	(*msg_proc[(int)t])();
@@ -767,13 +835,13 @@ enum msg t;
 }
 
 static void
-do_blank()
+do_blank(void)
 {
 	status_msg_set((unsigned char *) 0, 0);
 }
 
 static void
-do_disconnected()
+do_disconnected(void)
 {
 	if (*standard_font)
 		status_msg_set(a_not_connected,
@@ -783,7 +851,16 @@ do_disconnected()
 }
 
 static void
-do_connecting()
+do_resolving(void)
+{
+	if (*standard_font)
+		status_msg_set(a_resolving, strlen((char *)a_resolving));
+	else
+		status_msg_set(rslv_msg, rslv_len);
+}
+
+static void
+do_connecting(void)
 {
 	if (*standard_font)
 		status_msg_set(a_connecting, strlen((char *)a_connecting));
@@ -792,20 +869,20 @@ do_connecting()
 }
 
 static void
-do_nonspecific()
+do_nonspecific(void)
 {
 	static unsigned char nonspecific[] = {
 		CG_lock
 	};
 
 	if (*standard_font)
-		status_msg_set((unsigned char *)"X", 1);
+		status_msg_set((unsigned const char *)"X", 1);
 	else
 		status_msg_set(nonspecific, sizeof(nonspecific));
 }
 
 static void
-do_inhibit()
+do_inhibit(void)
 {
 	static unsigned char inhibit[] = {
 		CG_lock, CG_space, CG_I, CG_n, CG_h, CG_i, CG_b, CG_i, CG_t
@@ -818,7 +895,7 @@ do_inhibit()
 }
 
 static void
-do_twait()
+do_twait(void)
 {
 	static unsigned char twait[] = {
 		CG_lock, CG_space, CG_clockleft, CG_clockright
@@ -831,7 +908,7 @@ do_twait()
 }
 
 static void
-do_syswait()
+do_syswait(void)
 {
 	static unsigned char syswait[] = {
 		CG_lock, CG_space, CG_S, CG_Y, CG_S, CG_T, CG_E, CG_M
@@ -844,7 +921,7 @@ do_syswait()
 }
 
 static void
-do_protected()
+do_protected(void)
 {
 	static unsigned char protected[] = {
 		CG_lock, CG_space, CG_leftarrow, CG_human, CG_rightarrow
@@ -857,7 +934,7 @@ do_protected()
 }
 
 static void
-do_numeric()
+do_numeric(void)
 {
 	static unsigned char numeric[] = {
 		CG_lock, CG_space, CG_human, CG_N, CG_U, CG_M
@@ -870,7 +947,7 @@ do_numeric()
 }
 
 static void
-do_overflow()
+do_overflow(void)
 {
 	static unsigned char overflow[] = {
 		CG_lock, CG_space, CG_human, CG_greater
@@ -883,7 +960,7 @@ do_overflow()
 }
 
 static void
-do_scrolled()
+do_scrolled(void)
 {
 	static unsigned char scrolled[] = {
 		CG_lock, CG_space, CG_S, CG_c, CG_r, CG_o, CG_l, CG_l, CG_e,
@@ -913,39 +990,47 @@ do_scrolled()
 	}
 }
 
+static void
+do_minus(void)
+{
+	static unsigned char minus[] = {
+		CG_lock, CG_space, CG_minus, CG_f
+	};
+
+	if (*standard_font)
+		status_msg_set(a_minus, strlen((char *)a_minus));
+	else
+		status_msg_set(minus, sizeof(minus));
+}
+
 /* Insert, reverse, kmap, script, shift, compose */
 
 static void
-do_insert(on)
-Boolean on;
+do_insert(Boolean on)
 {
 	status_add(INSERT, on ? (*standard_font ? 'I' : CG_insert) : nullblank, KT_STD);
 }
 
 static void
-do_reverse(on)
-Boolean on;
+do_reverse(Boolean on)
 {
 	status_add(REVERSE, on ? (*standard_font ? 'R' : CG_R) : nullblank, KT_STD);
 }
 
 static void
-do_kmap(on)
-Boolean on;
+do_kmap(Boolean on)
 {
 	status_add(KMAP, on ? (*standard_font ? 'K' : CG_K) : nullblank, KT_STD);
 }
 
 static void
-do_script(on)
-Boolean on;
+do_script(Boolean on)
 {
 	status_add(SCRIPT, on ? (*standard_font ? 'S' : CG_S) : nullblank, KT_STD);
 }
 
 static void
-do_shift(state)
-int state;
+do_shift(int state)
 {
 	status_add(SHIFT-2, (state & MetaKeyDown) ?
 		(*standard_font ? 'M' : CG_M) : nullblank, KT_STD);
@@ -956,17 +1041,13 @@ int state;
 }
 
 static void
-do_typeahead(state)
-int state;
+do_typeahead(int state)
 {
 	status_add(TYPEAHD, state ? (*standard_font ? 'T' : CG_T) : nullblank, KT_STD);
 }
 
 static void
-do_compose(on, c, keytype)
-Boolean on;
-unsigned char c;
-enum keytype keytype;
+do_compose(Boolean on, unsigned char c, enum keytype keytype)
 {
 	if (on) {
 		status_add(COMPOSE,
@@ -981,8 +1062,7 @@ enum keytype keytype;
 
 /* Timing */
 static void
-do_timing(buf)
-char *buf;
+do_timing(char *buf)
 {
 	register int	i;
 
@@ -1003,8 +1083,7 @@ char *buf;
 
 /* Cursor position */
 static void
-do_cursor(buf)
-char *buf;
+do_cursor(char *buf)
 {
 	register int	i;
 
@@ -1019,22 +1098,18 @@ char *buf;
 /* Prepare status messages */
 
 static unsigned char *
-make_amsg(key)
-char *key;
+make_amsg(const char *key)
 {
 	return (unsigned char *)xs_buffer("X %s", get_message(key));
 }
 
 static unsigned char *
-make_emsg(prefix, key, len)
-unsigned char prefix[];
-char *key;
-int *len;
+make_emsg(unsigned char prefix[], const char *key, int *len)
 {
-	char *text = get_message(key);
-	unsigned char *buf = (unsigned char *) XtMalloc(*len + strlen(key));
+	const char *text = get_message(key);
+	unsigned char *buf = (unsigned char *)XtMalloc(*len + strlen(text));
 
-	(void) MEMORY_MOVE((char *)buf, (char *) prefix, *len);
+	(void) memmove(buf, prefix, *len);
 	while (*text)
 		buf[(*len)++] = asc2cg[(int)*text++];
 
